@@ -1,7 +1,25 @@
-/* EPaned - A slightly more advanced paned widget.
- * Copyright (C) 2000 Helix Code, Inc.
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-vpaned.c - A slightly more advanced paned widget.
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@helixcode.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  *
- * Author: Christopher James Lahey <clahey@helixcode.com>
  *
  * based on GtkPaned from Gtk+.  Gtk+ Copyright notice follows.
  */
@@ -33,6 +51,7 @@
  */
 
 #include "e-vpaned.h"
+#include <gal/util/e-util.h>
 
 static void     e_vpaned_class_init     (EVPanedClass   *klass);
 static void     e_vpaned_init           (EVPaned        *vpaned);
@@ -40,8 +59,6 @@ static void     e_vpaned_size_request   (GtkWidget      *widget,
 					 GtkRequisition *requisition);
 static void     e_vpaned_size_allocate  (GtkWidget      *widget,
 					 GtkAllocation  *allocation);
-static void     e_vpaned_draw           (GtkWidget      *widget,
-					 GdkRectangle   *area);
 static void     e_vpaned_xor_line       (EPaned         *paned);
 static gboolean e_vpaned_button_press   (GtkWidget      *widget,
 					 GdkEventButton *event);
@@ -51,31 +68,15 @@ static gboolean e_vpaned_motion         (GtkWidget      *widget,
 					 GdkEventMotion *event);
 static gboolean e_vpaned_handle_shown   (EPaned *paned);
 
-GtkType
-e_vpaned_get_type (void)
-{
-  static GtkType vpaned_type = 0;
-
-  if (!vpaned_type)
-    {
-      static const GtkTypeInfo vpaned_info =
-      {
-	"EVPaned",
-	sizeof (EVPaned),
-	sizeof (EVPanedClass),
-	(GtkClassInitFunc) e_vpaned_class_init,
-	(GtkObjectInitFunc) e_vpaned_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
-      };
-
-      vpaned_type = gtk_type_unique (E_TYPE_PANED, &vpaned_info);
-    }
-
-  return vpaned_type;
-}
+#define PARENT_TYPE E_TYPE_PANED
 
+E_MAKE_TYPE (e_vpaned,
+	     "EVPaned",
+	     EVPaned,
+	     e_vpaned_class_init,
+	     e_vpaned_init,
+	     PARENT_TYPE)
+	     
 static void
 e_vpaned_class_init (EVPanedClass *klass)
 {
@@ -87,7 +88,6 @@ e_vpaned_class_init (EVPanedClass *klass)
 
   widget_class->size_request = e_vpaned_size_request;
   widget_class->size_allocate = e_vpaned_size_allocate;
-  widget_class->draw = e_vpaned_draw;
   widget_class->button_press_event = e_vpaned_button_press;
   widget_class->button_release_event = e_vpaned_button_release;
   widget_class->motion_notify_event = e_vpaned_motion;
@@ -121,7 +121,7 @@ e_vpaned_new (void)
 {
   EVPaned *vpaned;
 
-  vpaned = gtk_type_new (E_TYPE_VPANED);
+  vpaned = g_object_new (E_TYPE_VPANED, NULL);
 
   return GTK_WIDGET (vpaned);
 }
@@ -265,6 +265,7 @@ e_vpaned_size_allocate (GtkWidget     *widget,
     }
 }
 
+#if 0
 static void
 e_vpaned_draw (GtkWidget    *widget,
 		 GdkRectangle *area)
@@ -293,6 +294,7 @@ e_vpaned_draw (GtkWidget    *widget,
 	gtk_widget_draw (paned->child2, &child_area);
     }
 }
+#endif
 
 static void
 e_vpaned_xor_line (EPaned *paned)
