@@ -55,6 +55,7 @@
 #include <config.h>
 #include <string.h> /* strcmp() */
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
 #include "gal/util/e-util.h"
 #include "gal/widgets/e-unicode.h"
 #include "e-table-item.h"
@@ -71,9 +72,9 @@
 #define E_CELL_COMBO_UTF8_KEY		"UTF-8-TEXT"
 
 
-static void e_cell_combo_class_init	(GtkObjectClass	*object_class);
+static void e_cell_combo_class_init	(GObjectClass	*object_class);
 static void e_cell_combo_init		(ECellCombo	*ecc);
-static void e_cell_combo_destroy	(GtkObject	*object);
+static void e_cell_combo_dispose	(GObject	*object);
 
 static gint e_cell_combo_do_popup	(ECellPopup	*ecp,
 					 GdkEvent	*event,
@@ -110,19 +111,19 @@ static ECellPopupClass *parent_class;
 
 E_MAKE_TYPE (e_cell_combo, "ECellCombo", ECellCombo,
 	     e_cell_combo_class_init, e_cell_combo_init,
-	     e_cell_popup_get_type());
+	     e_cell_popup_get_type())
 
 
 static void
-e_cell_combo_class_init			(GtkObjectClass	*object_class)
+e_cell_combo_class_init			(GObjectClass	*object_class)
 {
 	ECellPopupClass *ecpc = (ECellPopupClass *) object_class;
 
-	object_class->destroy = e_cell_combo_destroy;
+	object_class->dispose = e_cell_combo_dispose;
 
 	ecpc->popup = e_cell_combo_do_popup;
 
-	parent_class = gtk_type_class (e_cell_popup_get_type ());
+	parent_class = g_type_class_ref (E_CELL_POPUP_TYPE);
 }
 
 
@@ -162,19 +163,19 @@ e_cell_combo_init			(ECellCombo	*ecc)
 					     gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (ecc->popup_scrolled_window)));
 	gtk_widget_show (ecc->popup_list);
 
-	gtk_signal_connect (GTK_OBJECT (ecc->popup_window),
-			    "button_press_event",
-			    GTK_SIGNAL_FUNC (e_cell_combo_button_press),
-			    ecc);
+	g_signal_connect (ecc->popup_window,
+			  "button_press_event",
+			  G_CALLBACK (e_cell_combo_button_press),
+			  ecc);
 	/* We use connect_after here so the list updates the selection before
 	   we hide the popup and update the cell. */
-	gtk_signal_connect_after (GTK_OBJECT (ecc->popup_window),
-				  "button_release_event",
-				  GTK_SIGNAL_FUNC (e_cell_combo_button_release),
-				  ecc);
-	gtk_signal_connect (GTK_OBJECT (ecc->popup_window),
-			    "key_press_event",
-			    GTK_SIGNAL_FUNC (e_cell_combo_key_press), ecc);
+	g_signal_connect_after (ecc->popup_window,
+				"button_release_event",
+				G_CALLBACK (e_cell_combo_button_release),
+				ecc);
+	g_signal_connect (ecc->popup_window,
+			  "key_press_event",
+			  G_CALLBACK (e_cell_combo_key_press), ecc);
 }
 
 
@@ -188,24 +189,25 @@ e_cell_combo_init			(ECellCombo	*ecc)
 ECell *
 e_cell_combo_new			(void)
 {
-	ECellCombo *ecc = gtk_type_new (e_cell_combo_get_type ());
+	ECellCombo *ecc = g_object_new (E_CELL_COMBO_TYPE, NULL);
 
 	return (ECell*) ecc;
 }
 
 
 /*
- * GtkObject::destroy method
+ * GObject::dispose method
  */
 static void
-e_cell_combo_destroy			(GtkObject *object)
+e_cell_combo_dispose			(GObject *object)
 {
 	ECellCombo *ecc = E_CELL_COMBO (object);
 
-	gtk_widget_destroy (ecc->popup_window);
+	if (ecc->popup_window)
+		gtk_widget_destroy (ecc->popup_window);
 	ecc->popup_window = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -236,9 +238,9 @@ e_cell_combo_set_popdown_strings	(ECellCombo	*ecc,
 		gtk_widget_show (listitem);
 		gtk_container_add (GTK_CONTAINER (ecc->popup_list), listitem);
 
-		gtk_object_set_data_full (GTK_OBJECT (listitem),
-					  E_CELL_COMBO_UTF8_KEY,
-					  g_strdup (utf8_text), g_free);
+		g_object_set_data_full (G_OBJECT (listitem),
+					E_CELL_COMBO_UTF8_KEY,
+					g_strdup (utf8_text), g_free);
 
 		elem = elem->next;
 	}
@@ -304,8 +306,8 @@ e_cell_combo_select_matching_item	(ECellCombo	*ecc)
 		listitem = GTK_WIDGET (elem->data);
 
 		/* We need to compare against the UTF-8 text. */
-		list_item_text = gtk_object_get_data (GTK_OBJECT (listitem),
-						      E_CELL_COMBO_UTF8_KEY);
+		list_item_text = g_object_get_data (G_OBJECT (listitem),
+						    E_CELL_COMBO_UTF8_KEY);
 
 		if (list_item_text && !strcmp (list_item_text, cell_text)) {
 			found = TRUE;
@@ -405,7 +407,7 @@ e_cell_combo_get_popup_pos		(ECellCombo	*ecc,
 	*y += y1 + 1;
 
 	scrollbar_width = popup->vscrollbar->requisition.width
-		+ GTK_SCROLLED_WINDOW_CLASS (GTK_OBJECT (popup)->klass)->scrollbar_spacing;
+		+ GTK_SCROLLED_WINDOW_CLASS (GTK_OBJECT_GET_CLASS (popup))->scrollbar_spacing;
 
 	avail_height = gdk_screen_height () - *y;
 
@@ -422,11 +424,11 @@ e_cell_combo_get_popup_pos		(ECellCombo	*ecc,
   
 	/* Calculate the desired width. */
 	*width = list_requisition.width
-		+ 2 * popwin->child->style->klass->xthickness
+		+ 2 * popwin->child->style->xthickness
 		+ 2 * GTK_CONTAINER (popwin->child)->border_width
 		+ 2 * GTK_CONTAINER (popup)->border_width
 		+ 2 * GTK_CONTAINER (GTK_BIN (popup)->child)->border_width
-		+ 2 * GTK_BIN (popup)->child->style->klass->xthickness;
+		+ 2 * GTK_BIN (popup)->child->style->xthickness;
 
 	/* Use at least the same width as the column. */
 	if (*width < column_width)
@@ -440,16 +442,16 @@ e_cell_combo_get_popup_pos		(ECellCombo	*ecc,
 	}
 
 	/* Calculate all the borders etc. that we need to add to the height. */
-	work_height = (2 * popwin->child->style->klass->ythickness
+	work_height = (2 * popwin->child->style->ythickness
 		       + 2 * GTK_CONTAINER (popwin->child)->border_width
 		       + 2 * GTK_CONTAINER (popup)->border_width
 		       + 2 * GTK_CONTAINER (GTK_BIN (popup)->child)->border_width
-		       + 2 * GTK_BIN (popup)->child->style->klass->xthickness);
+		       + 2 * GTK_BIN (popup)->child->style->xthickness);
 
 	/* Add on the height of the horizontal scrollbar if we need it. */
 	if (show_hscroll)
 		work_height += popup->hscrollbar->requisition.height +
-			GTK_SCROLLED_WINDOW_CLASS (GTK_OBJECT (popup)->klass)->scrollbar_spacing;
+			GTK_SCROLLED_WINDOW_CLASS (GTK_OBJECT_GET_CLASS (popup))->scrollbar_spacing;
 
 	/* Check if it fits in the available height. */
 	if (work_height + list_requisition.height > avail_height) {
@@ -620,8 +622,8 @@ e_cell_combo_update_cell		(ECellCombo	*ecc)
 
 	/* Get the text of the selected item. */
 	listitem = list->selection->data;
-	text = gtk_object_get_data (GTK_OBJECT (listitem),
-				    E_CELL_COMBO_UTF8_KEY);
+	text = g_object_get_data (G_OBJECT (listitem),
+				  E_CELL_COMBO_UTF8_KEY);
 	g_return_if_fail (text != NULL);
 
 	/* Compare it with the existing cell contents. */
