@@ -42,7 +42,7 @@
 
 static void	e_cell_popup_class_init	(GtkObjectClass	*object_class);
 static void	e_cell_popup_init	(ECellPopup	*ecp);
-static void	e_cell_popup_destroy	(GtkObject	*object);
+static void	e_cell_popup_dispose	(GObject	*object);
 
 
 static ECellView* ecp_new_view		(ECell		*ecell,
@@ -114,7 +114,7 @@ static ECellClass *parent_class;
 
 
 E_MAKE_TYPE (e_cell_popup, "ECellPopup", ECellPopup, e_cell_popup_class_init,
-	     e_cell_popup_init, e_cell_get_type());
+	     e_cell_popup_init, e_cell_get_type())
 
 
 static void
@@ -122,7 +122,7 @@ e_cell_popup_class_init		(GtkObjectClass	*object_class)
 {
 	ECellClass *ecc = (ECellClass *) object_class;
 
-	object_class->destroy = e_cell_popup_destroy;
+	G_OBJECT_CLASS (object_class)->dispose = e_cell_popup_dispose;
 
 	ecc->new_view     = ecp_new_view;
 	ecc->kill_view    = ecp_kill_view;
@@ -139,7 +139,7 @@ e_cell_popup_class_init		(GtkObjectClass	*object_class)
 	ecc->show_tooltip = ecp_show_tooltip;
 	ecc->get_bg_color = ecp_get_bg_color;
 
-	parent_class = gtk_type_class (e_cell_get_type ());
+	parent_class = g_type_class_ref (E_CELL_TYPE);
 }
 
 
@@ -161,7 +161,7 @@ e_cell_popup_init		(ECellPopup	*ecp)
 ECell *
 e_cell_popup_new		(void)
 {
-	ECellPopup *ecp = gtk_type_new (e_cell_popup_get_type ());
+	ECellPopup *ecp = g_object_new (E_CELL_POPUP_TYPE, NULL);
 
 	return (ECell*) ecp;
 }
@@ -171,13 +171,15 @@ e_cell_popup_new		(void)
  * GtkObject::destroy method
  */
 static void
-e_cell_popup_destroy		(GtkObject *object)
+e_cell_popup_dispose (GObject *object)
 {
 	ECellPopup *ecp = E_CELL_POPUP (object);
 
-	gtk_object_unref (GTK_OBJECT (ecp->child));
+	if (ecp->child)
+		g_object_unref (ecp->child);
+	ecp->child = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -277,11 +279,6 @@ ecp_draw (ECellView *ecv, GdkDrawable *drawable,
 	if (flags & E_CELL_CURSOR)
 		ecp->popup_arrow_shown = show_popup_arrow;
 
-#if 0
-	g_print ("In ecp_draw row:%i col: %i %i,%i %i,%i Show Arrow:%i\n",
-		 row, view_col, x1, y1, x2, y2, show_popup_arrow);
-#endif
-
 	if (show_popup_arrow) {
 		e_cell_draw (ecp_view->child_view, drawable, model_col,
 			     view_col, row, flags,
@@ -482,10 +479,10 @@ e_cell_popup_set_child			(ECellPopup	*ecp,
 	g_return_if_fail (E_IS_CELL_POPUP (ecp));
 
 	if (ecp->child)
-		gtk_object_unref (GTK_OBJECT (ecp->child));
+		g_object_unref (ecp->child);
 
 	ecp->child = child;
-	gtk_object_ref (GTK_OBJECT (child));
+	g_object_ref (child);
 }
 
 
@@ -500,7 +497,7 @@ e_cell_popup_do_popup			(ECellPopupView	*ecp_view,
 
 	ecp->popup_cell_view = ecp_view;
 
-	popup_func = E_CELL_POPUP_CLASS (GTK_OBJECT (ecp)->klass)->popup;
+	popup_func = E_CELL_POPUP_CLASS (GTK_OBJECT_GET_CLASS (ecp))->popup;
 
 	ecp->popup_view_col = view_col;
 	ecp->popup_row = row;
