@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-table-header-item.c
  * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
@@ -29,17 +29,20 @@
 #include <string.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkdnd.h>
-#include <libgnomeui/gnome-canvas.h>
-#include <libgnomeui/gnome-canvas-util.h>
-#include <libgnomeui/gnome-canvas-polygon.h>
-#include <libgnomeui/gnome-canvas-rect-ellipse.h>
+#include <gtk/gtkimage.h>
+#include <libgnomecanvas/gnome-canvas.h>
+#include <libgnomecanvas/gnome-canvas-util.h>
+#include <libgnomecanvas/gnome-canvas-polygon.h>
+#include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gal/widgets/e-cursors.h"
 #include "gal/util/e-i18n.h"
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
+#include "gal/util/e-marshal.h"
 #include "gal/widgets/e-canvas.h"
 #include "gal/widgets/e-popup-menu.h"
+#include "gal/widgets/e-gui-utils.h"
 #include "e-table-header.h"
 #include "e-table-header-utils.h"
 #include "e-table-col-dnd.h"
@@ -91,22 +94,34 @@ static GdkPixmap *remove_col_pixmap, *remove_col_mask;
 static GdkPixmap *add_col_pixmap, *add_col_mask;
 
 enum {
-	ARG_0,
-	ARG_TABLE_HEADER,
-	ARG_FULL_HEADER,
-	ARG_DND_CODE,
-	ARG_TABLE_FONTSET,
-	ARG_SORT_INFO,
-	ARG_TABLE,
-	ARG_TREE,
+	PROP_0,
+	PROP_TABLE_HEADER,
+	PROP_FULL_HEADER,
+	PROP_DND_CODE,
+	PROP_TABLE_FONTSET,
+	PROP_SORT_INFO,
+	PROP_TABLE,
+	PROP_TREE
 };
 
+enum {
+	ET_SCROLL_UP = 1 << 0,
+	ET_SCROLL_DOWN = 1 << 1,
+	ET_SCROLL_LEFT = 1 << 2,
+	ET_SCROLL_RIGHT = 1 << 3
+};
+
+static void scroll_off (ETableHeaderItem *ethi);
+static void scroll_on (ETableHeaderItem *ethi, guint scroll_direction);
+
 static void
-ethi_destroy (GtkObject *object){
+ethi_dispose (GObject *object){
 	ETableHeaderItem *ethi = E_TABLE_HEADER_ITEM (object);
 
 	ethi_drop_table_header (ethi);
 
+	scroll_off (ethi);
+
 	if (ethi->dnd_code) {
 		g_free (ethi->dnd_code);
 		ethi->dnd_code = NULL;
@@ -114,21 +129,23 @@ ethi_destroy (GtkObject *object){
 
 	if (ethi->sort_info) {
 		if (ethi->sort_info_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT(ethi->sort_info), ethi->sort_info_changed_id);
+			g_signal_handler_disconnect (ethi->sort_info, ethi->sort_info_changed_id);
 		if (ethi->group_info_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT(ethi->sort_info), ethi->group_info_changed_id);
-		gtk_object_unref (GTK_OBJECT(ethi->sort_info));
+			g_signal_handler_disconnect (ethi->sort_info, ethi->group_info_changed_id);
+		g_object_unref (ethi->sort_info);
 		ethi->sort_info = NULL;
 	}
 
 	if (ethi->full_header)
-		gtk_object_unref (GTK_OBJECT(ethi->full_header));
+		g_object_unref (ethi->full_header);
+	ethi->full_header = NULL;
 
 	if (ethi->config)
-		gtk_object_destroy (GTK_OBJECT (ethi->config));
+		g_object_unref (ethi->config);
+	ethi->config = NULL;
 	
-	if (GTK_OBJECT_CLASS (ethi_parent_class)->destroy)
-		(*GTK_OBJECT_CLASS (ethi_parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (ethi_parent_class)->dispose)
+		(*G_OBJECT_CLASS (ethi_parent_class)->dispose) (object);
 }
 
 static int
@@ -153,7 +170,8 @@ e_table_header_item_get_height (ETableHeaderItem *ethi)
 		ETableCol *ecol = e_table_header_get_column (eth, col);
 		int height;
 
-		height = e_table_header_compute_height (ecol, style, ethi->font);
+		height = e_table_header_compute_height (ecol,
+							GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas));
 
 		if (height > maxheight)
 			maxheight = height;
@@ -198,8 +216,12 @@ ethi_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags
 			item->y1 = c1.y;
 			item->x2 = c2.x;
 			item->y2 = c2.y;
-
+#ifndef NO_WARNINGS
+#warning FOO BAA
+#endif
+#if 0
 			gnome_canvas_group_child_bounds (GNOME_CANVAS_GROUP (item->parent), item);
+#endif
 		}
 	gnome_canvas_request_redraw (item->canvas, item->x1, item->y1, item->x2, item->y2);
 }
@@ -218,7 +240,7 @@ ethi_font_set (ETableHeaderItem *ethi, GdkFont *font)
 }
 
 static void
-ethi_font_load (ETableHeaderItem *ethi, char *fontname)
+ethi_font_load (ETableHeaderItem *ethi, const char *fontname)
 {
 	GdkFont *font = NULL;
 
@@ -226,7 +248,7 @@ ethi_font_load (ETableHeaderItem *ethi, char *fontname)
 		font = gdk_fontset_load (fontname);
 
 	if (font == NULL) {
-		font = GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas)->style->font;
+		font = gtk_style_get_font (GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas)->style);
 		gdk_font_ref (font);
 	}
 	
@@ -237,16 +259,16 @@ ethi_font_load (ETableHeaderItem *ethi, char *fontname)
 static void
 ethi_drop_table_header (ETableHeaderItem *ethi)
 {
-	GtkObject *header;
+	GObject *header;
 	
 	if (!ethi->eth)
 		return;
 
-	header = GTK_OBJECT (ethi->eth);
-	gtk_signal_disconnect (header, ethi->structure_change_id);
-	gtk_signal_disconnect (header, ethi->dimension_change_id);
+	header = G_OBJECT (ethi->eth);
+	g_signal_handler_disconnect (header, ethi->structure_change_id);
+	g_signal_handler_disconnect (header, ethi->dimension_change_id);
 
-	gtk_object_unref (header);
+	g_object_unref (header);
 	ethi->eth = NULL;
 	ethi->width = 0;
 }
@@ -267,16 +289,16 @@ static void
 ethi_add_table_header (ETableHeaderItem *ethi, ETableHeader *header)
 {
 	ethi->eth = header;
-	gtk_object_ref (GTK_OBJECT (ethi->eth));
+	g_object_ref (ethi->eth);
 
 	ethi->height = e_table_header_item_get_height (ethi);
 
-	ethi->structure_change_id = gtk_signal_connect (
-		GTK_OBJECT (header), "structure_change",
-		GTK_SIGNAL_FUNC(structure_changed), ethi);
-	ethi->dimension_change_id = gtk_signal_connect (
-		GTK_OBJECT (header), "dimension_change",
-		GTK_SIGNAL_FUNC(dimension_changed), ethi);
+	ethi->structure_change_id = g_signal_connect (
+		header, "structure_change",
+		G_CALLBACK (structure_changed), ethi);
+	ethi->dimension_change_id = g_signal_connect (
+		header, "dimension_change",
+		G_CALLBACK (dimension_changed), ethi);
 	e_canvas_item_request_reflow(GNOME_CANVAS_ITEM(ethi));
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM(ethi));
 }
@@ -288,70 +310,73 @@ ethi_sort_info_changed (ETableSortInfo *sort_info, ETableHeaderItem *ethi)
 }
 
 static void
-ethi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ethi_set_property (GObject *object,
+		   guint prop_id,
+		   const GValue *value,
+		   GParamSpec *pspec)
 {
 	GnomeCanvasItem *item;
 	ETableHeaderItem *ethi;
 
-	item = GNOME_CANVAS_ITEM (o);
-	ethi = E_TABLE_HEADER_ITEM (o);
+	item = GNOME_CANVAS_ITEM (object);
+	ethi = E_TABLE_HEADER_ITEM (object);
 
-	switch (arg_id){
-	case ARG_TABLE_HEADER:
+	switch (prop_id){
+	case PROP_TABLE_HEADER:
 		ethi_drop_table_header (ethi);
-		ethi_add_table_header (ethi, E_TABLE_HEADER(GTK_VALUE_OBJECT (*arg)));
+		ethi_add_table_header (ethi, E_TABLE_HEADER(g_value_get_object (value)));
 		break;
 
-	case ARG_FULL_HEADER:
+	case PROP_FULL_HEADER:
 		if (ethi->full_header)
-			gtk_object_unref(GTK_OBJECT(ethi->full_header));
-		ethi->full_header = E_TABLE_HEADER(GTK_VALUE_OBJECT (*arg));
+			g_object_unref(ethi->full_header);
+		ethi->full_header = E_TABLE_HEADER(g_value_get_object (value));
 		if (ethi->full_header)
-			gtk_object_ref(GTK_OBJECT(ethi->full_header));
+			g_object_ref(ethi->full_header);
 		break;
 
-	case ARG_DND_CODE:
+	case PROP_DND_CODE:
 		g_free(ethi->dnd_code);
-		ethi->dnd_code = g_strdup (GTK_VALUE_STRING (*arg));
+		ethi->dnd_code = g_strdup (g_value_get_string (value));
 		break;
 
-	case ARG_TABLE_FONTSET:
-		ethi_font_load (ethi, GTK_VALUE_STRING (*arg));
+	case PROP_TABLE_FONTSET:
+		ethi_font_load (ethi, g_value_get_string (value));
 		break;
 
-	case ARG_SORT_INFO:
+	case PROP_SORT_INFO:
 		if (ethi->sort_info){
 			if (ethi->sort_info_changed_id)
-				gtk_signal_disconnect (
-					GTK_OBJECT(ethi->sort_info),
+				g_signal_handler_disconnect (
+					ethi->sort_info,
 					ethi->sort_info_changed_id);
 
 			if (ethi->group_info_changed_id)
-				gtk_signal_disconnect (
-					GTK_OBJECT(ethi->sort_info),
+				g_signal_handler_disconnect (
+					ethi->sort_info,
 					ethi->group_info_changed_id);
-			gtk_object_unref (GTK_OBJECT(ethi->sort_info));
+			g_object_unref (ethi->sort_info);
 		}
-		ethi->sort_info = GTK_VALUE_POINTER (*arg);
-		gtk_object_ref (GTK_OBJECT(ethi->sort_info));
+		ethi->sort_info = g_value_get_object (value);
+		g_object_ref (ethi->sort_info);
 		ethi->sort_info_changed_id =
-			gtk_signal_connect (
-				GTK_OBJECT(ethi->sort_info), "sort_info_changed",
-				GTK_SIGNAL_FUNC(ethi_sort_info_changed), ethi);
+			g_signal_connect (
+				ethi->sort_info, "sort_info_changed",
+				G_CALLBACK (ethi_sort_info_changed), ethi);
 		ethi->group_info_changed_id =
-			gtk_signal_connect (
-				GTK_OBJECT(ethi->sort_info), "group_info_changed",
-				GTK_SIGNAL_FUNC(ethi_sort_info_changed), ethi);
+			g_signal_connect (
+				ethi->sort_info, "group_info_changed",
+				G_CALLBACK(ethi_sort_info_changed), ethi);
 		break;
-	case ARG_TABLE:
-		if (GTK_VALUE_OBJECT(*arg))
-			ethi->table = E_TABLE(GTK_VALUE_OBJECT(*arg));
+	case PROP_TABLE:
+		if (g_value_get_object (value))
+			ethi->table = E_TABLE(g_value_get_object (value));
 		else
 			ethi->table = NULL;
 		break;
-	case ARG_TREE:
-		if (GTK_VALUE_OBJECT(*arg))
-			ethi->tree = E_TREE(GTK_VALUE_OBJECT(*arg));
+	case PROP_TREE:
+		if (g_value_get_object (value))
+			ethi->tree = E_TREE(g_value_get_object (value));
 		else
 			ethi->tree = NULL;
 		break;
@@ -360,21 +385,24 @@ ethi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 }
 
 static void
-ethi_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ethi_get_property (GObject *object,
+		   guint prop_id,
+		   GValue *value,
+		   GParamSpec *pspec)
 {
 	ETableHeaderItem *ethi;
 
-	ethi = E_TABLE_HEADER_ITEM (o);
+	ethi = E_TABLE_HEADER_ITEM (object);
 
-	switch (arg_id){
-	case ARG_FULL_HEADER:
-		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (ethi->full_header);
+	switch (prop_id){
+	case PROP_FULL_HEADER:
+		g_value_set_object (value, ethi->full_header);
 		break;
-	case ARG_DND_CODE:
-		GTK_VALUE_STRING (*arg) = g_strdup (ethi->dnd_code);
+	case PROP_DND_CODE:
+		g_value_set_string (value, g_strdup (ethi->dnd_code));
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
@@ -458,14 +486,13 @@ make_shaped_window_from_xpm (const char **xpm)
 	gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &bitmap, 128);
 	gdk_pixbuf_unref (pixbuf);
 
-	gtk_widget_push_visual (gdk_rgb_get_visual ());
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 	win = gtk_window_new (GTK_WINDOW_POPUP);
-	pix = gtk_pixmap_new (pixmap, bitmap);
+
+	pix = gtk_image_new_from_pixmap (pixmap, bitmap);
 	gtk_widget_realize (win);
 	gtk_container_add (GTK_CONTAINER (win), pix);
 	gtk_widget_shape_combine_mask (win, bitmap, 0, 0);
-	gtk_widget_pop_visual ();
 	gtk_widget_pop_colormap ();
 	
 	gdk_pixmap_unref (pixmap);
@@ -475,12 +502,12 @@ make_shaped_window_from_xpm (const char **xpm)
 }
 
 static void
-ethi_add_drop_marker (ETableHeaderItem *ethi, int col)
+ethi_add_drop_marker (ETableHeaderItem *ethi, int col, gboolean recreate)
 {
 	int rx, ry;
 	int x;
 	
-	if (ethi->drag_mark == col)
+	if (!recreate && ethi->drag_mark == col)
 		return;
 
 	ethi->drag_mark = col;
@@ -498,6 +525,9 @@ ethi_add_drop_marker (ETableHeaderItem *ethi, int col)
 		GTK_WIDGET (GNOME_CANVAS_ITEM (ethi)->canvas)->window,
 		&rx, &ry);
 
+	rx -= gtk_layout_get_hadjustment (GTK_LAYOUT (GNOME_CANVAS_ITEM (ethi)->canvas))->value;
+	ry -= gtk_layout_get_vadjustment (GTK_LAYOUT (GNOME_CANVAS_ITEM (ethi)->canvas))->value;
+
 	gtk_widget_set_uposition (arrow_down, rx + x - ARROW_PTR, ry - ARROW_DOWN_HEIGHT);
 	gtk_widget_show_all (arrow_down);
 
@@ -568,29 +598,21 @@ moved (ETableHeaderItem *ethi, guint col, guint model_col)
 }
 #endif
 
-static gboolean
-ethi_drag_motion (GtkObject *canvas, GdkDragContext *context,
-		  gint x, gint y, guint time,
-		  ETableHeaderItem *ethi)
+static void
+do_drag_motion(ETableHeaderItem *ethi,
+	       GdkDragContext *context,
+	       gint x,
+	       gint y,
+	       guint time,
+	       gboolean recreate)
 {
-	char *droptype, *headertype;
-
-	gdk_drag_status (context, 0, time);
-
-	droptype = gdk_atom_name (GPOINTER_TO_INT (context->targets->data));
-	headertype = g_strdup_printf ("%s-%s", TARGET_ETABLE_COL_TYPE,
-				      ethi->dnd_code);
-
-	if (strcmp (droptype, headertype) != 0) {
-		g_free (headertype);
-		return FALSE;
-	}
-
-	g_free (headertype);
+	d(g_print("In do_drag_motion\n"));
+	d(g_print("x = %d, y = %d, ethi->width = %d, ethi->height = %d\n", x, y, ethi->width, ethi->height));
 
 	if ((x >= 0) && (x <= (ethi->width)) &&
 	    (y >= 0) && (y <= (ethi->height))){
 		int col;
+		d(g_print("In header\n"));
 		
 		col = ethi_find_col_by_x_nearest (ethi, x);
 
@@ -605,7 +627,7 @@ ethi_drag_motion (GtkObject *canvas, GdkDragContext *context,
 			if (ethi->drag_col != -1)
 				ethi_remove_destroy_marker (ethi);
 
-			ethi_add_drop_marker (ethi, col);
+			ethi_add_drop_marker (ethi, col, recreate);
 			gdk_drag_status (context, context->suggested_action, time);
 		} else {
 			ethi_remove_drop_marker (ethi);
@@ -617,6 +639,130 @@ ethi_drag_motion (GtkObject *canvas, GdkDragContext *context,
 		if (ethi->drag_col != -1)
 			ethi_add_destroy_marker (ethi);
 	}
+}
+
+static gboolean
+scroll_timeout (gpointer data)
+{
+	ETableHeaderItem *ethi = data;
+	int dx = 0;
+	GtkAdjustment *h, *v;
+	double value;
+
+	if (ethi->scroll_direction & ET_SCROLL_RIGHT)
+		dx += 20;
+	if (ethi->scroll_direction & ET_SCROLL_LEFT)
+		dx -= 20;
+
+	h = GTK_LAYOUT(GNOME_CANVAS_ITEM (ethi)->canvas)->hadjustment;
+	v = GTK_LAYOUT(GNOME_CANVAS_ITEM (ethi)->canvas)->vadjustment;
+
+	value = h->value;
+
+	gtk_adjustment_set_value(h, CLAMP(h->value + dx, h->lower, h->upper - h->page_size));
+
+	if (h->value != value)
+		do_drag_motion(ethi,
+			       ethi->last_drop_context,
+			       ethi->last_drop_x + h->value,
+			       ethi->last_drop_y + v->value,
+			       ethi->last_drop_time,
+			       TRUE);
+
+	return TRUE;
+}
+
+static void
+scroll_on (ETableHeaderItem *ethi, guint scroll_direction)
+{
+	if (ethi->scroll_idle_id == 0 || scroll_direction != ethi->scroll_direction) {
+		if (ethi->scroll_idle_id != 0)
+			g_source_remove (ethi->scroll_idle_id);
+		ethi->scroll_direction = scroll_direction;
+		ethi->scroll_idle_id = g_timeout_add (100, scroll_timeout, ethi);
+	}
+}
+
+static void
+scroll_off (ETableHeaderItem *ethi)
+{
+	if (ethi->scroll_idle_id) {
+		g_source_remove (ethi->scroll_idle_id);
+		ethi->scroll_idle_id = 0;
+	}
+}
+
+static void
+context_destroyed (gpointer data)
+{
+	ETableHeaderItem *ethi = data;
+
+	ethi->last_drop_x       = 0;
+	ethi->last_drop_y       = 0;
+	ethi->last_drop_time    = 0;
+	ethi->last_drop_context = NULL;
+	scroll_off (ethi);
+
+	g_object_unref (ethi);
+}
+
+static void
+context_connect (ETableHeaderItem *ethi, GdkDragContext *context)
+{
+	if (g_dataset_get_data (context, "e-table-header-item") == NULL) {
+		g_object_ref (ethi);
+		g_dataset_set_data_full (context, "e-table-header-item", ethi, context_destroyed);
+	}
+}
+
+static gboolean
+ethi_drag_motion (GtkWidget *widget, GdkDragContext *context,
+		  gint x, gint y, guint time,
+		  ETableHeaderItem *ethi)
+{
+	char *droptype, *headertype;
+	guint direction = 0;
+
+	gdk_drag_status (context, 0, time);
+
+	droptype = gdk_atom_name (GDK_POINTER_TO_ATOM (context->targets->data));
+	headertype = g_strdup_printf ("%s-%s", TARGET_ETABLE_COL_TYPE,
+				      ethi->dnd_code);
+
+	if (strcmp (droptype, headertype) != 0) {
+		g_free (headertype);
+		return FALSE;
+	}
+
+	g_free (headertype);
+
+	d(g_print ("y = %d, widget->allocation.y = %d, GTK_LAYOUT (widget)->vadjustment->value = %f\n", y, widget->allocation.y, GTK_LAYOUT (widget)->vadjustment->value));
+
+	x -= widget->allocation.x;
+	y -= widget->allocation.y;
+
+	if (x < 20)
+		direction |= ET_SCROLL_LEFT;
+	if (x > widget->allocation.width - 20)
+		direction |= ET_SCROLL_RIGHT;
+
+	ethi->last_drop_x = x;
+	ethi->last_drop_y = y;
+	ethi->last_drop_time = time;
+	ethi->last_drop_context = context;
+	context_connect (ethi, context);
+
+	do_drag_motion (ethi,
+			context,
+			x + GTK_LAYOUT(widget)->hadjustment->value,
+			y + GTK_LAYOUT(widget)->vadjustment->value,
+			time,
+			FALSE);
+
+	if (direction != 0)
+		scroll_on (ethi, direction);
+	else
+		scroll_off (ethi);
 
 	return TRUE;
 }
@@ -631,6 +777,7 @@ ethi_drag_end (GtkWidget *canvas, GdkDragContext *context, ETableHeaderItem *eth
 	ethi_remove_drop_marker (ethi);
 	ethi_remove_destroy_marker (ethi);
 	ethi->drag_col = -1;
+	scroll_off (ethi);
 }
 
 static void
@@ -717,17 +864,19 @@ ethi_drag_drop (GtkWidget *canvas,
 		
 		col = ethi_find_col_by_x_nearest (ethi, x);
 		
-		ethi_add_drop_marker (ethi, col);
+		ethi_add_drop_marker (ethi, col, FALSE);
 
 		ethi->drop_col = col;
 		
 		if (col != -1) {
 			char *target = g_strdup_printf ("%s-%s", TARGET_ETABLE_COL_TYPE, ethi->dnd_code);
+			d(g_print ("ethi -  %s\n", target));
 			gtk_drag_get_data (canvas, context, gdk_atom_intern(target, FALSE), time);
 			g_free (target);
 		}
 	}
 	gtk_drag_finish (context, successful, successful, time);
+	scroll_off (ethi);
 	return successful;
 }
 
@@ -755,7 +904,7 @@ ethi_realize (GnomeCanvasItem *item)
 	window = GTK_WIDGET (item->canvas)->window;
 
 	if (!ethi->font)
-		ethi_font_set (ethi, GTK_WIDGET (item->canvas)->style->font);
+		ethi_font_set (ethi, gtk_style_get_font (GTK_WIDGET (item->canvas)->style));
 
 	/*
 	 * Now, configure DnD
@@ -767,20 +916,20 @@ ethi_realize (GnomeCanvasItem *item)
   	g_free(ethi_drop_types[0].target); 
 
 	/* Drop signals */
-	ethi->drag_motion_id = gtk_signal_connect        (GTK_OBJECT (item->canvas), "drag_motion",
-							  GTK_SIGNAL_FUNC (ethi_drag_motion), ethi);
-	ethi->drag_leave_id = gtk_signal_connect         (GTK_OBJECT (item->canvas), "drag_leave",
-							  GTK_SIGNAL_FUNC (ethi_drag_leave), ethi);
-	ethi->drag_drop_id = gtk_signal_connect          (GTK_OBJECT (item->canvas), "drag_drop",
-							  GTK_SIGNAL_FUNC (ethi_drag_drop), ethi);
-	ethi->drag_data_received_id = gtk_signal_connect (GTK_OBJECT (item->canvas), "drag_data_received",
-							  GTK_SIGNAL_FUNC (ethi_drag_data_received), ethi);
+	ethi->drag_motion_id = g_signal_connect (item->canvas, "drag_motion",
+						 G_CALLBACK (ethi_drag_motion), ethi);
+	ethi->drag_leave_id = g_signal_connect (item->canvas, "drag_leave",
+						G_CALLBACK (ethi_drag_leave), ethi);
+	ethi->drag_drop_id = g_signal_connect (item->canvas, "drag_drop",
+					       G_CALLBACK (ethi_drag_drop), ethi);
+	ethi->drag_data_received_id = g_signal_connect (item->canvas, "drag_data_received",
+							G_CALLBACK (ethi_drag_data_received), ethi);
 
 	/* Drag signals */
-	ethi->drag_end_id = gtk_signal_connect           (GTK_OBJECT (item->canvas), "drag_end",
-							  GTK_SIGNAL_FUNC (ethi_drag_end), ethi);
-	ethi->drag_data_get_id = gtk_signal_connect      (GTK_OBJECT (item->canvas), "drag_data_get",
-							  GTK_SIGNAL_FUNC (ethi_drag_data_get), ethi);
+	ethi->drag_end_id = g_signal_connect (item->canvas, "drag_end",
+					      G_CALLBACK (ethi_drag_end), ethi);
+	ethi->drag_data_get_id = g_signal_connect (item->canvas, "drag_data_get",
+						   G_CALLBACK (ethi_drag_data_get), ethi);
 
 }
 
@@ -791,13 +940,15 @@ ethi_unrealize (GnomeCanvasItem *item)
 
 	gdk_font_unref (ethi->font);
 
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_motion_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_leave_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_drop_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_data_received_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_motion_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_leave_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_drop_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_data_received_id);
 
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_end_id);
-	gtk_signal_disconnect (GTK_OBJECT (item->canvas), ethi->drag_data_get_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_end_id);
+	g_signal_handler_disconnect (item->canvas, ethi->drag_data_get_id);
+
+	gtk_drag_dest_unset (GTK_WIDGET (item->canvas));
 
 	if (ethi->stipple){
 		gdk_bitmap_unref (ethi->stipple);
@@ -862,7 +1013,7 @@ ethi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width
 			continue;
 
 		e_table_header_draw_button (drawable, ecol,
-					    GTK_WIDGET (canvas)->style, ethi->font,
+					    GTK_WIDGET (canvas)->style,
 					    GTK_WIDGET_STATE (canvas),
 					    GTK_WIDGET (canvas),
 					    x1 - x, -y,
@@ -1054,7 +1205,7 @@ ethi_start_drag (ETableHeaderItem *ethi, GdkEvent *event)
 
 	e_table_header_draw_button (
 		pixmap, ecol,
-		widget->style, ethi->font,
+		widget->style,
 		GTK_WIDGET_STATE (widget),
 		widget,
 		0, 0,
@@ -1227,11 +1378,11 @@ static void
 ethi_popup_field_chooser(GtkWidget *widget, EthiHeaderInfo *info)
 {
 	GtkWidget *etfcd = e_table_field_chooser_dialog_new();
-	gtk_object_set(GTK_OBJECT(etfcd),
-		       "full_header", info->ethi->full_header,
-		       "header", info->ethi->eth,
-		       "dnd_code", info->ethi->dnd_code,
-		       NULL);
+	g_object_set(etfcd,
+		     "full_header", info->ethi->full_header,
+		     "header", info->ethi->eth,
+		     "dnd_code", info->ethi->dnd_code,
+		     NULL);
 	gtk_widget_show(etfcd);
 }
 
@@ -1246,9 +1397,9 @@ ethi_popup_best_fit(GtkWidget *widget, EthiHeaderInfo *info)
 	ETableHeaderItem *ethi = info->ethi;
 	int width;
 
-	gtk_signal_emit_by_name (GTK_OBJECT (ethi->eth),
-				 "request_width",
-				 info->col, &width);
+	g_signal_emit_by_name (ethi->eth,
+			       "request_width",
+			       info->col, &width);
 	/* Add 10 to stop it from "..."ing */
 	e_table_header_set_size (ethi->eth, info->col, width + 10);
 	
@@ -1262,8 +1413,9 @@ ethi_popup_format_columns(GtkWidget *widget, EthiHeaderInfo *info)
 }
 
 static void
-config_destroyed (GtkObject *object, ETableHeaderItem *ethi)
+config_destroyed (gpointer data, GObject *where_object_was)
 {
+	ETableHeaderItem *ethi = data;
 	ethi->config = NULL;
 }
 
@@ -1300,36 +1452,41 @@ ethi_popup_customize_view(GtkWidget *widget, EthiHeaderInfo *info)
 
 		ethi->config = e_table_config_new (
 				_("Customize Current View"),
-				spec, state);
-		gtk_signal_connect (
-			GTK_OBJECT (ethi->config), "destroy",
-			GTK_SIGNAL_FUNC (config_destroyed), ethi);
-		gtk_signal_connect (
-			GTK_OBJECT (ethi->config), "changed",
-			GTK_SIGNAL_FUNC (apply_changes), ethi);
+				spec, state, NULL);
+		g_object_weak_ref (G_OBJECT (ethi->config),
+				   config_destroyed, ethi);
+		g_signal_connect (
+			ethi->config, "changed",
+			G_CALLBACK (apply_changes), ethi);
 	}
 }
 
+static void
+free_popup_info (GtkWidget *w, EthiHeaderInfo *info)
+{
+	g_free (info);
+}
+
 /* Bit 1 is always disabled. */
 /* Bit 2 is disabled if not "sortable". */
 /* Bit 4 is disabled if we don't have a pointer to our table object. */
 static EPopupMenu ethi_context_menu [] = {
-	{ N_("Sort Ascending"),            NULL, GTK_SIGNAL_FUNC(ethi_popup_sort_ascending),  NULL, 2},
-	{ N_("Sort Descending"),           NULL, GTK_SIGNAL_FUNC(ethi_popup_sort_descending), NULL, 2},
-	{ N_("Unsort"),                    NULL, GTK_SIGNAL_FUNC(ethi_popup_unsort),          NULL, 0},
-	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       NULL, 0},
-	{ N_("Group By This Field"),       NULL, GTK_SIGNAL_FUNC(ethi_popup_group_field),     NULL, 16},
-	{ N_("Group By Box"),              NULL, GTK_SIGNAL_FUNC(ethi_popup_group_box),       NULL, 128},
-	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       NULL, 1},
-	{ N_("Remove This Column"),        NULL, GTK_SIGNAL_FUNC(ethi_popup_remove_column),   NULL, 8},
-	{ N_("Add a Column..."),           NULL, GTK_SIGNAL_FUNC(ethi_popup_field_chooser),   NULL, 0},
-	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       NULL, 1},
-	{ N_("Alignment"),                 NULL, GTK_SIGNAL_FUNC(ethi_popup_alignment),       NULL, 128},
-	{ N_("Best Fit"),                  NULL, GTK_SIGNAL_FUNC(ethi_popup_best_fit),        NULL, 2},
-	{ N_("Format Columns..."),         NULL, GTK_SIGNAL_FUNC(ethi_popup_format_columns),  NULL, 128},
-	{ "",                              NULL, GTK_SIGNAL_FUNC(NULL),                       NULL, 1},
-	{ N_("Customize Current View..."), NULL, GTK_SIGNAL_FUNC(ethi_popup_customize_view),  NULL, 4},
-	{ NULL, NULL, NULL, NULL, 0 }
+	E_POPUP_ITEM (N_("Sort Ascending"),            G_CALLBACK(ethi_popup_sort_ascending),  2),
+	E_POPUP_ITEM (N_("Sort Descending"),           G_CALLBACK(ethi_popup_sort_descending), 2),
+	E_POPUP_ITEM (N_("Unsort"),                    G_CALLBACK(ethi_popup_unsort),          0),
+	E_POPUP_SEPARATOR,
+	E_POPUP_ITEM (N_("Group By This Field"),       G_CALLBACK(ethi_popup_group_field),     16),
+	E_POPUP_ITEM (N_("Group By Box"),              G_CALLBACK(ethi_popup_group_box),       128),
+	E_POPUP_SEPARATOR,
+	E_POPUP_ITEM (N_("Remove This Column"),        G_CALLBACK(ethi_popup_remove_column),   8),
+	E_POPUP_ITEM (N_("Add a Column..."),           G_CALLBACK(ethi_popup_field_chooser),   0),
+	E_POPUP_SEPARATOR,
+	E_POPUP_ITEM (N_("Alignment"),                 G_CALLBACK(ethi_popup_alignment),       128),
+	E_POPUP_ITEM (N_("Best Fit"),                  G_CALLBACK(ethi_popup_best_fit),        2),
+	E_POPUP_ITEM (N_("Format Columns..."),         G_CALLBACK(ethi_popup_format_columns),  128),
+	E_POPUP_SEPARATOR,
+	E_POPUP_ITEM (N_("Customize Current View..."), G_CALLBACK(ethi_popup_customize_view),  4),
+	E_POPUP_TERMINATOR
 };
 
 static void
@@ -1337,23 +1494,30 @@ ethi_header_context_menu (ETableHeaderItem *ethi, GdkEventButton *event)
 {
 	EthiHeaderInfo *info = g_new(EthiHeaderInfo, 1);
 	ETableCol *col;
+	GtkMenu *popup;
 	info->ethi = ethi;
 	info->col = ethi_find_col_by_x (ethi, event->x);
 	col = e_table_header_get_column (ethi->eth, info->col);
-	e_popup_menu_run (ethi_context_menu, (GdkEvent *) event,
-			  1 +
-			  (col->sortable ? 0 : 2) +
-			  ((ethi->table || ethi->tree) ? 0 : 4) + 
-			  ((e_table_header_count (ethi->eth) > 1) ? 0 : 8),
-			  ((e_table_sort_info_get_can_group (ethi->sort_info)) ? 0 : 16) +
-			  128, info);
+
+	popup = e_popup_menu_create_with_domain (ethi_context_menu,
+						 1 +
+						 (col->sortable ? 0 : 2) +
+						 ((ethi->table || ethi->tree) ? 0 : 4) + 
+						 ((e_table_header_count (ethi->eth) > 1) ? 0 : 8),
+						 ((e_table_sort_info_get_can_group (ethi->sort_info)) ? 0 : 16) +
+						 128, info, E_I18N_DOMAIN);
+	g_object_ref (popup);
+	gtk_object_sink (GTK_OBJECT (popup));
+	g_signal_connect (popup, "selection-done",
+			  G_CALLBACK (free_popup_info), info);
+	e_popup_menu (popup, (GdkEvent *) event);
 }
 
 static void
 ethi_button_pressed (ETableHeaderItem *ethi, GdkEventButton *event)
 {
-	gtk_signal_emit (GTK_OBJECT (ethi),
-			 ethi_signals [BUTTON_PRESSED], event);
+	g_signal_emit (ethi,
+		       ethi_signals [BUTTON_PRESSED], 0, event);
 }
 
 static void
@@ -1518,9 +1682,9 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 			break;
 		else {
 			int width = 0;
-			gtk_signal_emit_by_name (GTK_OBJECT (ethi->eth),
-						 "request_width",
-						 (int)ethi->resize_col, &width);
+			g_signal_emit_by_name (ethi->eth,
+					       "request_width",
+					       (int)ethi->resize_col, &width);
 			/* Add 10 to stop it from "..."ing */
 			e_table_header_set_size (ethi->eth, ethi->resize_col, width + 10);
 
@@ -1555,15 +1719,15 @@ ethi_event (GnomeCanvasItem *item, GdkEvent *e)
 }
 
 static void
-ethi_class_init (GtkObjectClass *object_class)
+ethi_class_init (GObjectClass *object_class)
 {
 	GnomeCanvasItemClass *item_class = (GnomeCanvasItemClass *) object_class;
 
-	ethi_parent_class = gtk_type_class (PARENT_OBJECT_TYPE);
+	ethi_parent_class = g_type_class_ref (PARENT_OBJECT_TYPE);
 	
-	object_class->destroy = ethi_destroy;
-	object_class->set_arg = ethi_set_arg;
-	object_class->get_arg = ethi_get_arg;
+	object_class->dispose = ethi_dispose;
+	object_class->set_property = ethi_set_property;
+	object_class->get_property = ethi_get_property;
 
 	item_class->update      = ethi_update;
 	item_class->realize     = ethi_realize;
@@ -1571,21 +1735,55 @@ ethi_class_init (GtkObjectClass *object_class)
 	item_class->draw        = ethi_draw;
 	item_class->point       = ethi_point;
 	item_class->event       = ethi_event;
-	
-	gtk_object_add_arg_type ("ETableHeaderItem::ETableHeader", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_TABLE_HEADER);
-	gtk_object_add_arg_type ("ETableHeaderItem::full_header", GTK_TYPE_OBJECT,
-				 GTK_ARG_READWRITE, ARG_FULL_HEADER);
-	gtk_object_add_arg_type ("ETableHeaderItem::dnd_code", GTK_TYPE_STRING,
-				 GTK_ARG_READWRITE, ARG_DND_CODE);
-	gtk_object_add_arg_type ("ETableHeaderItem::fontset", GTK_TYPE_STRING,
-				 GTK_ARG_WRITABLE, ARG_TABLE_FONTSET);
-	gtk_object_add_arg_type ("ETableHeaderItem::sort_info", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_SORT_INFO);
-	gtk_object_add_arg_type ("ETableHeaderItem::table", GTK_TYPE_OBJECT,
-				 GTK_ARG_WRITABLE, ARG_TABLE);
-	gtk_object_add_arg_type ("ETableHeaderItem::tree", E_TREE_TYPE,
-				 GTK_ARG_WRITABLE, ARG_TREE);
+
+	g_object_class_install_property (object_class, PROP_DND_CODE,
+					 g_param_spec_string ("dnd_code",
+							      _("DnD code"),
+							      /*_( */"XXX blurb" /*)*/,
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_TABLE_FONTSET,
+					 g_param_spec_string ("fontset",
+							      _("Fontset"),
+							      /*_( */"XXX blurb" /*)*/,
+							      NULL,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_FULL_HEADER,
+					 g_param_spec_object ("full_header",
+							      _("Full Header"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_HEADER_TYPE,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_TABLE_HEADER,
+					 g_param_spec_object ("ETableHeader",
+							      _("Header"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_HEADER_TYPE,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_SORT_INFO,
+					 g_param_spec_object ("sort_info",
+							      _("Sort Info"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_SORT_INFO_TYPE,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_TABLE,
+					 g_param_spec_object ("table",
+							      _("Table"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TABLE_TYPE,
+							      G_PARAM_WRITABLE));
+
+	g_object_class_install_property (object_class, PROP_TREE,
+					 g_param_spec_object ("tree",
+							      _("Tree"),
+							      /*_( */"XXX blurb" /*)*/,
+							      E_TREE_TYPE,
+							      G_PARAM_WRITABLE));
 
 	/*
 	 * Create our pixmaps for DnD
@@ -1600,14 +1798,13 @@ ethi_class_init (GtkObjectClass *object_class)
 		&add_col_mask, NULL, add_col_xpm);
 
 	ethi_signals [BUTTON_PRESSED] =
-		gtk_signal_new ("button_pressed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ETableHeaderItemClass, button_pressed),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_GDK_EVENT);
-		
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, ethi_signals, LAST_SIGNAL);
+		g_signal_new ("button_pressed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETableHeaderItemClass, button_pressed),
+			      NULL, NULL,
+			      e_marshal_NONE__BOXED,
+			      G_TYPE_NONE, 1, GDK_TYPE_EVENT);
 }
 
 static void
@@ -1635,26 +1832,9 @@ ethi_init (GnomeCanvasItem *item)
 	ethi->tree = NULL;
 }
 
-GtkType
-e_table_header_item_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"ETableHeaderItem",
-			sizeof (ETableHeaderItem),
-			sizeof (ETableHeaderItemClass),
-			(GtkClassInitFunc) ethi_class_init,
-			(GtkObjectInitFunc) ethi_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (PARENT_OBJECT_TYPE, &info);
-	}
-
-	return type;
-}
-
+E_MAKE_TYPE (e_table_header_item,
+	     "ETableHeaderItem",
+	     ETableHeaderItem,
+	     ethi_class_init,
+	     ethi_init,
+	     PARENT_OBJECT_TYPE)
