@@ -5,8 +5,8 @@
  *
  * Authors:
  *   Chris Lahey <clahey@ximian.com>
- *   Miguel de Icaza <miguel@ximian.com>
- *   Federico Mena-Quintero <federico@ximian.com>
+ *          Miguel de Icaza <miguel@ximian.com>
+ *          Federico Mena-Quintero <federico@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,39 +27,66 @@
 #include <config.h>
 #endif
 
-#include <string.h> /* strlen() */
-#include <glib.h>
-#include "e-table-defines.h"
 #include "e-table-header-utils.h"
 
+#include <string.h> /* strlen() */
+#include <glib.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkwindow.h>
+#include "e-table-defines.h"
+#include <gal/widgets/e-unicode.h>
+
 
 
+static PangoLayout*
+build_header_layout (GtkWidget *widget, const char *str)
+{
+	PangoLayout *layout;
+
+	layout = gtk_widget_create_pango_layout (widget, str);
+
+#ifdef FROB_FONT_DESC
+	{
+		PangoFontDescription *desc;
+		desc = pango_font_description_copy (gtk_widget_get_style (widget)->font_desc);
+		pango_font_description_set_size (desc,
+						 pango_font_description_get_size (desc) * 1.2);
+
+		pango_font_description_set_weight (desc, PANGO_WEIGHT_BOLD);
+		pango_layout_set_font_description (layout, desc);
+
+		pango_font_description_free (desc);
+	}
+#endif
+
+	return layout;
+}
+
 /**
  * e_table_header_compute_height:
  * @ecol: Table column description.
- * @style: Style for the button's bevel.
- * @font: Font for the button's text, or NULL if no font is available.
+ * @widget: The widget from which to build the PangoLayout.
  *
  * Computes the minimum height required for a table header button.
  *
  * Return value: The height of the button, in pixels.
  **/
 double
-e_table_header_compute_height (ETableCol *ecol, GtkStyle *style, GdkFont *font)
+e_table_header_compute_height (ETableCol *ecol, GtkWidget *widget)
 {
 	int ythick;
 	int height;
+	PangoLayout *layout;
 
 	g_return_val_if_fail (ecol != NULL, -1);
 	g_return_val_if_fail (E_IS_TABLE_COL (ecol), -1);
-	g_return_val_if_fail (style != NULL, -1);
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
 
-	ythick = style->klass->ythickness;
+	ythick = gtk_widget_get_style (widget)->ythickness;
 
-	if (font)
-		height = font->ascent + font->descent;
-	else
-		height = 16; /* FIXME: default? */
+	layout = build_header_layout (widget, ecol->text);
+
+	pango_layout_get_pixel_size (layout, NULL, &height);
 
 	if (ecol->is_pixbuf) {
 		g_assert (ecol->pixbuf != NULL);
@@ -70,6 +97,8 @@ e_table_header_compute_height (ETableCol *ecol, GtkStyle *style, GdkFont *font)
 
 	height += 2 * (ythick + HEADER_PADDING);
 
+	g_object_unref (layout);
+
 	return height;
 }
 
@@ -78,7 +107,7 @@ e_table_header_width_extras (GtkStyle *style)
 {
 	g_return_val_if_fail (style != NULL, -1);
 
-	return 2 * (style->klass->xthickness + HEADER_PADDING);
+	return 2 * (style->xthickness + HEADER_PADDING);
 }
 
 /* Creates a pixmap that is a composite of a background color and the upper-left
@@ -192,6 +221,97 @@ make_composite_pixmap (GdkDrawable *drawable, GdkGC *gc,
 	return pixmap;
 }
 
+/* Default width of the elision arrow in pixels */
+#define ARROW_WIDTH 4
+
+/**
+ * e_table_draw_elided_string:
+ * @drawable: Destination drawable.
+ * @font: Font for the text.
+ * @gc: GC to use for drawing.
+ * @x: X insertion point for the string.
+ * @y: Y insertion point for the string's baseline.
+ * @layout: the PangoLayout to draw.
+ * @str: the string we're drawing, passed in so we can change the layout if it needs eliding.
+ * @max_width: Maximum width in which the string must fit.
+ * @center: Whether to center the string in the available area if it does fit.
+ * 
+ * Draws a string, possibly trimming it so that it fits inside the specified
+ * maximum width.  If it does not fit, an elision indicator is drawn after the
+ * last character that does fit.
+ **/
+static void
+e_table_draw_elided_string (GdkDrawable *drawable, GdkGC *gc, GtkWidget *widget, 
+			    int x, int y, PangoLayout *layout, char *str,
+			    int max_width, gboolean center)
+{
+ 	int width;
+	int height;
+	int index;
+	GSList *lines;
+	PangoLayoutLine *line;
+
+	g_return_if_fail (drawable != NULL);
+	g_return_if_fail (gc != NULL);
+	g_return_if_fail (layout != NULL);
+	g_return_if_fail (max_width >= 0);
+
+	pango_layout_get_pixel_size (layout, &width, &height);
+
+	gdk_gc_set_clip_rectangle (gc, NULL);
+
+	if (width <= max_width) {
+		int xpos;
+
+		if (center)
+			xpos = x + (max_width - width) / 2;
+		else
+			xpos = x;
+
+		gdk_draw_layout (drawable, gc,
+				 xpos, y,
+				 layout);
+	} else {
+		int arrow_width;
+		int i;
+
+		if (max_width < ARROW_WIDTH + 1)
+			arrow_width = max_width - 1;
+		else
+			arrow_width = ARROW_WIDTH;
+
+
+		lines = pango_layout_get_lines (layout);
+		line = lines->data;
+
+		if (!pango_layout_line_x_to_index (line,
+						   (max_width - arrow_width) * PANGO_SCALE,
+						   &index,
+						   NULL)) {
+			g_warning ("pango_layout_line_x_to_index returned false");
+			return;
+		}
+
+		pango_layout_set_text (layout, str, index);
+
+		gdk_draw_layout (drawable, gc, x, y, layout);
+
+		for (i = 0; i < arrow_width; i++) {
+			int h;
+
+			h = 2 * i + 1;
+
+			gdk_draw_line (drawable, gc,
+				       x + max_width - i,
+				       y + height / 2 - i,
+				       x + max_width - i,
+				       y + height / 2 + i + 1);
+		}
+	}
+
+	g_object_unref (layout);
+}
+
 static GtkWidget *g_label;
 
 /**
@@ -199,10 +319,8 @@ static GtkWidget *g_label;
  * @drawable: Destination drawable.
  * @ecol: Table column for the header information.
  * @style: Style to use for drawing the button.
- * @font: Font for the button's text.
  * @state: State of the table widget.
  * @widget: The table widget.
- * @gc: GC to use for drawing.
  * @x: Leftmost coordinate of the button.
  * @y: Topmost coordinate of the button.
  * @width: Width of the region to draw.
@@ -215,7 +333,7 @@ static GtkWidget *g_label;
  **/
 void
 e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
-			    GtkStyle *style, GdkFont *font, GtkStateType state,
+			    GtkStyle *style, GtkStateType state,
 			    GtkWidget *widget,
 			    int x, int y, int width, int height,
 			    int button_width, int button_height,
@@ -225,12 +343,13 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 	int inner_x, inner_y;
 	int inner_width, inner_height;
 	GdkGC *gc;
+	char *text;
+	PangoLayout *layout;
 
 	g_return_if_fail (drawable != NULL);
 	g_return_if_fail (ecol != NULL);
 	g_return_if_fail (E_IS_TABLE_COL (ecol));
 	g_return_if_fail (style != NULL);
-	g_return_if_fail (font != NULL);
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 	g_return_if_fail (button_width > 0 && button_height > 0);
@@ -245,10 +364,12 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 		gtk_widget_ensure_style (g_label);
 	}
 
-	gc = g_label->style->fg_gc[GTK_STATE_NORMAL];
+	gc = g_label->style->fg_gc[state];
 
-	xthick = style->klass->xthickness;
-	ythick = style->klass->ythickness;
+	gdk_gc_set_clip_rectangle (gc, NULL);
+
+	xthick = style->xthickness;
+	ythick = style->ythickness;
 
 	/* Button bevel */
 
@@ -300,8 +421,9 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 	if (inner_width < 1)
 		return; /* nothing else fits */
 
-	/* Pixbuf or label */
+	layout = build_header_layout (widget, ecol->text);
 
+	/* Pixbuf or label */
 	if (ecol->is_pixbuf) {
 		int pwidth, pheight;
 		int clip_width, clip_height;
@@ -319,20 +441,22 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 		xpos = inner_x;
 
 		if (inner_width - pwidth > 11) {
-			int rbearing;
 			int width;
 			int ypos;
 
-			gdk_string_extents (font, ecol->text, NULL, &rbearing, &width, NULL, NULL);
-			if (rbearing < inner_width - (pwidth + 1)) {
+			pango_layout_get_pixel_size (layout, &width, NULL);
+
+			if (width < inner_width - (pwidth + 1)) {
 				xpos = inner_x + (inner_width - width - (pwidth + 1)) / 2;
 			}
 
-			ypos = inner_y + (inner_height - font->ascent - font->descent) / 2 + font->ascent;
+			ypos = inner_y;
 
-			e_table_draw_elided_string (drawable, font, gc,
+			e_table_draw_elided_string (drawable, gc, widget,
 						    xpos + pwidth + 1, ypos,
-						    ecol->text, inner_width - (xpos - inner_x), FALSE);
+						    layout, ecol->text, inner_width - (xpos - inner_x), FALSE);
+			
+			g_object_unref (layout);
 		}
 
 		pixmap = make_composite_pixmap (drawable, gc,
@@ -340,6 +464,9 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 						clip_width, clip_height,
 						xpos,
 						inner_y + (inner_height - clip_height) / 2);
+
+		gdk_gc_set_clip_rectangle (gc, NULL);
+
 		if (pixmap) {
 			gdk_draw_pixmap (drawable, gc, pixmap,
 					 0, 0,
@@ -349,119 +476,8 @@ e_table_header_draw_button (GdkDrawable *drawable, ETableCol *ecol,
 			gdk_pixmap_unref (pixmap);
 		}
 	} else {
-		int ypos;
-
-		ypos = inner_y + (inner_height - font->ascent - font->descent) / 2 + font->ascent;
-
-		e_table_draw_elided_string (drawable, font, gc,
-					    inner_x, ypos,
-					    ecol->text, inner_width, TRUE);
-	}
-}
-
-/* Computes the length of a string that needs to be trimmed for elision */
-static int
-compute_elision_length (GdkFont *font, const char *str, int max_width)
-{
-	int len;
-	int l = 0, left, right;
-	int rbearing;
-
-	len = strlen (str);
-
-	if (len <= 0)
-	  return 0;
-
-	left = 0;
-	right = len;
-
-	while (left < right) {
-		l = (left + right) / 2;
-		gdk_text_extents (font, str, l, NULL, &rbearing, NULL, NULL, NULL);
-
-		if (rbearing < max_width)
-			left = l + 1;
-		else if (rbearing > max_width)
-			right = l;
-		else
-			return l;
-	}
-
-	if (rbearing > max_width)
-		return MAX (0, l - 1);
-	else
-		return l;
-}
-
-/* Default width of the elision arrow in pixels */
-#define ARROW_WIDTH 4
-
-/**
- * e_table_draw_elided_string:
- * @drawable: Destination drawable.
- * @font: Font for the text.
- * @gc: GC to use for drawing.
- * @x: X insertion point for the string.
- * @y: Y insertion point for the string's baseline.
- * @str: String to draw.
- * @max_width: Maximum width in which the string must fit.
- * @center: Whether to center the string in the available area if it does fit.
- * 
- * Draws a string, possibly trimming it so that it fits inside the specified
- * maximum width.  If it does not fit, an elision indicator is drawn after the
- * last character that does fit.
- **/
-void
-e_table_draw_elided_string (GdkDrawable *drawable, GdkFont *font, GdkGC *gc,
-			    int x, int y, const char *str, int max_width, gboolean center)
-{
-	int rbearing;
-	int width;
-
-	g_return_if_fail (drawable != NULL);
-	g_return_if_fail (font != NULL);
-	g_return_if_fail (gc != NULL);
-	g_return_if_fail (str != NULL);
-	g_return_if_fail (max_width >= 0);
-
-	gdk_string_extents (font, str, NULL, &rbearing, &width, NULL, NULL);
-
-	if (rbearing <= max_width) {
-		int xpos;
-
-		if (center)
-			xpos = x + (max_width - width) / 2;
-		else
-			xpos = x;
-
-		gdk_draw_string (drawable, font, gc, xpos, y, str);
-	} else {
-		int arrow_width;
-		int len;
-		int i;
-
-		if (max_width < ARROW_WIDTH + 1)
-			arrow_width = max_width - 1;
-		else
-			arrow_width = ARROW_WIDTH;
-
-		len = compute_elision_length (font, str, max_width - arrow_width - 1);
-		gdk_draw_text (drawable, font, gc, x, y, str, len);
-
-		gdk_text_extents (font, str, len, NULL, &rbearing, NULL, NULL, NULL);
-
-		y -= font->ascent;
-
-		for (i = 0; i < arrow_width; i++) {
-			int h;
-
-			h = 2 * i + 1;
-
-			gdk_draw_line (drawable, gc,
-				       x + rbearing + arrow_width - i,
-				       y + (font->ascent + font->descent - h) / 2,
-				       x + rbearing + arrow_width - i,
-				       y + (font->ascent + font->descent - h) / 2 + h - 1);
-		}
+		e_table_draw_elided_string (drawable, gc, widget, 
+					    inner_x, inner_y,
+					    layout, ecol->text, inner_width, TRUE);
 	}
 }
