@@ -1,10 +1,10 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-tree-model.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
- *   Chris Lahey <clahey@ximian.com>
+ *   Chris Lahey  <clahey@ximian.com>
  *   Chris Toshok <toshok@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -30,19 +30,19 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
 
 #include <gtk/gtksignal.h>
 #include "gal/util/e-util.h"
 #include "gal/util/e-xml-utils.h"
 #include "e-tree-model.h"
 
-#define ETM_CLASS(e) ((ETreeModelClass *)((GtkObject *)e)->klass)
+#define ETM_CLASS(e) (E_TREE_MODEL_GET_CLASS(e))
 
-#define PARENT_TYPE (gtk_object_get_type())
+#define d(x)
 
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class;
 
 enum {
 	PRE_CHANGE,
@@ -53,6 +53,7 @@ enum {
 	NODE_INSERTED,
 	NODE_REMOVED,
 	NODE_DELETED,
+	NODE_REQUEST_COLLAPSE,
 	LAST_SIGNAL
 };
 
@@ -60,129 +61,140 @@ static guint e_tree_model_signals [LAST_SIGNAL] = {0, };
 
 
 static void
-e_tree_model_class_init (GtkObjectClass *klass)
+e_tree_model_class_init (GObjectClass *klass)
 {
 	ETreeModelClass *tree_class = (ETreeModelClass *) klass;
 
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_peek_parent (klass);
 
 	e_tree_model_signals [PRE_CHANGE] =
-		gtk_signal_new ("pre_change",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, pre_change),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+		g_signal_new ("pre_change",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, pre_change),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 
 	e_tree_model_signals [NO_CHANGE] =
-		gtk_signal_new ("no_change",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, no_change),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+		g_signal_new ("no_change",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, no_change),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 
 	e_tree_model_signals [NODE_CHANGED] =
-		gtk_signal_new ("node_changed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, node_changed),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+		g_signal_new ("node_changed",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_changed),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	e_tree_model_signals [NODE_DATA_CHANGED] =
-		gtk_signal_new ("node_data_changed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, node_data_changed),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+		g_signal_new ("node_data_changed",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_data_changed),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	e_tree_model_signals [NODE_COL_CHANGED] =
-		gtk_signal_new ("node_col_changed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, node_col_changed),
-				gtk_marshal_NONE__POINTER_INT,
-				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_INT);
+		g_signal_new ("node_col_changed",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_col_changed),
+			      (GSignalAccumulator) NULL, NULL,
+			      e_marshal_VOID__POINTER_INT,
+			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_INT);
 
 	e_tree_model_signals [NODE_INSERTED] =
-		gtk_signal_new ("node_inserted",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, node_inserted),
-				gtk_marshal_NONE__POINTER_POINTER,
-				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_POINTER);
+		g_signal_new ("node_inserted",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_inserted),
+			      (GSignalAccumulator) NULL, NULL,
+			      e_marshal_VOID__POINTER_POINTER,
+			      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
 
 	e_tree_model_signals [NODE_REMOVED] =
-		gtk_signal_new ("node_removed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, node_removed),
-				e_marshal_NONE__POINTER_POINTER_INT,
-				GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER, GTK_TYPE_INT);
+		g_signal_new ("node_removed",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_removed),
+			      (GSignalAccumulator) NULL, NULL,
+			      e_marshal_VOID__POINTER_POINTER_INT,
+			      G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_INT);
 
 	e_tree_model_signals [NODE_DELETED] =
-		gtk_signal_new ("node_deleted",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (klass),
-				GTK_SIGNAL_OFFSET (ETreeModelClass, node_deleted),
-				gtk_marshal_NONE__POINTER,
-				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+		g_signal_new ("node_deleted",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_deleted),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
-	E_OBJECT_CLASS_ADD_SIGNALS (klass, e_tree_model_signals, LAST_SIGNAL);
+	e_tree_model_signals [NODE_REQUEST_COLLAPSE] =
+		g_signal_new ("node_request_collapse",
+			      E_OBJECT_CLASS_TYPE (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETreeModelClass, node_request_collapse),
+			      (GSignalAccumulator) NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
-	tree_class->get_root             = NULL;
+	tree_class->get_root              = NULL;
 
-	tree_class->get_parent           = NULL;
-	tree_class->get_first_child      = NULL;
-	tree_class->get_last_child       = NULL;
-	tree_class->get_next             = NULL;
-	tree_class->get_prev             = NULL;
+	tree_class->get_parent            = NULL;
+	tree_class->get_first_child       = NULL;
+	tree_class->get_last_child        = NULL;
+	tree_class->get_next              = NULL;
+	tree_class->get_prev              = NULL;
 
-	tree_class->is_root              = NULL;
-	tree_class->is_expandable        = NULL;
-	tree_class->get_children         = NULL;
-	tree_class->depth                = NULL;
+	tree_class->is_root               = NULL;
+	tree_class->is_expandable         = NULL;
+	tree_class->get_children          = NULL;
+	tree_class->depth                 = NULL;
 
-	tree_class->icon_at              = NULL;
+	tree_class->icon_at               = NULL;
 
-	tree_class->get_expanded_default = NULL;
-	tree_class->column_count         = NULL;
+	tree_class->get_expanded_default  = NULL;
+	tree_class->column_count          = NULL;
 
-	tree_class->has_save_id          = NULL;
-	tree_class->get_save_id          = NULL;
-	tree_class->has_get_node_by_id   = NULL;
-	tree_class->get_node_by_id       = NULL;
+	tree_class->has_save_id           = NULL;
+	tree_class->get_save_id           = NULL;
+	tree_class->has_get_node_by_id    = NULL;
+	tree_class->get_node_by_id        = NULL;
 
-	tree_class->has_change_pending   = NULL;
+	tree_class->has_change_pending    = NULL;
 
-	tree_class->value_at             = NULL;
-	tree_class->set_value_at         = NULL;
-	tree_class->is_editable          = NULL;
+	tree_class->value_at              = NULL;
+	tree_class->set_value_at          = NULL;
+	tree_class->is_editable           = NULL;
 
-	tree_class->duplicate_value      = NULL;
-	tree_class->free_value           = NULL;
-	tree_class->initialize_value     = NULL;
-	tree_class->value_is_empty       = NULL;
-	tree_class->value_to_string      = NULL;
+	tree_class->duplicate_value       = NULL;
+	tree_class->free_value            = NULL;
+	tree_class->initialize_value      = NULL;
+	tree_class->value_is_empty        = NULL;
+	tree_class->value_to_string       = NULL;
 
-	tree_class->pre_change           = NULL;
-	tree_class->no_change            = NULL;
-	tree_class->node_changed         = NULL;
-	tree_class->node_data_changed    = NULL;
-	tree_class->node_col_changed     = NULL;
-	tree_class->node_inserted        = NULL;
-	tree_class->node_removed         = NULL;
-	tree_class->node_deleted         = NULL;
+	tree_class->pre_change            = NULL;
+	tree_class->no_change             = NULL;
+	tree_class->node_changed          = NULL;
+	tree_class->node_data_changed     = NULL;
+	tree_class->node_col_changed      = NULL;
+	tree_class->node_inserted         = NULL;
+	tree_class->node_removed          = NULL;
+	tree_class->node_deleted          = NULL;
+	tree_class->node_request_collapse = NULL;
 }
 
-static void
-e_tree_init (GtkObject *object)
-{
-}
-
-E_MAKE_TYPE(e_tree_model, "ETreeModel", ETreeModel, e_tree_model_class_init, e_tree_init, PARENT_TYPE)
+E_MAKE_TYPE(e_tree_model, "ETreeModel", ETreeModel, e_tree_model_class_init, NULL, G_TYPE_OBJECT)
 
 
 /* signals */
@@ -202,8 +214,9 @@ e_tree_model_pre_change  (ETreeModel *tree_model)
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [PRE_CHANGE]);
+	d(g_print("Emitting pre_change on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [PRE_CHANGE], 0);
 }
 
 /**
@@ -221,8 +234,9 @@ e_tree_model_no_change  (ETreeModel *tree_model)
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NO_CHANGE]);
+	d(g_print("Emitting no_change on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NO_CHANGE], 0);
 }
 
 /**
@@ -240,8 +254,9 @@ e_tree_model_node_changed  (ETreeModel *tree_model, ETreePath node)
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NODE_CHANGED], node);
+	d(g_print("Emitting node_changed on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_CHANGED], 0, node);
 }
 
 /**
@@ -259,8 +274,9 @@ e_tree_model_node_data_changed  (ETreeModel *tree_model, ETreePath node)
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NODE_DATA_CHANGED], node);
+	d(g_print("Emitting node_data_changed on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_DATA_CHANGED], 0, node);
 }
 
 /**
@@ -278,8 +294,9 @@ e_tree_model_node_col_changed  (ETreeModel *tree_model, ETreePath node, int col)
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NODE_COL_CHANGED], node, col);
+	d(g_print("Emitting node_col_changed on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_COL_CHANGED], 0, node, col);
 }
 
 /**
@@ -298,9 +315,10 @@ e_tree_model_node_inserted (ETreeModel *tree_model,
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NODE_INSERTED],
-			 parent_node, inserted_node);
+	d(g_print("Emitting node_inserted on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_INSERTED], 0,
+		       parent_node, inserted_node);
 }
 
 /**
@@ -317,9 +335,10 @@ e_tree_model_node_removed  (ETreeModel *tree_model, ETreePath parent_node, ETree
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NODE_REMOVED],
-			 parent_node, removed_node, old_position);
+	d(g_print("Emitting node_removed on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_REMOVED], 0,
+		       parent_node, removed_node, old_position);
 }
 
 /**
@@ -335,9 +354,27 @@ e_tree_model_node_deleted  (ETreeModel *tree_model, ETreePath deleted_node)
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
 	
-	gtk_signal_emit (GTK_OBJECT (tree_model),
-			 e_tree_model_signals [NODE_DELETED],
-			 deleted_node);
+	d(g_print("Emitting node_deleted on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_DELETED], 0, deleted_node);
+}
+
+/**
+ * e_tree_model_node_request_collapse:
+ * @tree_model: 
+ * @collapsed_node: 
+ * 
+ * 
+ **/
+void
+e_tree_model_node_request_collapse  (ETreeModel *tree_model, ETreePath collapsed_node)
+{
+	g_return_if_fail (tree_model != NULL);
+	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
+	
+	d(g_print("Emitting node_request_collapse on model 0x%p, a %s.\n", tree_model, gtk_type_name (GTK_OBJECT(tree_model)->klass->type)));
+
+	g_signal_emit (G_OBJECT (tree_model), e_tree_model_signals [NODE_REQUEST_COLLAPSE], 0, collapsed_node);
 }
 
 
@@ -352,11 +389,7 @@ e_tree_model_node_deleted  (ETreeModel *tree_model, ETreePath deleted_node)
 ETreeModel *
 e_tree_model_new ()
 {
-	ETreeModel *et;
-
-	et = gtk_type_new (e_tree_model_get_type ());
-
-	return et;
+	return (ETreeModel *) g_object_new (E_TREE_MODEL_TYPE, NULL);
 }
 
 /**
@@ -516,6 +549,7 @@ gboolean
 e_tree_model_node_is_expandable (ETreeModel *etree, ETreePath node)
 {
 	g_return_val_if_fail(etree != NULL, FALSE);
+	g_return_val_if_fail(node != NULL, FALSE);
 
 	if (ETM_CLASS(etree)->is_expandable)
 		return ETM_CLASS(etree)->is_expandable(etree, node);
@@ -721,14 +755,24 @@ e_tree_model_has_change_pending (ETreeModel *etree)
 }
 
 /**
- * e_tree_model_icon_of_node
+ * e_tree_model_value_at:
  * @etree: The ETreeModel.
- * @path: The ETreePath to the node we're getting the icon of.
+ * @node: The ETreePath to the node we're getting the data from.
+ * @col: the column to retrieve data from
+ * 
+ * Return value: This function returns the value that is stored by the
+ * @etree in column @col and node @node.  The data returned can be a
+ * pointer or any data value that can be stored inside a pointer.
  *
- * XXX docs here.
+ * The data returned is typically used by an ECell renderer.
  *
- * return values: the GdkPixbuf associated with this node.
- */
+ * The data returned must be valid until the model sends a signal that
+ * affect that piece of data.  node_changed and node_deleted affect
+ * all data in tha t node and all nodes under that node.
+ * node_data_changed affects the data in that node.  node_col_changed
+ * affects the data in that node for that column.  node_inserted,
+ * node_removed, and no_change don't affect any data in this way.
+ **/
 void *
 e_tree_model_value_at (ETreeModel *etree, ETreePath node, int col)
 {
@@ -943,3 +987,112 @@ e_tree_model_node_traverse_preorder (ETreeModel *model, ETreePath path, ETreePat
 	}
 }
 
+/**
+ * e_tree_model_node_traverse_preorder:
+ * @model: 
+ * @path: 
+ * @func: 
+ * @data: 
+ * 
+ * 
+ **/
+static ETreePath
+e_tree_model_node_real_traverse (ETreeModel *model, ETreePath path, ETreePath end_path, gboolean forward_direction, ETreePathFunc func, gpointer data)
+{
+	ETreePath child;
+
+	g_return_val_if_fail (model != NULL, NULL);
+	g_return_val_if_fail (E_IS_TREE_MODEL (model), NULL);
+	g_return_val_if_fail (path != NULL, NULL);
+
+	if (forward_direction)
+		child = e_tree_model_node_get_first_child (model, path);
+	else
+		child = e_tree_model_node_get_last_child (model, path);
+
+	while (child) {
+		ETreePath result;
+
+		if (forward_direction && (child == end_path || func (model, child, data)))
+			return child;
+
+		if ((result = e_tree_model_node_real_traverse (model, child, end_path, forward_direction, func, data)))
+			return result;
+
+		if (!forward_direction && (child == end_path || func (model, child, data)))
+			return child;
+
+		if (forward_direction)
+			child = e_tree_model_node_get_next (model, child);
+		else
+			child = e_tree_model_node_get_prev (model, child);
+	}
+	return NULL;
+}
+
+/**
+ * e_tree_model_node_traverse_preorder:
+ * @model: 
+ * @path: 
+ * @func: 
+ * @data: 
+ * 
+ * 
+ **/
+ETreePath
+e_tree_model_node_find (ETreeModel *model, ETreePath path, ETreePath end_path, gboolean forward_direction, ETreePathFunc func, gpointer data)
+{
+	ETreePath result;
+	ETreePath next;
+
+	g_return_val_if_fail (model != NULL, NULL);
+	g_return_val_if_fail (E_IS_TREE_MODEL (model), NULL);
+
+	/* Just search the whole tree in this case. */
+	if (path == NULL) {
+		ETreePath root;
+		root = e_tree_model_get_root (model);
+
+		if (forward_direction && (end_path == root || func (model, root, data)))
+			return root;
+
+		if ((result = e_tree_model_node_real_traverse (model, root, end_path, forward_direction, func, data)))
+			return result;
+
+		if (!forward_direction && (end_path == root || func (model, root, data)))
+			return root;
+
+		return NULL;
+	}
+
+	while (1) {
+
+		if (forward_direction) {
+			if ((result = e_tree_model_node_real_traverse (model, path, end_path, forward_direction, func, data)))
+				return result;
+			next = e_tree_model_node_get_next (model, path);
+		} else {
+			next = e_tree_model_node_get_prev (model, path);
+			if (next && (result = e_tree_model_node_real_traverse (model, next, end_path, forward_direction, func, data)))
+				return result;
+		}
+
+		while (next == NULL) {
+			path = e_tree_model_node_get_parent (model, path);
+
+			if (path == NULL)
+				return NULL;
+
+			if (forward_direction)
+				next = e_tree_model_node_get_next (model, path);
+			else
+				next = path;
+		}
+
+		if (end_path == next || func (model, next, data))
+			return next;
+
+		path = next;
+	}
+}
+
