@@ -24,10 +24,10 @@
 #ifndef _E_TREE_H_
 #define _E_TREE_H_
 
+#include <gtk/gtkdnd.h>
 #include <gtk/gtktable.h>
-#include <gnome-xml/tree.h>
-#include <libgnomeui/gnome-canvas.h>
-
+#include <libxml/tree.h>
+#include <libgnomecanvas/gnome-canvas.h>
 #include <gal/widgets/e-printable.h>
 
 #include <gal/e-table/e-table-extras.h>
@@ -35,7 +35,6 @@
 #include <gal/e-table/e-table-state.h>
 #include <gal/e-table/e-tree-model.h>
 #include <gal/e-table/e-tree-table-adapter.h>
-#include <libgnome/gnome-defs.h>
 
 #define E_TREE_USE_TREE_SELECTION
 
@@ -43,13 +42,13 @@
 #include <gal/e-table/e-tree-selection-model.h>
 #endif
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define E_TREE_TYPE        (e_tree_get_type ())
-#define E_TREE(o)          (GTK_CHECK_CAST ((o), E_TREE_TYPE, ETree))
-#define E_TREE_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TREE_TYPE, ETreeClass))
-#define E_IS_TREE(o)       (GTK_CHECK_TYPE ((o), E_TREE_TYPE))
-#define E_IS_TREE_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), E_TREE_TYPE))
+#define E_TREE(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), E_TREE_TYPE, ETree))
+#define E_TREE_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST((k), E_TREE_TYPE, ETreeClass))
+#define E_IS_TREE(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), E_TREE_TYPE))
+#define E_IS_TREE_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), E_TREE_TYPE))
 typedef struct _ETreeDragSourceSite ETreeDragSourceSite;
 typedef struct ETreePriv ETreePriv;
 
@@ -137,7 +136,7 @@ typedef struct {
 						 guint               time);
 } ETreeClass;
 
-GtkType         e_tree_get_type                   (void);
+GType           e_tree_get_type                   (void);
 ETree          *e_tree_construct                  (ETree                *e_tree,
 						   ETreeModel           *etm,
 						   ETableExtras         *ete,
@@ -167,6 +166,8 @@ ETableState    *e_tree_get_state_object           (ETree                *e_tree)
 ETableSpecification    *e_tree_get_spec           (ETree                *e_tree);
 
 /* note that it is more efficient to provide the state at creation time */
+void            e_tree_set_search_column          (ETree *e_tree,
+						   gint  col);
 void            e_tree_set_state                  (ETree                *e_tree,
 						   const gchar          *state);
 void            e_tree_set_state_object           (ETree                *e_tree,
@@ -303,7 +304,7 @@ gboolean        e_tree_find_next                  (ETree                *et,
 void            e_tree_right_click_up             (ETree                *et);
 
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif /* _E_TREE_H_ */
 
