@@ -1,6 +1,25 @@
-/*
-  ECompleteTest
-*/
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-completion-test.c
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Jon Trowbridge <trow@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
 
 #include <gnome.h>
 #include "e-completion.h"
@@ -169,28 +188,28 @@ main (int argc, gchar **argv)
   read_dict ();
 
   complete = e_completion_new ();
-  gtk_signal_connect (GTK_OBJECT (complete),
-		      "request_completion",
-		      GTK_SIGNAL_FUNC (request_dict_search),
-		      NULL);
-  gtk_signal_connect (GTK_OBJECT (complete),
-		      "end_completion",
-		      GTK_SIGNAL_FUNC (end_dict_search),
-		      NULL);
-  gtk_signal_connect (GTK_OBJECT (complete),
-		      "cancel_completion",
-		      GTK_SIGNAL_FUNC (end_dict_search),
-		      NULL);
+  g_signal_connect (complete,
+		    "request_completion",
+		    G_CALLBACK (request_dict_search),
+		    NULL);
+  g_signal_connect (complete,
+		    "end_completion",
+		    G_CALLBACK (end_dict_search),
+		    NULL);
+  g_signal_connect (complete,
+		    "cancel_completion",
+		    G_CALLBACK (end_dict_search),
+		    NULL);
 
   win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   entry = e_entry_new ();
   e_entry_enable_completion_full (E_ENTRY (entry), complete, 0, NULL);
   e_entry_set_editable (E_ENTRY (entry), TRUE);
 
-  gtk_signal_connect (GTK_OBJECT (entry),
-		      "popup",
-		      GTK_SIGNAL_FUNC (popup_cb),
-		      NULL);
+  g_signal_connect (entry,
+		    "popup",
+		    G_CALLBACK (popup_cb),
+		    NULL);
 
   gtk_container_add (GTK_CONTAINER (win), entry);
   gtk_widget_show_all (win);
