@@ -25,18 +25,17 @@
 #ifndef E_COMPLETION_VIEW_H
 #define E_COMPLETION_VIEW_H
 
-#include <libgnome/gnome-defs.h>
 #include <gtk/gtk.h>
 #include <gal/e-table/e-table.h>
 #include "e-completion.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define E_COMPLETION_VIEW_TYPE        (e_completion_view_get_type ())
-#define E_COMPLETION_VIEW(o)          (GTK_CHECK_CAST ((o), E_COMPLETION_VIEW_TYPE, ECompletionView))
-#define E_COMPLETION_VIEW_CLASS(k)    (GTK_CHECK_CLASS_CAST ((k), E_COMPLETION_VIEW_TYPE, ECompletionViewClass))
-#define E_IS_COMPLETION_VIEW(o)       (GTK_CHECK_TYPE ((o), E_COMPLETION_VIEW_TYPE))
-#define E_IS_COMPLETION_VIEW_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), E_COMPLETION_VIEW_TYPE))
+#define E_COMPLETION_VIEW(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), E_COMPLETION_VIEW_TYPE, ECompletionView))
+#define E_COMPLETION_VIEW_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST ((k), E_COMPLETION_VIEW_TYPE, ECompletionViewClass))
+#define E_IS_COMPLETION_VIEW(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), E_COMPLETION_VIEW_TYPE))
+#define E_IS_COMPLETION_VIEW_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), E_COMPLETION_VIEW_TYPE))
 
 typedef struct _ECompletionView ECompletionView;
 typedef struct _ECompletionViewClass ECompletionViewClass;
@@ -97,7 +96,7 @@ void       e_completion_view_set_uncomplete_key (ECompletionView *cv, gint keyva
 void       e_completion_view_set_width    (ECompletionView *cv, gint width);
 void       e_completion_view_set_editable (ECompletionView *cv, gboolean);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 
 #endif /* E_COMPLETION_H */
