@@ -1,20 +1,25 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* ETextModel - Text item model for evolution.
- * Copyright (C) 2000 Helix Code, Inc.
+/* 
+ * e-text-model.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author: Chris Lahey <clahey@umich.edu>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * A majority of code taken from:
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * Text item type for GnomeCanvas widget
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * GnomeCanvas is basically a port of the Tk toolkit's most excellent
- * canvas widget.  Tk is copyrighted by the Regents of the University
- * of California, Sun Microsystems, and other parties.
- *
- * Copyright (C) 1998 The Free Software Foundation
- *
- * Author: Federico Mena <federico@nuclecu.unam.mx> */
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
 
 #undef  PARANOID_DEBUGGING
 
@@ -22,12 +27,11 @@
 #include <ctype.h>
 #include <string.h>
 #include <gtk/gtksignal.h>
+#include <gal/util/e-util.h>
 #include "e-text-model-repos.h"
 #include "e-text-model.h"
 #include "gal/util/e-util.h"
 
-#define CLASS(obj) (E_TEXT_MODEL_CLASS (GTK_OBJECT (obj)->klass))
-
 #define MAX_LENGTH (2047)
 
 enum {
@@ -47,7 +51,7 @@ struct _ETextModelPrivate {
 
 static void e_text_model_class_init (ETextModelClass *class);
 static void e_text_model_init       (ETextModel *model);
-static void e_text_model_destroy    (GtkObject *object);
+static void e_text_model_dispose    (GObject *object);
 
 static gint         e_text_model_real_validate_position (ETextModel *, gint pos);
 static const gchar *e_text_model_real_get_text          (ETextModel *model);
@@ -57,7 +61,8 @@ static void         e_text_model_real_insert            (ETextModel *model, gint
 static void         e_text_model_real_insert_length     (ETextModel *model, gint postion, const gchar *text, gint length);
 static void         e_text_model_real_delete            (ETextModel *model, gint postion, gint length);
 
-static GtkObject *parent_class;
+#define PARENT_TYPE G_TYPE_OBJECT
+static GObject *parent_class;
 
 
 
@@ -70,74 +75,60 @@ static GtkObject *parent_class;
  * 
  * Return value: The type ID of the &ETextModel class.
  **/
-GtkType
-e_text_model_get_type (void)
-{
-	static GtkType model_type = 0;
-
-	if (!model_type) {
-		GtkTypeInfo model_info = {
-			"ETextModel",
-			sizeof (ETextModel),
-			sizeof (ETextModelClass),
-			(GtkClassInitFunc) e_text_model_class_init,
-			(GtkObjectInitFunc) e_text_model_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		model_type = gtk_type_unique (gtk_object_get_type (), &model_info);
-	}
-
-	return model_type;
-}
+E_MAKE_TYPE (e_text_model,
+	     "ETextModel",
+	     ETextModel,
+	     e_text_model_class_init,
+	     e_text_model_init,
+	     PARENT_TYPE)
 
 /* Class initialization function for the text item */
 static void
 e_text_model_class_init (ETextModelClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 
-	object_class = (GtkObjectClass *) klass;
+	object_class = (GObjectClass *) klass;
 
-	parent_class = gtk_type_class (gtk_object_get_type ());
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
 	e_text_model_signals[E_TEXT_MODEL_CHANGED] =
-		gtk_signal_new ("changed",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ETextModelClass, changed),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
+		g_signal_new ("changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETextModelClass, changed),
+			      NULL, NULL,
+			      e_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 
 	e_text_model_signals[E_TEXT_MODEL_REPOSITION] =
-		gtk_signal_new ("reposition",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ETextModelClass, reposition),
-				gtk_marshal_NONE__POINTER_POINTER,
-				GTK_TYPE_NONE, 2,
-				GTK_TYPE_POINTER, GTK_TYPE_POINTER);
+		g_signal_new ("reposition",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETextModelClass, reposition),
+			      NULL, NULL,
+			      e_marshal_NONE__POINTER_POINTER,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_POINTER, G_TYPE_POINTER);
 
 	e_text_model_signals[E_TEXT_MODEL_OBJECT_ACTIVATED] =
-		gtk_signal_new ("object_activated",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ETextModelClass, object_activated),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1,
-				GTK_TYPE_INT);
+		g_signal_new ("object_activated",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETextModelClass, object_activated),
+			      NULL, NULL,
+			      e_marshal_NONE__INT,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_INT);
 
 	e_text_model_signals[E_TEXT_MODEL_CANCEL_COMPLETION] =
-		gtk_signal_new ("cancel_completion",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ETextModelClass, cancel_completion),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-	
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_text_model_signals, E_TEXT_MODEL_LAST_SIGNAL);
+		g_signal_new ("cancel_completion",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ETextModelClass, cancel_completion),
+			      NULL, NULL,
+			      e_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 	
 	/* No default signal handlers. */
 	klass->changed          = NULL;
@@ -158,7 +149,7 @@ e_text_model_class_init (ETextModelClass *klass)
 	klass->obj_count        = NULL;
 	klass->get_nth_obj      = NULL;
 	
-	object_class->destroy = e_text_model_destroy;
+	object_class->dispose = e_text_model_dispose;
 }
 
 /* Object initialization function for the text item */
@@ -170,9 +161,9 @@ e_text_model_init (ETextModel *model)
 	model->priv->len  = 0;
 }
 
-/* Destroy handler for the text item */
+/* Dispose handler for the text item */
 static void
-e_text_model_destroy (GtkObject *object)
+e_text_model_dispose (GObject *object)
 {
 	ETextModel *model;
 
@@ -181,13 +172,15 @@ e_text_model_destroy (GtkObject *object)
 
 	model = E_TEXT_MODEL (object);
 
-	g_free (model->priv->text);
+	if (model->priv) {
+		g_free (model->priv->text);
 
-	g_free (model->priv);
-	model->priv = NULL;
+		g_free (model->priv);
+		model->priv = NULL;
+	}
 
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static gint
@@ -359,11 +352,11 @@ e_text_model_changed (ETextModel *model)
 	  While this method could, in theory, do pretty much anything, it is meant
 	  for scanning objects and converting substrings into embedded objects.
 	*/
-	if (CLASS (model)->objectify)
-		CLASS (model)->objectify (model);
+	if (E_TEXT_MODEL_GET_CLASS (model)->objectify)
+		E_TEXT_MODEL_GET_CLASS (model)->objectify (model);
 
-	gtk_signal_emit (GTK_OBJECT (model),
-			 e_text_model_signals[E_TEXT_MODEL_CHANGED]);
+	g_signal_emit (model,
+		       e_text_model_signals[E_TEXT_MODEL_CHANGED], 0);
 }
 
 void
@@ -371,7 +364,7 @@ e_text_model_cancel_completion (ETextModel *model)
 {
 	g_return_if_fail (E_IS_TEXT_MODEL (model));
 	
-	gtk_signal_emit (GTK_OBJECT (model), e_text_model_signals[E_TEXT_MODEL_CANCEL_COMPLETION]);
+	g_signal_emit (model, e_text_model_signals[E_TEXT_MODEL_CANCEL_COMPLETION], 0);
 }
 
 void
@@ -381,9 +374,9 @@ e_text_model_reposition (ETextModel *model, ETextModelReposFn fn, gpointer repos
 	g_return_if_fail (E_IS_TEXT_MODEL (model));
 	g_return_if_fail (fn != NULL);
 
-	gtk_signal_emit (GTK_OBJECT (model),
-			 e_text_model_signals[E_TEXT_MODEL_REPOSITION],
-			 fn, repos_data);
+	g_signal_emit (model,
+		       e_text_model_signals[E_TEXT_MODEL_REPOSITION], 0,
+		       fn, repos_data);
 }
 
 gint
@@ -392,8 +385,8 @@ e_text_model_validate_position (ETextModel *model, gint pos)
 	g_return_val_if_fail (model != NULL, 0);
 	g_return_val_if_fail (E_IS_TEXT_MODEL (model), 0);
 
-	if (CLASS (model)->validate_pos)
-		pos = CLASS (model)->validate_pos (model, pos);
+	if (E_TEXT_MODEL_GET_CLASS (model)->validate_pos)
+		pos = E_TEXT_MODEL_GET_CLASS (model)->validate_pos (model, pos);
 
 	return pos;
 }
@@ -404,8 +397,8 @@ e_text_model_get_text (ETextModel *model)
 	g_return_val_if_fail (model != NULL, NULL);
 	g_return_val_if_fail (E_IS_TEXT_MODEL (model), NULL);
 
-	if (CLASS (model)->get_text)
-		return CLASS (model)->get_text (model);
+	if (E_TEXT_MODEL_GET_CLASS (model)->get_text)
+		return E_TEXT_MODEL_GET_CLASS (model)->get_text (model);
 
 	return "";
 }
@@ -416,9 +409,9 @@ e_text_model_get_text_length (ETextModel *model)
 	g_return_val_if_fail (model != NULL, 0);
 	g_return_val_if_fail (E_IS_TEXT_MODEL (model), 0);
 
-	if (CLASS (model)->get_text_len (model)) {
+	if (E_TEXT_MODEL_GET_CLASS (model)->get_text_len (model)) {
 
-		gint len = CLASS (model)->get_text_len (model);
+		gint len = E_TEXT_MODEL_GET_CLASS (model)->get_text_len (model);
 
 #ifdef PARANOID_DEBUGGING
 		const gchar *str = e_text_model_get_text (model);
@@ -442,8 +435,8 @@ e_text_model_set_text (ETextModel *model, const gchar *text)
 	g_return_if_fail (model != NULL);
 	g_return_if_fail (E_IS_TEXT_MODEL (model));
 
-	if (CLASS (model)->set_text)
-		CLASS (model)->set_text (model, text);
+	if (E_TEXT_MODEL_GET_CLASS (model)->set_text)
+		E_TEXT_MODEL_GET_CLASS (model)->set_text (model, text);
 }
 
 void
@@ -455,8 +448,8 @@ e_text_model_insert (ETextModel *model, gint position, const gchar *text)
 	if (text == NULL)
 		return;
 
-	if (CLASS (model)->insert)
-		CLASS (model)->insert (model, position, text);
+	if (E_TEXT_MODEL_GET_CLASS (model)->insert)
+		E_TEXT_MODEL_GET_CLASS (model)->insert (model, position, text);
 }
 
 void
@@ -470,8 +463,8 @@ e_text_model_insert_length (ETextModel *model, gint position, const gchar *text,
 	if (text == NULL || length == 0)
 		return;
 
-	if (CLASS (model)->insert_length)
-		CLASS (model)->insert_length (model, position, text, length);
+	if (E_TEXT_MODEL_GET_CLASS (model)->insert_length)
+		E_TEXT_MODEL_GET_CLASS (model)->insert_length (model, position, text, length);
 }
 
 void
@@ -514,8 +507,8 @@ e_text_model_delete (ETextModel *model, gint position, gint length)
 	if (length <= 0)
 		return;
 
-	if (CLASS (model)->delete)
-		CLASS (model)->delete (model, position, length);
+	if (E_TEXT_MODEL_GET_CLASS (model)->delete)
+		E_TEXT_MODEL_GET_CLASS (model)->delete (model, position, length);
 }
 
 gint
@@ -524,8 +517,8 @@ e_text_model_object_count (ETextModel *model)
 	g_return_val_if_fail (model != NULL, 0);
 	g_return_val_if_fail (E_IS_TEXT_MODEL (model), 0);
 
-	if (CLASS (model)->obj_count)
-		return CLASS (model)->obj_count (model);
+	if (E_TEXT_MODEL_GET_CLASS (model)->obj_count)
+		return E_TEXT_MODEL_GET_CLASS (model)->obj_count (model);
 
 	return 0;
 }
@@ -539,8 +532,8 @@ e_text_model_get_nth_object (ETextModel *model, gint n, gint *len)
 	if (n < 0 || n >= e_text_model_object_count (model))
 		return NULL;
 
-	if (CLASS (model)->get_nth_obj)
-		return CLASS (model)->get_nth_obj (model, n, len);
+	if (E_TEXT_MODEL_GET_CLASS (model)->get_nth_obj)
+		return E_TEXT_MODEL_GET_CLASS (model)->get_nth_obj (model, n, len);
 
 	return NULL;
 }
@@ -589,9 +582,9 @@ e_text_model_get_object_at_offset (ETextModel *model, gint offset)
 		return -1;
 
 	/* If an optimized version has been provided, we use it. */
-	if (CLASS (model)->obj_at_offset) {
+	if (E_TEXT_MODEL_GET_CLASS (model)->obj_at_offset) {
 
-		return CLASS (model)->obj_at_offset (model, offset);
+		return E_TEXT_MODEL_GET_CLASS (model)->obj_at_offset (model, offset);
 
 	} else { 
 		/* If not, we fake it.*/
@@ -629,12 +622,12 @@ e_text_model_activate_nth_object (ETextModel *model, gint n)
 	g_return_if_fail (n >= 0);
 	g_return_if_fail (n < e_text_model_object_count (model));
 
-	gtk_signal_emit (GTK_OBJECT (model), e_text_model_signals[E_TEXT_MODEL_OBJECT_ACTIVATED], n);
+	g_signal_emit (model, e_text_model_signals[E_TEXT_MODEL_OBJECT_ACTIVATED], 0, n);
 }
 
 ETextModel *
 e_text_model_new (void)
 {
-	ETextModel *model = gtk_type_new (e_text_model_get_type ());
+	ETextModel *model = g_object_new (E_TYPE_TEXT_MODEL, NULL);
 	return model;
 }
