@@ -38,14 +38,13 @@
 #define E_TEXT_H
 
 #include <gtk/gtkobject.h>
-#include <gtk/gtkpacker.h>
 
 #include <gal/widgets/e-font.h>
 #include <gal/util/e-text-event-processor.h>
 #include <gal/e-text/e-text-model.h>
 #include <gal/widgets/e-canvas.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 
 /* Text item for the canvas.  Text items are positioned by an anchor point and an anchor direction.
@@ -63,9 +62,6 @@ BEGIN_GNOME_DECLS
  * name			type			read/write	description
  * ------------------------------------------------------------------------------------------
  * text			string			RW		The string of the text label
- * font			string			W		X logical font descriptor
- * fontset		string			W		X logical fontset descriptor
- * font_gdk		GdkFont*		RW		Pointer to a GdkFont
  * bold                 boolean                 RW              Bold?
  * anchor		GtkAnchorType		RW		Anchor side for the text
  * justification	GtkJustification	RW		Justification for multiline text
@@ -96,37 +92,15 @@ BEGIN_GNOME_DECLS
  */
 
 #define E_TYPE_TEXT            (e_text_get_type ())
-#define E_TEXT(obj)            (GTK_CHECK_CAST ((obj), E_TYPE_TEXT, EText))
-#define E_TEXT_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), E_TYPE_TEXT, ETextClass))
-#define E_IS_TEXT(obj)         (GTK_CHECK_TYPE ((obj), E_TYPE_TEXT))
-#define E_IS_TEXT_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), E_TYPE_TEXT))
+#define E_TEXT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), E_TYPE_TEXT, EText))
+#define E_TEXT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), E_TYPE_TEXT, ETextClass))
+#define E_IS_TEXT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), E_TYPE_TEXT))
+#define E_IS_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), E_TYPE_TEXT))
 
 
 typedef struct _EText EText;
 typedef struct _ETextClass ETextClass;
 
-#if 0
-typedef struct _ETextSuckFont ETextSuckFont;
-typedef struct _ETextSuckChar ETextSuckChar;
-
-struct _ETextSuckChar {
-	int     left_sb;
-	int     right_sb;
-	int     width;
-	int     ascent;
-	int     descent;
-	int     bitmap_offset; /* in pixels */
-};
-
-struct _ETextSuckFont {
-	guchar *bitmap;
-	gint    bitmap_width;
-	gint    bitmap_height;
-	gint    ascent;
-	ETextSuckChar chars[256];
-};
-#endif
-
 struct _EText {
 	GnomeCanvasItem item;
 	
@@ -135,16 +109,11 @@ struct _EText {
 	gint model_repos_signal_id;
 
 	const gchar *text;              /* Text to display --- from the ETextModel */
-	gpointer lines;			/* Text split into lines (private field) */
+	PangoLayout *layout;
 	int num_lines;			/* Number of lines of text */
 
 	gchar *revert;                  /* Text to revert to */
 
-#if 0
-	GdkFont *font;			/* Font for text */
-#else
-	EFont *font;
-#endif
 	GtkAnchorType anchor;		/* Anchor side for text */
 	GtkJustification justification;	/* Justification for text */
 
@@ -158,16 +127,13 @@ struct _EText {
 	GdkGC *gc;			/* GC for drawing text */
 
 	int cx, cy;			/* Top-left canvas coordinates for text */
+	int text_cx, text_cy;		/* Top-left canvas coordinates for text */
 	int clip_cx, clip_cy;		/* Top-left canvas coordinates for clip rectangle */
 	int clip_cwidth, clip_cheight;	/* Size of clip rectangle in pixels */
 	int max_width;			/* Maximum width of text lines */
 	int width;                      /* Rendered text width in pixels */
 	int height;			/* Rendered text height in pixels */
 
-	/* Antialiased specific stuff follows */
-#if 0
-	ETextSuckFont *suckfont; /* Sucked font */
-#endif
 	guint32 rgba;			/* RGBA color for text */
 	double affine[6];               /* The item -> canvas affine */
 
@@ -217,8 +183,8 @@ struct _EText {
 	guint needs_redraw : 1;         /* Needs redraw */
 	guint needs_recalc_bounds : 1;  /* Need recalc_bounds */
 	guint needs_calc_height : 1;    /* Need calc_height */
-	guint needs_calc_line_widths : 1; /* Needs calc_line_widths */
 	guint needs_split_into_lines : 1; /* Needs split_into_lines */
+	guint needs_reset_layout : 1; /* Needs split_into_lines */
 
 	guint bold : 1;
 	guint strikeout : 1;
@@ -248,17 +214,18 @@ struct _EText {
 
 	gint     last_type_request;       /* Last selection type requested. */
 	guint32  last_time_request;       /* The time of the last selection request. */
-	guint32  last_selection_request;  /* The time of the last selection request. */
+	GdkAtom  last_selection_request;  /* The time of the last selection request. */
 	GList   *queued_requests;         /* Queued selection requests. */
 };
 
 struct _ETextClass {
 	GnomeCanvasItemClass parent_class;
 
-	void (* changed)   (EText *text);
-	void (* activate)  (EText *text);
-	void (* keypress)  (EText *text, guint keyval, guint state);
-	void (* popup)     (EText *text, GdkEventButton *ev, gint pos);
+	void (* changed)    (EText *text);
+	void (* activate)   (EText *text);
+	void (* keypress)   (EText *text, guint keyval, guint state);
+	void (* popup)      (EText *text, GdkEventButton *ev, gint pos);
+	void (* style_set)  (EText *text, GtkStyle *previous_style);
 };
 
 
@@ -267,6 +234,6 @@ GtkType  e_text_get_type        (void);
 void     e_text_cancel_editing  (EText *text);
 void     e_text_stop_editing    (EText *text);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
