@@ -1,25 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
 /* 
- * Author : 
- *  Damon Chaplin <damon@helixcode.com>
+ * e-group-bar.c
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Copyright 1999, Helix Code, Inc.
+ * Authors:
+ *   Damon Chaplin <damon@ximian.com>
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 /*
@@ -36,6 +35,7 @@
 
 #include <config.h>
 #include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
 #include <math.h>
 #include <gnome.h>
 #include "e-group-bar.h"
@@ -48,7 +48,7 @@
 
 static void e_group_bar_class_init (EGroupBarClass *class);
 static void e_group_bar_init (EGroupBar *group_bar);
-static void e_group_bar_destroy (GtkObject *object);
+static void e_group_bar_dispose (GObject *object);
 static void e_group_bar_realize (GtkWidget *widget);
 static void e_group_bar_unrealize (GtkWidget *widget);
 static void e_group_bar_map (GtkWidget *widget);
@@ -57,10 +57,6 @@ static void e_group_bar_size_request (GtkWidget      *widget,
 				      GtkRequisition *requisition);
 static void e_group_bar_size_allocate (GtkWidget *widget,
 				       GtkAllocation *allocation);
-static gint e_group_bar_expose (GtkWidget      *widget,
-				GdkEventExpose *event);
-static void e_group_bar_draw (GtkWidget    *widget,
-			      GdkRectangle *area);
 static void e_group_bar_add (GtkContainer *container,
 			     GtkWidget    *widget);
 static void e_group_bar_remove (GtkContainer     *container,
@@ -107,48 +103,32 @@ static gboolean e_group_bar_auto_show (gpointer data);
 static void e_group_bar_stop_all_animation (EGroupBar	*group_bar);
 
 
+#define PARENT_TYPE GTK_TYPE_CONTAINER
 static GtkContainerClass *parent_class;
 
 
-GtkType
-e_group_bar_get_type (void)
-{
-	static GtkType e_group_bar_type = 0;
-
-	if (!e_group_bar_type){
-		GtkTypeInfo e_group_bar_info = {
-			"EGroupBar",
-			sizeof (EGroupBar),
-			sizeof (EGroupBarClass),
-			(GtkClassInitFunc) e_group_bar_class_init,
-			(GtkObjectInitFunc) e_group_bar_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		parent_class = gtk_type_class (GTK_TYPE_CONTAINER);
-		e_group_bar_type = gtk_type_unique (GTK_TYPE_CONTAINER,
-						    &e_group_bar_info);
-	}
-
-	return e_group_bar_type;
-}
-
+E_MAKE_TYPE (e_group_bar,
+	     "EGroupBar",
+	     EGroupBar,
+	     e_group_bar_class_init,
+	     e_group_bar_init,
+	     PARENT_TYPE)
 
 static void
 e_group_bar_class_init (EGroupBarClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GtkObjectClass *) class;
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
 
+	parent_class = g_type_class_ref (PARENT_TYPE);
+
 	/* Method override */
-	object_class->destroy		= e_group_bar_destroy;
+	object_class->dispose		= e_group_bar_dispose;
 
 	widget_class->realize		= e_group_bar_realize;
 	widget_class->unrealize		= e_group_bar_unrealize;
@@ -156,8 +136,6 @@ e_group_bar_class_init (EGroupBarClass *class)
 	widget_class->unmap		= e_group_bar_unmap;
 	widget_class->size_request	= e_group_bar_size_request;
  	widget_class->size_allocate	= e_group_bar_size_allocate;
-	widget_class->expose_event	= e_group_bar_expose;
-	widget_class->draw		= e_group_bar_draw;
 
 	container_class->add		= e_group_bar_add;
 	container_class->remove		= e_group_bar_remove;
@@ -196,14 +174,14 @@ e_group_bar_new (void)
 {
 	GtkWidget *group_bar;
 
-	group_bar = GTK_WIDGET (gtk_type_new (e_group_bar_get_type ()));
+	group_bar = GTK_WIDGET (g_object_new (E_GROUP_BAR_TYPE, NULL));
 
 	return group_bar;
 }
 
 
 static void
-e_group_bar_destroy (GtkObject *object)
+e_group_bar_dispose (GObject *object)
 {
 	EGroupBar *group_bar;
 
@@ -214,9 +192,11 @@ e_group_bar_destroy (GtkObject *object)
 	/* The parent GtkContainer class will automatically destroy all the
 	   child widgets, but it calls gtk_container_foreach() so we must not
 	   destroy our children array until after. */
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 
-	g_array_free (group_bar->children, TRUE);
+	if (group_bar->children)
+	    g_array_free (group_bar->children, TRUE);
+	group_bar->children = NULL;
 }
 
 
@@ -522,6 +502,7 @@ e_group_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 				gdk_window_move_resize (group->button_window,
 							0, y, width, button_allocation.height);
 			}
+			gdk_window_raise (group->button_window);
 		}
 		gtk_widget_size_allocate (group->button, &button_allocation);
 
@@ -530,10 +511,13 @@ e_group_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 				gdk_window_resize (group->child_window,
 						   width, child_height);
 				group->child_window_target_y = y + button_allocation.height;
+				gdk_window_raise (group->child_window);
 			} else {
 				gdk_window_move_resize (group->child_window,
 							0, y + button_allocation.height,
 							width, child_height);
+				if (group_num == group_bar->current_group_num)
+					gdk_window_raise (group->child_window);
 			}
 		}
 		gtk_widget_size_allocate (group->child, &child_allocation);
@@ -541,81 +525,6 @@ e_group_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 }
 
 
-static gint
-e_group_bar_expose (GtkWidget      *widget,
-		    GdkEventExpose *event)
-{
-	EGroupBar *group_bar;
-	EGroupBarChild *group;
-	GdkEventExpose child_event;
-	gint group_num;
-
-	g_return_val_if_fail (widget != NULL, FALSE);
-	g_return_val_if_fail (E_IS_GROUP_BAR (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
-
-	if (GTK_WIDGET_DRAWABLE (widget)) {
-		group_bar = E_GROUP_BAR (widget);
-
-		child_event = *event;
-
-		for (group_num = 0;
-		     group_num < group_bar->children->len;
-		     group_num++) {
-			group = &g_array_index (group_bar->children,
-						EGroupBarChild, group_num);
-
-			if (event->window == group->button_window
-			    && GTK_WIDGET_DRAWABLE (group->button)
-			    && GTK_WIDGET_NO_WINDOW (group->button)
-			    && gtk_widget_intersect (group->button, &event->area, &child_event.area))
-				gtk_widget_event (group->button, (GdkEvent*) &child_event);
-
-			if (event->window == group->child_window
-			    && GTK_WIDGET_DRAWABLE (group->child)
-			    && GTK_WIDGET_NO_WINDOW (group->child)
-			    && gtk_widget_intersect (group->child, &event->area, &child_event.area))
-				gtk_widget_event (group->child, (GdkEvent*) &child_event);
-		}
-	}
-
-	return FALSE;
-}
-
-
-static void
-e_group_bar_draw (GtkWidget    *widget,
-		  GdkRectangle *area)
-{
-	EGroupBar *group_bar;
-	EGroupBarChild *group;
-	gint group_num;
-	GdkRectangle child_area;
-
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (E_IS_GROUP_BAR (widget));
-   
-	if (GTK_WIDGET_DRAWABLE (widget)) {
-		group_bar = E_GROUP_BAR (widget);
-	
-		for (group_num = 0;
-		     group_num < group_bar->children->len;
-		     group_num++) {
-			group = &g_array_index (group_bar->children,
-						EGroupBarChild, group_num);
-
-			if (GTK_WIDGET_DRAWABLE (group->button)
-			    && gtk_widget_intersect (group->button, area, &child_area))
-				gtk_widget_draw (group->button, &child_area);
-
-			if (GTK_WIDGET_DRAWABLE (group->child)
-			    && gtk_widget_intersect (group->child, area, &child_area))
-				gtk_widget_draw (group->child, &child_area);
-		}
-	}
-}
-
-
 static void
 e_group_bar_add (GtkContainer *container,
 		 GtkWidget    *widget)
@@ -674,6 +583,9 @@ e_group_bar_forall (GtkContainer   *container,
 
 	group_bar = E_GROUP_BAR (container);
 
+	if (group_bar->children == NULL)
+		return;
+
 	/* Note that drag-and-drop does not check the Z-order of widgets, so
 	   we have to iterate through them from top to bottom, or it will
 	   not work properly. We also have to use temporary lists so widgets
@@ -986,17 +898,23 @@ e_group_bar_add_group		(EGroupBar	*group_bar,
 		}
 	}
 
-	gtk_signal_connect (GTK_OBJECT (group->button), "clicked",
-			    GTK_SIGNAL_FUNC (e_group_bar_on_button_clicked),
-			    group_bar);
+	g_signal_connect (group->button, "clicked",
+			  G_CALLBACK (e_group_bar_on_button_clicked),
+			  group_bar);
+	
+	g_signal_connect (group->button, "drag_leave",
+			  G_CALLBACK (e_group_bar_on_button_drag_leave),
+			  group_bar);
 
-	gtk_signal_connect (GTK_OBJECT (group->button), "drag_motion",
-			    GTK_SIGNAL_FUNC (e_group_bar_on_button_drag_motion),
-			    group_bar);
-	gtk_signal_connect (GTK_OBJECT (group->button), "drag_leave",
-			    GTK_SIGNAL_FUNC (e_group_bar_on_button_drag_leave),
-			    group_bar);
+	/* Set up the button as a drop target.  */
 
+	gtk_drag_dest_set (GTK_WIDGET (group->button),
+			   0, NULL, 0,
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
+
+	g_signal_connect (group->button, "drag_motion",
+			  G_CALLBACK (e_group_bar_on_button_drag_motion),
+			  group_bar);
 	return group_num;
 }
 
@@ -1490,7 +1408,8 @@ e_group_bar_on_button_drag_motion (GtkWidget	      *widget,
 			group_bar->auto_show_group_num = group_num;
 		}
 	}
-	return TRUE;
+
+	return FALSE;
 }
 
 
@@ -1545,13 +1464,15 @@ e_group_bar_stop_all_animation	(EGroupBar	*group_bar)
 		group_bar->auto_show_timeout_id = 0;
 	}
 
-	for (group_num = 0;
-	     group_num < group_bar->children->len;
-	     group_num++) {
-		group = &g_array_index (group_bar->children,
-					EGroupBarChild, group_num);
-		group->button_window_in_animation = FALSE;
-		group->child_window_in_animation = FALSE;
+	if (group_bar->children != NULL) {
+		for (group_num = 0;
+		     group_num < group_bar->children->len;
+		     group_num++) {
+			group = &g_array_index (group_bar->children,
+						EGroupBarChild, group_num);
+			group->button_window_in_animation = FALSE;
+			group->child_window_in_animation = FALSE;
+		}
 	}
 }
 
