@@ -30,15 +30,16 @@
 
 #include "e-icon-bar.h"
 
+#include <gtk/gtkdnd.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkwindow.h>
 #include <gtk/gtksignal.h>
-#include <gdk-pixbuf/gnome-canvas-pixbuf.h>
 #include "e-icon-bar-bg-item.h"
+#include <gal/util/e-util.h>
 #include <gal/e-text/e-text.h>
 #include <gal/widgets/e-canvas-utils.h>
-#include <libgnomeui/gnome-canvas-rect-ellipse.h>
-#include "gal/util/e-util.h"
+#include <libgnomecanvas/gnome-canvas-pixbuf.h>
+#include <libgnomecanvas/gnome-canvas-rect-ellipse.h>
 
 /* These are the offsets of the icons & text in both views. Note that the
    shadow around icons is drawn in the space between items (as is the
@@ -77,7 +78,7 @@
 
 static void e_icon_bar_class_init (EIconBarClass *class);
 static void e_icon_bar_init (EIconBar *icon_bar);
-static void e_icon_bar_destroy (GtkObject *object);
+static void e_icon_bar_dispose (GObject *object);
 static void e_icon_bar_size_allocate (GtkWidget *widget,
 				      GtkAllocation *allocation);
 static gint e_icon_bar_leave_notify_event (GtkWidget *widget,
@@ -142,70 +143,50 @@ enum
 
 static guint e_icon_bar_signals[LAST_SIGNAL] = {0};
 
+#define PARENT_TYPE E_CANVAS_TYPE
 static ECanvasClass *parent_class;
 
-
-GtkType
-e_icon_bar_get_type (void)
-{
-	static GtkType e_icon_bar_type = 0;
-
-	if (!e_icon_bar_type){
-		GtkTypeInfo e_icon_bar_info = {
-			"EIconBar",
-			sizeof (EIconBar),
-			sizeof (EIconBarClass),
-			(GtkClassInitFunc) e_icon_bar_class_init,
-			(GtkObjectInitFunc) e_icon_bar_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		parent_class = gtk_type_class (e_canvas_get_type ());
-		e_icon_bar_type = gtk_type_unique (e_canvas_get_type (),
-						   &e_icon_bar_info);
-	}
-
-	return e_icon_bar_type;
-}
-
+E_MAKE_TYPE (e_icon_bar,
+	     "EIconBar",
+	     EIconBar,
+	     e_icon_bar_class_init,
+	     e_icon_bar_init,
+	     PARENT_TYPE)
 
 static void
 e_icon_bar_class_init (EIconBarClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	ECanvasClass *ecanvas_class;
 
-	object_class = (GtkObjectClass *) class;
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	ecanvas_class = E_CANVAS_CLASS( class );
 
-	e_icon_bar_signals[ITEM_SELECTED] =
-		gtk_signal_new ("item_selected",
-				GTK_RUN_LAST | GTK_RUN_ACTION,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EIconBarClass,
-						   selected_item),
-				gtk_marshal_NONE__POINTER_INT,
-				GTK_TYPE_NONE, 2, GTK_TYPE_GDK_EVENT,
-				GTK_TYPE_INT);
-	e_icon_bar_signals[ITEM_DRAGGED] =
-		gtk_signal_new ("item_dragged",
-				GTK_RUN_LAST | GTK_RUN_ACTION,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EIconBarClass,
-						   dragged_item),
-				gtk_marshal_NONE__POINTER_INT,
-				GTK_TYPE_NONE, 2, GTK_TYPE_GDK_EVENT,
-				GTK_TYPE_INT);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_icon_bar_signals,
-				      LAST_SIGNAL);
+	e_icon_bar_signals[ITEM_SELECTED] =
+		g_signal_new ("item_selected",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EIconBarClass, selected_item),
+			      NULL, NULL,
+			      e_marshal_NONE__BOXED_INT,
+			      G_TYPE_NONE, 2, GDK_TYPE_EVENT,
+			      G_TYPE_INT);
+	e_icon_bar_signals[ITEM_DRAGGED] =
+		g_signal_new ("item_dragged",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EIconBarClass, dragged_item),
+			      NULL, NULL,
+			      e_marshal_NONE__BOXED_INT,
+			      G_TYPE_NONE, 2, GDK_TYPE_EVENT,
+			      G_TYPE_INT);
 
 	/* Method override */
-	object_class->destroy		 = e_icon_bar_destroy;
+	object_class->dispose		 = e_icon_bar_dispose;
 
  	widget_class->size_allocate	 = e_icon_bar_size_allocate;
  	widget_class->leave_notify_event = e_icon_bar_leave_notify_event;
@@ -270,7 +251,11 @@ e_icon_bar_init (EIconBar *icon_bar)
 					     E_ICON_BAR_COLOR_LAST, FALSE,
 					     TRUE, success);
 	if (nfailed)
-		g_warning ("Failed to allocate all colors");
+		g_warning ("EIconBar failed to allocate all colors");
+
+	gtk_drag_dest_set (GTK_WIDGET (icon_bar),
+			   0, NULL, 0,
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 }
 
 
@@ -285,14 +270,14 @@ e_icon_bar_new (void)
 {
 	GtkWidget *icon_bar;
 
-	icon_bar = GTK_WIDGET (gtk_type_new (e_icon_bar_get_type ()));
+	icon_bar = GTK_WIDGET (g_object_new (E_ICON_BAR_TYPE, NULL));
 
 	return icon_bar;
 }
 
 
 static void
-e_icon_bar_destroy (GtkObject *object)
+e_icon_bar_dispose (GObject *object)
 {
 	EIconBar *icon_bar;
 	int item_num;
@@ -300,21 +285,24 @@ e_icon_bar_destroy (GtkObject *object)
 
 	icon_bar = E_ICON_BAR (object);
 
-	for (item_num = 0; item_num < icon_bar->items->len; item_num++) {
-		item = &g_array_index (icon_bar->items,
-				       EIconBarItem, item_num);
-		if (item->destroy)
-			item->destroy (item->data);
+	if (icon_bar->items) {
+		for (item_num = 0; item_num < icon_bar->items->len; item_num++) {
+			item = &g_array_index (icon_bar->items,
+					       EIconBarItem, item_num);
+			if (item->destroy)
+				item->destroy (item->data);
+		}
+		
+		g_array_free (icon_bar->items, TRUE);
+		icon_bar->items = NULL;
 	}
 
-	g_array_free (icon_bar->items, TRUE);
-
 	if (icon_bar->auto_scroll_timeout_id != 0) {
 		gtk_timeout_remove (icon_bar->auto_scroll_timeout_id);
 		icon_bar->auto_scroll_timeout_id = 0;
 	}
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -323,9 +311,7 @@ e_icon_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
 	EIconBar *icon_bar;
 	gint canvas_width, canvas_height, height;
-#if 0
-	g_print ("In e_icon_bar_size_allocate\n");
-#endif
+
 	icon_bar = E_ICON_BAR (widget);
 
 	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
@@ -347,7 +333,10 @@ e_icon_bar_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 
 	GTK_LAYOUT (widget)->vadjustment->step_increment = 16;
 	if (icon_bar->vadjustment_value_changed_id == 0)
-		icon_bar->vadjustment_value_changed_id = gtk_signal_connect (GTK_OBJECT (GTK_LAYOUT (widget)->vadjustment), "value_changed", GTK_SIGNAL_FUNC (e_icon_bar_vadjustment_value_changed), icon_bar);
+		icon_bar->vadjustment_value_changed_id = g_signal_connect (GTK_LAYOUT (widget)->vadjustment,
+									   "value_changed",
+									   G_CALLBACK (e_icon_bar_vadjustment_value_changed),
+									   icon_bar);
 
 	e_icon_bar_update_highlight (icon_bar);
 }
@@ -404,14 +393,14 @@ e_icon_bar_recalc_item_positions (EIconBar *icon_bar)
 		max_lines = 2;
 		text_x = icon_bar->text_x + (icon_bar->text_w / 2);
 		line_wrap = TRUE;
-		font = GTK_WIDGET (icon_bar)->style->font;
+		font = gtk_style_get_font (GTK_WIDGET (icon_bar)->style);
 		text_h = font->ascent + font->descent;
 	} else {
 		justify = GTK_JUSTIFY_LEFT;
 		anchor = GTK_ANCHOR_NW;
 		max_lines = 1;
 		text_x = icon_bar->text_x;
-		font = GTK_WIDGET (icon_bar)->style->font;
+		font = gtk_style_get_font (GTK_WIDGET (icon_bar)->style);
 		text_h = font->ascent + font->descent;
 		line_wrap = FALSE;
 	}
@@ -593,7 +582,6 @@ e_icon_bar_style_set (GtkWidget *widget, GtkStyle *previous_style) {
 				       "GnomeCanvasPixbuf::pixbuf", flattened ? flattened : item->pixbuf,
 				       NULL);
 		gnome_canvas_item_set (item->text,
-				       "font_gdk", style->font,
 				       "fill_color_gdk", &style->fg [GTK_STATE_NORMAL],
 				       NULL);
 
@@ -650,11 +638,10 @@ e_icon_bar_add_item (EIconBar	    *icon_bar,
 
 	gtk_widget_ensure_style (GTK_WIDGET (icon_bar));
 	style = GTK_WIDGET (icon_bar)->style;
-	font = style->font;
+	font = gtk_style_get_font (style);
 	clip_height = max_lines * (font->ascent + font->descent);
 	item.text = gnome_canvas_item_new (GNOME_CANVAS_GROUP (GNOME_CANVAS (icon_bar)->root),
 					   e_text_get_type (),
-					   "font_gdk", font,
 					   "fill_color_gdk", &style->fg [GTK_STATE_NORMAL],
 					   "use_ellipsis", TRUE,
 					   "anchor", anchor,
@@ -672,9 +659,9 @@ e_icon_bar_add_item (EIconBar	    *icon_bar,
 	e_canvas_item_move_absolute(item.text,
 				    text_x, 0);
 
-	gtk_signal_connect (GTK_OBJECT (item.text), "event",
-			    GTK_SIGNAL_FUNC (e_icon_bar_on_item_event),
-			    icon_bar);
+	g_signal_connect (item.text, "event",
+			  G_CALLBACK (e_icon_bar_on_item_event),
+			  icon_bar);
 
 	item.pixbuf = image;
 	if (image)
@@ -695,9 +682,9 @@ e_icon_bar_add_item (EIconBar	    *icon_bar,
 		/* the canvas item holds the reference now */
 		gdk_pixbuf_unref (flattened);
 
-	gtk_signal_connect (GTK_OBJECT (item.image), "event",
-			    GTK_SIGNAL_FUNC (e_icon_bar_on_item_event),
-			    icon_bar);
+	g_signal_connect (item.image, "event",
+			  G_CALLBACK (e_icon_bar_on_item_event),
+			  icon_bar);
 
 	item.data = NULL;
 	item.destroy = NULL;
@@ -807,9 +794,9 @@ e_icon_bar_get_item_image	(EIconBar	  *icon_bar,
 	g_return_val_if_fail (item_num < icon_bar->items->len, NULL);
 
 	item = &g_array_index (icon_bar->items, EIconBarItem, item_num);
-	gtk_object_get (GTK_OBJECT (item->image),
-			"GnomeCanvasPixbuf::pixbuf", &image,
-			NULL);
+	g_object_get (item->image,
+		      "GnomeCanvasPixbuf::pixbuf", &image,
+		      NULL);
 	return image;
 }
 
@@ -870,9 +857,9 @@ e_icon_bar_get_item_text	(EIconBar	  *icon_bar,
 	g_return_val_if_fail (item_num < icon_bar->items->len, NULL);
 
 	item = &g_array_index (icon_bar->items, EIconBarItem, item_num);
-	gtk_object_get (GTK_OBJECT (item->text),
-			"EText::text", &text,
-			NULL);
+	g_object_get (item->text,
+		      "EText::text", &text,
+		      NULL);
 
 	return text;
 }
@@ -980,7 +967,6 @@ static void
 e_icon_bar_reflow (ECanvas *canvas)
 {
 #if 0
-	g_print ("In e_icon_bar_on_canvas_reflow\n");
 	gtk_widget_queue_resize (GTK_WIDGET (canvas));
 #endif
 }
@@ -1030,8 +1016,8 @@ e_icon_bar_on_item_event (GnomeCanvasItem *item,
 		if (icon_bar->editing_item_num == -1
 		    || icon_bar->editing_item_num != item_num) {
 			e_icon_bar_item_pressed (icon_bar, item_num, event);
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (item),
-						      "event");
+			g_signal_stop_emission_by_name (item,
+							"event");
 			return TRUE;
 		}
 		break;
@@ -1045,8 +1031,8 @@ e_icon_bar_on_item_event (GnomeCanvasItem *item,
 		if (icon_bar->editing_item_num == -1
 		    || icon_bar->editing_item_num != item_num) {
 			e_icon_bar_item_released (icon_bar, item_num, event);
-			gtk_signal_emit_stop_by_name (GTK_OBJECT (item),
-						      "event");
+			g_signal_stop_emission_by_name (item,
+							"event");
 			return TRUE;
 		}
 		break;
@@ -1093,9 +1079,6 @@ e_icon_bar_item_pressed (EIconBar *icon_bar,
 
 	if (button == 1 && item_num != -1) {
 		icon_bar->mouse_over_item_num = item_num;
-#if 0
-		g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
-#endif
 		icon_bar->pressed_item_num = item_num;
 		icon_bar->pressed_x = event->button.x;
 		icon_bar->pressed_y = event->button.y;
@@ -1108,9 +1091,9 @@ e_icon_bar_item_pressed (EIconBar *icon_bar,
 				  GDK_BUTTON1_MOTION_MASK,
 				  NULL, NULL, event->button.time);
 	} else if (button == 3) {
-		gtk_signal_emit (GTK_OBJECT (icon_bar),
-				 e_icon_bar_signals[ITEM_SELECTED],
-				 event, item_num);
+		g_signal_emit (icon_bar,
+			       e_icon_bar_signals[ITEM_SELECTED], 0,
+			       event, item_num);
 	}
 }
 
@@ -1133,9 +1116,9 @@ e_icon_bar_item_released (EIconBar *icon_bar,
 
 		if (icon_bar->pressed_item_num != -1
 		    && icon_bar->pressed_item_num == icon_bar->mouse_over_item_num) {
-			gtk_signal_emit (GTK_OBJECT (icon_bar),
-					 e_icon_bar_signals[ITEM_SELECTED],
-					 event, item_num);
+			g_signal_emit (icon_bar,
+				       e_icon_bar_signals[ITEM_SELECTED], 0,
+				       event, item_num);
 		}
 
 		icon_bar->pressed_item_num = -1;
@@ -1151,10 +1134,6 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 {
 	gboolean need_redraw = TRUE;
 
-#if 0
-	g_print ("In e_icon_bar_item_motion\n");
-#endif
-
 	if (event && event->motion.state & GDK_BUTTON1_MASK
 	    && icon_bar->pressed_item_num != -1) {
 		if (icon_bar->enable_drags
@@ -1168,9 +1147,9 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 			   a pointer grab. */
 			gdk_pointer_ungrab (event->motion.time);
 
-			gtk_signal_emit (GTK_OBJECT (icon_bar),
-					 e_icon_bar_signals[ITEM_DRAGGED],
-					 event, icon_bar->dragged_item_num);
+			g_signal_emit (icon_bar,
+				       e_icon_bar_signals[ITEM_DRAGGED], 0,
+				       event, icon_bar->dragged_item_num);
 			
 			/* Don't show the button as pressed while dragging. */
 			icon_bar->pressed_item_num = -1;
@@ -1193,9 +1172,6 @@ e_icon_bar_item_motion (EIconBar *icon_bar,
 	}
 
 	icon_bar->mouse_over_item_num = item_num;
-#if 0
-	g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
-#endif
 
 	if (need_redraw)
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
@@ -1553,7 +1529,9 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 			guint           time)
 {
 	EIconBar *icon_bar;
-	gint item_num, before_item, scroll_x, scroll_y;
+	GdkDragAction action;
+	gint item_num, before_item;
+	gint scroll_x, scroll_y;
 
 	g_return_val_if_fail (E_IS_ICON_BAR (widget), FALSE);
 
@@ -1571,6 +1549,8 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 	e_icon_bar_item_motion (icon_bar, item_num, NULL);
 	e_icon_bar_set_dragging_before_item (icon_bar, before_item);
 
+	action = 0;
+
 	/* Check if the mouse is at the top or bottom of the bar, and if it is
 	   scroll up/down. */
 	if (y < E_ICON_BAR_DRAG_AUTO_SCROLL_OFFSET)
@@ -1582,7 +1562,7 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 			gtk_timeout_remove (icon_bar->auto_scroll_timeout_id);
 			icon_bar->auto_scroll_timeout_id = 0;
 		}
-		return FALSE;
+		return TRUE;
 	}
 
 	if (icon_bar->auto_scroll_timeout_id == 0) {
@@ -1590,7 +1570,7 @@ e_icon_bar_drag_motion (GtkWidget      *widget,
 		icon_bar->auto_scroll_delay = E_ICON_BAR_SCROLL_DELAY;
 	}
 
-	return FALSE;
+	return TRUE;
 }
 
 
@@ -1614,9 +1594,6 @@ e_icon_bar_drag_leave (GtkWidget      *widget,
 
 	if (icon_bar->mouse_over_item_num != -1) {
 		icon_bar->mouse_over_item_num = -1;
-#if 0
-		g_print ("Set mouse_over_item_num: %i\n", icon_bar->mouse_over_item_num);
-#endif
 		gtk_widget_queue_draw (GTK_WIDGET (icon_bar));
 	}
 }
