@@ -39,7 +39,6 @@
 #include <gtk/gtkrc.h>
 #include <gtk/gtkselection.h>
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-util.h>
 #include <gal/util/e-util.h>
 #include <gal/e-text/e-entry.h>
@@ -61,12 +60,14 @@ gchar	  *e_shortcut_bar_default_icon_filename = "gnome-folder.png";
 
 static void e_shortcut_bar_class_init		(EShortcutBarClass *class);
 static void e_shortcut_bar_init			(EShortcutBar	*shortcut_bar);
-static void e_shortcut_bar_destroy		(GtkObject	*object);
+static void e_shortcut_bar_dispose		(GObject	*object);
+static void e_shortcut_bar_finalize		(GObject	*object);
 
-static void e_shortcut_bar_disconnect_model	(EShortcutBar	*shortcut_bar);
+static void e_shortcut_bar_disconnect_model	(EShortcutBar	*shortcut_bar,
+						 gboolean        model_destroyed);
 
-static void e_shortcut_bar_on_model_destroyed	(EShortcutModel	*model,
-						 EShortcutBar	*shortcut_bar);
+static void e_shortcut_bar_on_model_destroyed	(void           *data,
+						 GObject        *where_the_model_was);
 static void e_shortcut_bar_on_group_added	(EShortcutModel	*model,
 						 gint		 group_num,
 						 gchar		*group_name,
@@ -130,6 +131,17 @@ static void e_shortcut_bar_on_drag_data_get	(GtkWidget      *widget,
 						 guint           info,
 						 guint           time,
 						 EShortcutBar	*shortcut_bar);
+static gboolean e_shortcut_bar_on_drag_motion 	(GtkWidget      *widget,
+					      	 GdkDragContext *context,
+					      	 gint            x,
+					      	 gint            y,
+					      	 guint           time,
+					      	 EShortcutBar    *shortcut_bar);
+static gboolean e_shortcut_bar_on_drag_drop     (GtkWidget	 *widget,
+						 GdkDragContext *context,
+						 gint            x,
+						 gint            y,
+						 guint           time);
 static void e_shortcut_bar_on_drag_data_received(GtkWidget      *widget,
 						 GdkDragContext *context,
 						 gint            x,
@@ -153,6 +165,8 @@ enum
   ITEM_SELECTED,
   SHORTCUT_DROPPED,
   SHORTCUT_DRAGGED,
+  SHORTCUT_DRAG_MOTION,
+  SHORTCUT_DRAG_DATA_RECEIVED,
   LAST_SIGNAL
 };
 
@@ -160,75 +174,87 @@ static guint e_shortcut_bar_signals[LAST_SIGNAL] = {0};
 
 static EGroupBarClass *parent_class;
 
-static void
-e_shortcut_bar_marshal_NONE__INT_INT_STRING_STRING (GtkObject *object,
-						    GtkSignalFunc func,
-						    gpointer func_data,
-						    GtkArg *args)
-{
-	void (*rfunc) (GtkObject *, gint, gint, gchar *, gchar *, gpointer) = func;
-
-	(*rfunc) (object,
-		  GTK_VALUE_INT (args[0]),
-		  GTK_VALUE_INT (args[1]),
-		  GTK_VALUE_STRING (args[2]),
-		  GTK_VALUE_STRING (args[3]),
-		  func_data);
-}
-
 E_MAKE_TYPE(e_shortcut_bar, "EShortcutBar", EShortcutBar,
 	    e_shortcut_bar_class_init, e_shortcut_bar_init,
 	    e_group_bar_get_type())
 
-
 static void
 e_shortcut_bar_class_init (EShortcutBarClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 
-	parent_class = gtk_type_class (e_group_bar_get_type ());
+	parent_class = g_type_class_ref (E_GROUP_BAR_TYPE);
 
-	object_class = (GtkObjectClass *) class;
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 
 	e_shortcut_bar_signals[ITEM_SELECTED] =
-		gtk_signal_new ("item_selected",
-				GTK_RUN_LAST | GTK_RUN_ACTION,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EShortcutBarClass,
-						   item_selected),
-				gtk_marshal_NONE__POINTER_INT_INT,
-				GTK_TYPE_NONE, 3, GTK_TYPE_GDK_EVENT,
-				GTK_TYPE_INT, GTK_TYPE_INT);
+		g_signal_new ("item_selected",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EShortcutBarClass, item_selected),
+			      NULL, NULL,
+			      e_marshal_NONE__BOXED_INT_INT,
+			      G_TYPE_NONE, 3, GDK_TYPE_EVENT,
+			      G_TYPE_INT, G_TYPE_INT);
 
 	e_shortcut_bar_signals[SHORTCUT_DROPPED] =
-		gtk_signal_new ("shortcut_dropped",
-				GTK_RUN_LAST | GTK_RUN_ACTION,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EShortcutBarClass, shortcut_dropped),
-				e_shortcut_bar_marshal_NONE__INT_INT_STRING_STRING,
-				GTK_TYPE_NONE, 4,
-				GTK_TYPE_INT,
-				GTK_TYPE_INT,
-				GTK_TYPE_STRING,
-				GTK_TYPE_STRING);
+		g_signal_new ("shortcut_dropped",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EShortcutBarClass, shortcut_dropped),
+			      NULL, NULL,
+			      e_marshal_NONE__INT_INT_STRING_STRING,
+			      G_TYPE_NONE, 4,
+			      G_TYPE_INT,
+			      G_TYPE_INT,
+			      G_TYPE_STRING,
+			      G_TYPE_STRING);
 
 	e_shortcut_bar_signals[SHORTCUT_DRAGGED] =
-		gtk_signal_new ("shortcut_dragged",
-				GTK_RUN_LAST | GTK_RUN_ACTION,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EShortcutBarClass, shortcut_dragged),
-				gtk_marshal_NONE__INT_INT,
-				GTK_TYPE_NONE, 2,
-				GTK_TYPE_INT,
-				GTK_TYPE_INT);
+		g_signal_new ("shortcut_dragged",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EShortcutBarClass, shortcut_dragged),
+			      NULL, NULL,
+			      e_marshal_NONE__INT_INT,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
 
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_shortcut_bar_signals,
-				      LAST_SIGNAL);
+	e_shortcut_bar_signals[SHORTCUT_DRAG_MOTION] =
+		g_signal_new ("shortcut_drag_motion",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EShortcutBarClass, shortcut_drag_motion),
+			      NULL, NULL,
+			      e_marshal_BOOLEAN__POINTER_POINTER_INT_INT_INT,
+			      G_TYPE_BOOLEAN, 5,
+			      G_TYPE_POINTER,
+			      G_TYPE_POINTER,
+			      G_TYPE_INT,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
+
+	e_shortcut_bar_signals[SHORTCUT_DRAG_DATA_RECEIVED] =
+		g_signal_new ("shortcut_drag_data_received",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EShortcutBarClass, shortcut_drag_data_received),
+			      NULL, NULL,
+			      e_marshal_BOOLEAN__POINTER_POINTER_POINTER_INT_INT_INT,
+			      G_TYPE_BOOLEAN, 6,
+			      G_TYPE_POINTER,
+			      G_TYPE_POINTER,
+			      G_TYPE_POINTER,
+			      G_TYPE_INT,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
 
 	/* Method override */
-	object_class->destroy		= e_shortcut_bar_destroy;
+	object_class->dispose  = e_shortcut_bar_dispose;
+	object_class->finalize = e_shortcut_bar_finalize;
 }
 
 
@@ -251,32 +277,41 @@ e_shortcut_bar_new (void)
 	GtkWidget *shortcut_bar;
 
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-	gtk_widget_push_visual (gdk_rgb_get_visual ());
 
-	shortcut_bar = GTK_WIDGET (gtk_type_new (e_shortcut_bar_get_type ()));
+	shortcut_bar = GTK_WIDGET (g_object_new (E_TYPE_SHORTCUT_BAR, NULL));
 
-	gtk_widget_pop_visual ();
 	gtk_widget_pop_colormap ();
 
 	return shortcut_bar;
 }
 
-
 static void
-e_shortcut_bar_destroy (GtkObject *object)
+e_shortcut_bar_dispose (GObject *object)
 {
 	EShortcutBar *shortcut_bar;
 
 	shortcut_bar = E_SHORTCUT_BAR (object);
 
-	e_shortcut_bar_disconnect_model (shortcut_bar);
+	e_shortcut_bar_disconnect_model (shortcut_bar, FALSE);
 
-	g_array_free (shortcut_bar->groups, TRUE);
+	(* G_OBJECT_CLASS (parent_class)->dispose) (object);
+}
+
+static void
+e_shortcut_bar_finalize (GObject *object)
+{
+	EShortcutBar *shortcut_bar;
+
+	shortcut_bar = E_SHORTCUT_BAR (object);
+
+	if (shortcut_bar->groups)
+		g_array_free (shortcut_bar->groups, TRUE);
 
 	g_free (shortcut_bar->dragged_url);
+
 	g_free (shortcut_bar->dragged_name);
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 
@@ -289,31 +324,30 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 	GdkPixbuf *item_image;
 
 	/* Disconnect any existing model. */
-	e_shortcut_bar_disconnect_model (shortcut_bar);
+	e_shortcut_bar_disconnect_model (shortcut_bar, FALSE);
 
 	shortcut_bar->model = model;
 
 	if (!model)
 		return;
 
-	gtk_signal_connect (GTK_OBJECT (model), "destroy",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_model_destroyed),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (model), "group_added",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_group_added),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (model), "group_removed",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_group_removed),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (model), "item_added",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_item_added),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (model), "item_removed",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_item_removed),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (model), "item_updated",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_item_updated),
-			    shortcut_bar);
+	g_object_weak_ref (G_OBJECT (model), e_shortcut_bar_on_model_destroyed, shortcut_bar);
+
+	g_signal_connect (model, "group_added",
+			  G_CALLBACK (e_shortcut_bar_on_group_added),
+			  shortcut_bar);
+	g_signal_connect (model, "group_removed",
+			  G_CALLBACK (e_shortcut_bar_on_group_removed),
+			  shortcut_bar);
+	g_signal_connect (model, "item_added",
+			  G_CALLBACK (e_shortcut_bar_on_item_added),
+			  shortcut_bar);
+	g_signal_connect (model, "item_removed",
+			  G_CALLBACK (e_shortcut_bar_on_item_removed),
+			  shortcut_bar);
+	g_signal_connect (model, "item_updated",
+			  G_CALLBACK (e_shortcut_bar_on_item_updated),
+			  shortcut_bar);
 
 	/* Add any items already in the model. */
 	num_groups = e_shortcut_model_get_num_groups (model);
@@ -341,25 +375,35 @@ e_shortcut_bar_set_model (EShortcutBar *shortcut_bar,
 
 
 static void
-e_shortcut_bar_disconnect_model (EShortcutBar *shortcut_bar)
+e_shortcut_bar_disconnect_model (EShortcutBar *shortcut_bar,
+				 gboolean model_destroyed)
 {
 	/* Remove all the current groups. */
 	while (shortcut_bar->groups->len)
 		e_shortcut_bar_remove_group (shortcut_bar, 0);
 
-	if (shortcut_bar->model) {
+	if (! model_destroyed && shortcut_bar->model != NULL) {
 		/* Disconnect all the signals in one go. */
-		gtk_signal_disconnect_by_data (GTK_OBJECT (shortcut_bar->model), shortcut_bar);
-		shortcut_bar->model = NULL;
+		g_signal_handlers_disconnect_matched (shortcut_bar->model,
+						      G_SIGNAL_MATCH_DATA,
+						      0, 0, NULL, NULL,
+						      shortcut_bar);
+
+		g_object_weak_unref (G_OBJECT (shortcut_bar->model),
+				     e_shortcut_bar_on_model_destroyed, shortcut_bar);
 	}
+
+	shortcut_bar->model = NULL;
 }
 
 
 static void
-e_shortcut_bar_on_model_destroyed	(EShortcutModel	*model,
-					 EShortcutBar	*shortcut_bar)
+e_shortcut_bar_on_model_destroyed	(void *data,
+					 GObject *where_the_model_was)
 {
-	e_shortcut_bar_disconnect_model (shortcut_bar);
+	EShortcutBar *shortcut_bar = E_SHORTCUT_BAR (data);
+
+	e_shortcut_bar_disconnect_model (shortcut_bar, TRUE);
 }
 
 
@@ -433,8 +477,9 @@ e_shortcut_bar_group_button_press (GtkWidget *widget,
 
 		child = &g_array_index (group_bar->children, EGroupBarChild, i);
 		if (widget == child->button) {
-			gtk_signal_emit (GTK_OBJECT (shortcut_bar), e_shortcut_bar_signals[ITEM_SELECTED],
-					 button_event, i, -1);
+			g_signal_emit (shortcut_bar,
+				       e_shortcut_bar_signals[ITEM_SELECTED], 0,
+				       button_event, i, -1);
 			break;
 		}
 	}
@@ -453,7 +498,6 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 	g_return_val_if_fail (group_name != NULL, -1);
 
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-	gtk_widget_push_visual (gdk_rgb_get_visual ());
 
 	group_num = position;
 	g_array_insert_val (shortcut_bar->groups, group_num, tmp_group);
@@ -463,12 +507,10 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 
 	group->vscrolled_bar = e_vscrolled_bar_new (NULL);
 	gtk_widget_show (group->vscrolled_bar);
-	gtk_signal_connect (
-		GTK_OBJECT (E_VSCROLLED_BAR (group->vscrolled_bar)->up_button),
-		"pressed", GTK_SIGNAL_FUNC (e_shortcut_bar_stop_editing), shortcut_bar);
-	gtk_signal_connect (
-		GTK_OBJECT (E_VSCROLLED_BAR (group->vscrolled_bar)->down_button),
-		"pressed", GTK_SIGNAL_FUNC (e_shortcut_bar_stop_editing), shortcut_bar);
+	g_signal_connect (E_VSCROLLED_BAR (group->vscrolled_bar)->up_button, "pressed",
+			  G_CALLBACK (e_shortcut_bar_stop_editing), shortcut_bar);
+	g_signal_connect (E_VSCROLLED_BAR (group->vscrolled_bar)->down_button, "pressed",
+			  G_CALLBACK (e_shortcut_bar_stop_editing), shortcut_bar);
 
 	group->icon_bar = e_icon_bar_new ();
 	e_icon_bar_set_enable_drags (E_ICON_BAR (group->icon_bar),
@@ -476,64 +518,60 @@ e_shortcut_bar_add_group	(EShortcutBar	*shortcut_bar,
 	gtk_widget_show (group->icon_bar);
 	gtk_container_add (GTK_CONTAINER (group->vscrolled_bar),
 			   group->icon_bar);
-	gtk_signal_connect (GTK_OBJECT (group->icon_bar), "item_selected",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_item_selected),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (group->icon_bar), "item_dragged",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_item_dragged),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (group->icon_bar), "drag_data_get",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_drag_data_get),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (group->icon_bar), "drag_data_received",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_drag_data_received),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (group->icon_bar), "drag_data_delete",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_drag_data_delete),
-			    shortcut_bar);
-	gtk_signal_connect (GTK_OBJECT (group->icon_bar), "drag_end",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_on_drag_end),
-			    shortcut_bar);
+	g_signal_connect (group->icon_bar, "item_selected",
+			  G_CALLBACK (e_shortcut_bar_item_selected),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "item_dragged",
+			  G_CALLBACK (e_shortcut_bar_item_dragged),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "drag_data_get",
+			  G_CALLBACK (e_shortcut_bar_on_drag_data_get),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "drag_motion",
+			  G_CALLBACK (e_shortcut_bar_on_drag_motion),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "drag_drop",
+			  G_CALLBACK (e_shortcut_bar_on_drag_drop),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "drag_data_received",
+			  G_CALLBACK (e_shortcut_bar_on_drag_data_received),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "drag_data_delete",
+			  G_CALLBACK (e_shortcut_bar_on_drag_data_delete),
+			  shortcut_bar);
+	g_signal_connect (group->icon_bar, "drag_end",
+			  G_CALLBACK (e_shortcut_bar_on_drag_end),
+			  shortcut_bar);
 
 #ifndef E_USE_STYLES
 	e_shortcut_bar_set_canvas_style (shortcut_bar, group->icon_bar);
 #endif
 
 	button = gtk_button_new ();
-	gtk_signal_connect (GTK_OBJECT (button), "button_press_event",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_group_button_press),
-			    shortcut_bar);
+	g_signal_connect (button, "button_press_event",
+			  G_CALLBACK (e_shortcut_bar_group_button_press),
+			  shortcut_bar);
 
 	label = e_entry_new ();
-	gtk_object_set(GTK_OBJECT(label),
-		       "draw_background", FALSE,
-		       "draw_borders", FALSE,
-		       "draw_button", TRUE,
-		       "editable", FALSE,
-		       "text", group_name,
-		       "use_ellipsis", TRUE,
-		       "justification", GTK_JUSTIFY_CENTER,
-		       NULL);
+	g_object_set(label,
+		     "draw_background", FALSE,
+		     "draw_borders", FALSE,
+		     "draw_button", TRUE,
+		     "editable", FALSE,
+		     "text", group_name,
+		     "use_ellipsis", TRUE,
+		     "justification", GTK_JUSTIFY_CENTER,
+		     NULL);
 	gtk_widget_show (label);
 	gtk_container_add (GTK_CONTAINER (button), label);
 	gtk_widget_show (button);
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC (e_shortcut_bar_stop_editing),
-			    shortcut_bar);
-
-	gtk_drag_dest_set (GTK_WIDGET (group->icon_bar),
-			   GTK_DEST_DEFAULT_ALL,
-			   target_table, n_targets,
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
-	gtk_drag_dest_set (GTK_WIDGET (button),
-			   GTK_DEST_DEFAULT_ALL,
-			   target_table, n_targets,
-			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (e_shortcut_bar_stop_editing),
+			  shortcut_bar);
 
 	e_group_bar_add_group (E_GROUP_BAR (shortcut_bar),
 			       group->vscrolled_bar, button, group_num);
 
-	gtk_widget_pop_visual ();
 	gtk_widget_pop_colormap ();
 
 	return group_num;
@@ -696,9 +734,9 @@ e_shortcut_bar_item_selected (EIconBar *icon_bar,
 	group_num = e_group_bar_get_group_num (E_GROUP_BAR (shortcut_bar),
 					       GTK_WIDGET (icon_bar)->parent);
 
-	gtk_signal_emit (GTK_OBJECT (shortcut_bar),
-			 e_shortcut_bar_signals[ITEM_SELECTED],
-			 event, group_num, item_num);
+	g_signal_emit (shortcut_bar,
+		       e_shortcut_bar_signals[ITEM_SELECTED], 0,
+		       event, group_num, item_num);
 }
 
 
@@ -749,6 +787,100 @@ e_shortcut_bar_on_drag_data_get (GtkWidget          *widget,
 }
 
 
+static gboolean
+e_shortcut_bar_on_drag_motion (GtkWidget          *widget,
+			       GdkDragContext     *context,
+			       gint                x,
+			       gint                y,
+			       guint               time,
+			       EShortcutBar       *shortcut_bar)
+{
+	EIconBar *icon_bar;
+	gboolean signal_retval;
+	gint group_num, position, before_item;
+	gint scroll_x, scroll_y;
+	GdkDragAction action;
+
+	icon_bar = E_ICON_BAR (widget);
+
+	/* FIXME The canvas scrolling offset should probably be taken into
+	   account by e_icon_bar_find_item_at_position(). */
+	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (icon_bar), &scroll_x, &scroll_y);
+	position = e_icon_bar_find_item_at_position (icon_bar, x + scroll_x, y + scroll_y,
+						     &before_item);
+	group_num = e_group_bar_get_group_num (E_GROUP_BAR (shortcut_bar),
+					       GTK_WIDGET (icon_bar)->parent);
+
+
+	/* We only care about the current group.  */
+	if (group_num != E_GROUP_BAR (shortcut_bar)->current_group_num) {
+		gdk_drag_status (context, 0, time);
+		return TRUE;
+	}
+ 
+	action = 0;
+
+	/* If we are between icons, we can just choose the action by ourselves
+	   if it's an E-SHORTCUT.  */
+	if (before_item != -1) {
+		/* A drag between two icons is allowed only if we are getting
+		   the E-SHORTCUTS target.  */
+
+		GList *p;
+
+		for (p = context->targets; p != NULL; p = p->next) {
+			char *target;
+
+			target = gdk_atom_name ((GdkAtom) p->data);
+			if (strcmp (target, "E-SHORTCUT") == 0) {
+				gdk_drag_status (context, GDK_ACTION_MOVE, time);
+				g_free (target);
+				return TRUE;
+			}
+
+			g_free (target);
+		}
+	}
+
+	signal_retval = FALSE;
+	g_signal_emit (shortcut_bar,
+		       e_shortcut_bar_signals[SHORTCUT_DRAG_MOTION], 0,
+		       widget, context, time,
+		       group_num, position, &signal_retval);
+
+	if (signal_retval) {
+		return TRUE;
+	} else {
+		gdk_drag_status (context, 0, time);
+		return TRUE;
+	}
+}
+
+static gboolean
+e_shortcut_bar_on_drag_drop (GtkWidget *widget,
+			     GdkDragContext *context,
+			     gint x,
+			     gint y,
+			     guint time)
+{
+	char *target;
+	GList *p;
+
+	/* We want the E-SHORTCUT type.  */
+
+	for (p = context->targets; p != NULL; p = p->next) {
+		target = gdk_atom_name (GDK_POINTER_TO_ATOM (p->data));
+		if (strcmp (target, "E-SHORTCUT") == 0) {
+			g_free (target);
+			gtk_drag_get_data (widget, context, GDK_POINTER_TO_ATOM (p->data), time);
+			return TRUE;
+		}
+	}
+
+	gtk_drag_get_data (widget, context, GDK_POINTER_TO_ATOM (context->targets->data), time);
+	return TRUE;
+}
+
 static void  
 e_shortcut_bar_on_drag_data_received  (GtkWidget          *widget,
 				       GdkDragContext     *context,
@@ -762,27 +894,50 @@ e_shortcut_bar_on_drag_data_received  (GtkWidget          *widget,
 	gchar *item_name, *item_url;
 	EIconBar *icon_bar;
 	gint position, group_num;
+	gboolean signal_retval;
+	gint scroll_x, scroll_y;
+	gboolean before_item;
+	char *target_type;
 
 	icon_bar = E_ICON_BAR (widget);
-	position = icon_bar->dragging_before_item_num;
 
-	if ((data->length >= 0) && (data->format == 8)
-	    && (position != -1) && (info == TARGET_SHORTCUT)) {
+	/* FIXME The canvas scrolling offset should probably be taken into
+	   account by e_icon_bar_find_item_at_position().  */
+	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (icon_bar), &scroll_x, &scroll_y);
+	position = e_icon_bar_find_item_at_position (icon_bar, x + scroll_x, y + scroll_y, &before_item);
+	group_num = e_group_bar_get_group_num (E_GROUP_BAR (shortcut_bar),
+					       GTK_WIDGET (icon_bar)->parent);
+
+	target_type = gdk_atom_name (data->target);
+
+	/* If it's a shortcut drop, we can handle it ourselves.  */
+	if (position == -1
+	    && strcmp (target_type, "E-SHORTCUT") == 0
+	    && data->length >= 0 && data->format == 8) {
 		item_name = data->data;
 		item_url = item_name + strlen (item_name) + 1;
 
-		group_num = e_group_bar_get_group_num (E_GROUP_BAR (shortcut_bar),
-						       GTK_WIDGET (icon_bar)->parent);
-
-		gtk_signal_emit (GTK_OBJECT (shortcut_bar),
-				 e_shortcut_bar_signals[SHORTCUT_DROPPED],
-				 group_num, position, item_url, item_name);
+		g_signal_emit (shortcut_bar,
+			       e_shortcut_bar_signals[SHORTCUT_DROPPED], 0,
+			       group_num, before_item, item_url, item_name);
 
 		gtk_drag_finish (context, TRUE, TRUE, time);
+		g_free (target_type);
 		return;
 	}
-  
-	gtk_drag_finish (context, FALSE, FALSE, time);
+
+	g_free (target_type);
+
+	/* The drop is of some custom kind, so we try emitting the signal.  */
+	signal_retval = FALSE;
+	g_signal_emit (shortcut_bar,
+		       e_shortcut_bar_signals[SHORTCUT_DRAG_DATA_RECEIVED], 0,
+		       widget, context, data, time, group_num, position, &signal_retval);
+
+	/* If the signal handler doesn't handle it, we just do a
+	   gtk_drag_finish() reporting failure.  */
+	if (! signal_retval)
+		gtk_drag_finish (context, FALSE, FALSE, time);
 }
 
 
@@ -799,8 +954,8 @@ e_shortcut_bar_on_drag_data_delete (GtkWidget          *widget,
 	group_num = e_group_bar_get_group_num (E_GROUP_BAR (shortcut_bar),
 					       widget->parent);
 
-	gtk_signal_emit (GTK_OBJECT (shortcut_bar), e_shortcut_bar_signals[SHORTCUT_DRAGGED],
-			 group_num, icon_bar->dragged_item_num);
+	g_signal_emit (shortcut_bar, e_shortcut_bar_signals[SHORTCUT_DRAGGED], 0,
+		       group_num, icon_bar->dragged_item_num);
 }
 
 
