@@ -31,6 +31,7 @@
 #include <gtk/gtkbutton.h>
 #include <gtk/gtksignal.h>
 #include "e-vscrolled-bar.h"
+#include <gal/util/e-util.h>
 
 /* These are the offsets of the up & down buttons from the right and top/bottom
    of the widget. */
@@ -42,16 +43,14 @@
 
 static void e_vscrolled_bar_class_init (EVScrolledBarClass *class);
 static void e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar);
-static void e_vscrolled_bar_destroy (GtkObject *object);
-static void e_vscrolled_bar_finalize (GtkObject *object);
+static void e_vscrolled_bar_dispose (GObject *object);
+static void e_vscrolled_bar_finalize (GObject *object);
 static void e_vscrolled_bar_map (GtkWidget *widget);
 static void e_vscrolled_bar_unmap (GtkWidget *widget);
 static void e_vscrolled_bar_size_request (GtkWidget      *widget,
 					  GtkRequisition *requisition);
 static void e_vscrolled_bar_size_allocate (GtkWidget *widget,
 					   GtkAllocation *allocation);
-static void e_vscrolled_bar_draw (GtkWidget    *widget,
-				  GdkRectangle *area);
 static gint e_vscrolled_bar_button_press   (GtkWidget        *widget,
 					    GdkEventButton   *event);
 static void e_vscrolled_bar_add (GtkContainer *container,
@@ -73,57 +72,39 @@ static void e_vscrolled_bar_button_clicked (GtkWidget *button,
 static gboolean e_vscrolled_bar_scroll (gpointer data);
 static gboolean e_vscrolled_bar_timeout_handler (gpointer data);
 
-
+#define PARENT_TYPE GTK_TYPE_BIN
 static GtkBinClass *parent_class;
 
 
-GtkType
-e_vscrolled_bar_get_type (void)
-{
-	static GtkType e_vscrolled_bar_type = 0;
-
-	if (!e_vscrolled_bar_type) {
-		GtkTypeInfo e_vscrolled_bar_info = {
-			"EVScrolledBar",
-			sizeof (EVScrolledBar),
-			sizeof (EVScrolledBarClass),
-			(GtkClassInitFunc) e_vscrolled_bar_class_init,
-			(GtkObjectInitFunc) e_vscrolled_bar_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		parent_class = gtk_type_class (GTK_TYPE_BIN);
-		e_vscrolled_bar_type = gtk_type_unique (GTK_TYPE_BIN,
-							&e_vscrolled_bar_info);
-	}
-
-	return e_vscrolled_bar_type;
-}
-
+E_MAKE_TYPE (e_vscrolled_bar,
+	     "EVScrolledBar",
+	     EVScrolledBar,
+	     e_vscrolled_bar_class_init,
+	     e_vscrolled_bar_init,
+	     PARENT_TYPE)
 
 static void
 e_vscrolled_bar_class_init (EVScrolledBarClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GtkWidgetClass *widget_class;
 	GtkContainerClass *container_class;
 
-	object_class = (GtkObjectClass *) class;
+	parent_class = g_type_class_ref (PARENT_TYPE);
+
+	object_class = (GObjectClass *) class;
 	widget_class = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
 
 	/* Method override */
-	object_class->destroy		= e_vscrolled_bar_destroy;
-	object_class->finalize		= e_vscrolled_bar_finalize;
+	object_class->dispose		= e_vscrolled_bar_dispose;
+	object_class->finalize          = e_vscrolled_bar_finalize;
 
 	widget_class->map		= e_vscrolled_bar_map;
 	widget_class->unmap		= e_vscrolled_bar_unmap;
 	widget_class->size_request	= e_vscrolled_bar_size_request;
  	widget_class->size_allocate	= e_vscrolled_bar_size_allocate;
-	widget_class->draw		= e_vscrolled_bar_draw;
-	widget_class->button_press_event   = e_vscrolled_bar_button_press;
+	widget_class->button_press_event= e_vscrolled_bar_button_press;
 
 	container_class->add		= e_vscrolled_bar_add;
 	container_class->remove		= e_vscrolled_bar_remove;
@@ -153,9 +134,12 @@ e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar)
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (vscrolled_bar->up_button), arrow);
 	gtk_widget_show (vscrolled_bar->up_button);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->up_button), "pressed", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_pressed), vscrolled_bar);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->up_button), "released", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_released), vscrolled_bar);
-	gtk_signal_connect (GTK_OBJECT (vscrolled_bar->up_button), "clicked", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_clicked), vscrolled_bar);
+	g_signal_connect_after (vscrolled_bar->up_button, "pressed",
+				G_CALLBACK (e_vscrolled_bar_button_pressed), vscrolled_bar);
+	g_signal_connect_after (vscrolled_bar->up_button, "released",
+				G_CALLBACK (e_vscrolled_bar_button_released), vscrolled_bar);
+	g_signal_connect (vscrolled_bar->up_button, "clicked",
+			  G_CALLBACK (e_vscrolled_bar_button_clicked), vscrolled_bar);
 
 	vscrolled_bar->down_button = gtk_button_new ();
 	gtk_widget_set_composite_name (vscrolled_bar->up_button,
@@ -167,9 +151,12 @@ e_vscrolled_bar_init (EVScrolledBar *vscrolled_bar)
 	gtk_widget_show (arrow);
 	gtk_container_add (GTK_CONTAINER (vscrolled_bar->down_button), arrow);
 	gtk_widget_show (vscrolled_bar->down_button);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->down_button), "pressed", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_pressed), vscrolled_bar);
-	gtk_signal_connect_after (GTK_OBJECT (vscrolled_bar->down_button), "released", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_released), vscrolled_bar);
-	gtk_signal_connect (GTK_OBJECT (vscrolled_bar->down_button), "clicked", GTK_SIGNAL_FUNC (e_vscrolled_bar_button_clicked), vscrolled_bar);
+	g_signal_connect_after (vscrolled_bar->down_button, "pressed",
+				G_CALLBACK (e_vscrolled_bar_button_pressed), vscrolled_bar);
+	g_signal_connect_after (vscrolled_bar->down_button, "released",
+				G_CALLBACK (e_vscrolled_bar_button_released), vscrolled_bar);
+	g_signal_connect (vscrolled_bar->down_button, "clicked",
+			  G_CALLBACK (e_vscrolled_bar_button_clicked), vscrolled_bar);
 
 	gtk_widget_pop_composite_child ();
 
@@ -194,7 +181,7 @@ e_vscrolled_bar_new (GtkAdjustment *adjustment)
 {
 	GtkWidget *vscrolled_bar;
 
-	vscrolled_bar = GTK_WIDGET (gtk_type_new (e_vscrolled_bar_get_type ()));
+	vscrolled_bar = GTK_WIDGET (g_object_new (E_VSCROLLED_BAR_TYPE, NULL));
 	e_vscrolled_bar_set_adjustment (E_VSCROLLED_BAR (vscrolled_bar),
 					adjustment);
 
@@ -203,7 +190,7 @@ e_vscrolled_bar_new (GtkAdjustment *adjustment)
 
 
 static void
-e_vscrolled_bar_destroy (GtkObject *object)
+e_vscrolled_bar_dispose (GObject *object)
 {
 	EVScrolledBar *vscrolled_bar;
 
@@ -214,23 +201,28 @@ e_vscrolled_bar_destroy (GtkObject *object)
 		vscrolled_bar->timeout_id = 0;
 	}
 
-	gtk_widget_unparent (vscrolled_bar->up_button);
-	gtk_widget_unparent (vscrolled_bar->down_button);
+	if (vscrolled_bar->up_button)
+		gtk_widget_unparent (vscrolled_bar->up_button);
+	vscrolled_bar->up_button = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	if (vscrolled_bar->down_button)
+		gtk_widget_unparent (vscrolled_bar->down_button);
+	vscrolled_bar->down_button = NULL;
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
 static void
-e_vscrolled_bar_finalize (GtkObject *object)
+e_vscrolled_bar_finalize (GObject *object)
 {
 	EVScrolledBar *vscrolled_bar;
 
 	vscrolled_bar = E_VSCROLLED_BAR (object);
 
-	gtk_object_unref (GTK_OBJECT (vscrolled_bar->adjustment));
+	g_object_unref (vscrolled_bar->adjustment);
 
-	GTK_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
@@ -372,7 +364,7 @@ e_vscrolled_bar_size_allocate (GtkWidget *widget,
 				  &button_allocation);
 }
 
-
+#if 0
 static void
 e_vscrolled_bar_draw (GtkWidget    *widget,
 		      GdkRectangle *area)
@@ -400,7 +392,7 @@ e_vscrolled_bar_draw (GtkWidget    *widget,
 	    gtk_widget_intersect (vscrolled_bar->down_button, area, &child_area))
 		gtk_widget_draw (vscrolled_bar->down_button, &child_area);
 }
-
+#endif
 
 static gint
 e_vscrolled_bar_button_press (GtkWidget      *widget,
@@ -430,8 +422,8 @@ e_vscrolled_bar_button_press (GtkWidget      *widget,
 
 	if (adjustment->value != new_value) {
 		adjustment->value = new_value;
-		gtk_signal_emit_by_name (GTK_OBJECT (adjustment),
-					 "value_changed");
+		g_signal_emit_by_name (adjustment,
+				       "value_changed");
 		return TRUE;
 	}
 
@@ -551,32 +543,34 @@ e_vscrolled_bar_set_adjustment (EVScrolledBar     *vscrolled_bar,
 	if (adjustment)
 		g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 	else
-		adjustment = (GtkAdjustment*) gtk_object_new (GTK_TYPE_ADJUSTMENT, NULL);
+		adjustment = (GtkAdjustment*) g_object_new (GTK_TYPE_ADJUSTMENT, NULL);
 
 	if (vscrolled_bar->adjustment == adjustment)
 		return;
 
 	if (vscrolled_bar->adjustment) {
-		gtk_signal_disconnect_by_func (GTK_OBJECT (vscrolled_bar->adjustment),
-					       GTK_SIGNAL_FUNC (e_vscrolled_bar_adjustment_changed),
-					       vscrolled_bar);
-		gtk_object_unref (GTK_OBJECT (vscrolled_bar->adjustment));
+		g_signal_handlers_disconnect_matched (vscrolled_bar->adjustment,
+						      G_SIGNAL_MATCH_FUNC,
+						      0, 0, 0,
+						      e_vscrolled_bar_adjustment_changed,
+						      NULL);
+		g_object_unref (vscrolled_bar->adjustment);
 	}
 
 	vscrolled_bar->adjustment = adjustment;
-	gtk_object_ref (GTK_OBJECT (vscrolled_bar->adjustment));
+	g_object_ref (vscrolled_bar->adjustment);
 	gtk_object_sink (GTK_OBJECT (vscrolled_bar->adjustment));
 
 	/* I've used connect_after here to avoid a problem when using a
 	   GnomeCanvas as the child widget. When just using connect it would
 	   leave a blank space when one of the buttons is hidden. We want
 	   the GtkLayout to handle the scrolling before we hide any buttons. */
-	gtk_signal_connect_after (GTK_OBJECT (adjustment), "changed",
-				  GTK_SIGNAL_FUNC (e_vscrolled_bar_adjustment_changed),
-				  vscrolled_bar);
-	gtk_signal_connect_after (GTK_OBJECT (adjustment), "value_changed",
-				  GTK_SIGNAL_FUNC (e_vscrolled_bar_adjustment_changed),
-				  vscrolled_bar);
+	g_signal_connect_after (adjustment, "changed",
+				G_CALLBACK (e_vscrolled_bar_adjustment_changed),
+				vscrolled_bar);
+	g_signal_connect_after (adjustment, "value_changed",
+				G_CALLBACK (e_vscrolled_bar_adjustment_changed),
+				vscrolled_bar);
 
 	e_vscrolled_bar_adjustment_changed (adjustment, vscrolled_bar);
 
@@ -593,9 +587,7 @@ e_vscrolled_bar_adjustment_changed (GtkAdjustment *adjustment,
 
 	g_return_if_fail (adjustment != NULL);
 	g_return_if_fail (data != NULL);
-#if 0
-	g_print ("Adjustment changed to: %g\n", adjustment->value);
-#endif
+
 	vscrolled_bar = E_VSCROLLED_BAR (data);
 
 	if (!GTK_WIDGET_MAPPED (vscrolled_bar))
@@ -695,8 +687,8 @@ e_vscrolled_bar_scroll (gpointer data)
 
 	if (adjustment->value != new_value) {
 		adjustment->value = new_value;
-		gtk_signal_emit_by_name (GTK_OBJECT (adjustment),
-					 "value_changed");
+		g_signal_emit_by_name (adjustment,
+				       "value_changed");
 	}
 
 	return retval;
