@@ -1,33 +1,36 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* e-text-event-processor.c
- * Copyright (C) 2000  Helix Code, Inc.
- * Author: Chris Lahey <clahey@helixcode.com>
+/* 
+ * e-text-event-processor-emacs-like.c
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtksignal.h>
+#include <gal/util/e-util.h>
 #include "e-text-event-processor-emacs-like.h"
 
 static void e_text_event_processor_emacs_like_init		(ETextEventProcessorEmacsLike		 *card);
 static void e_text_event_processor_emacs_like_class_init	(ETextEventProcessorEmacsLikeClass	 *klass);
 static gint e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventProcessorEvent *event);
 
+#define PARENT_TYPE E_TEXT_EVENT_PROCESSOR_TYPE
 static ETextEventProcessorClass *parent_class = NULL;
 
 /* The arguments we take */
@@ -59,7 +62,7 @@ static const ETextEventProcessorCommand control_keys[26] =
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* t */
 	{ E_TEP_START_OF_LINE,      E_TEP_DELETE, 0, "" }, /* u */
 	{ E_TEP_SELECTION,          E_TEP_PASTE, 0, "" }, /* v */
-	{ E_TEP_SELECTION,          E_TEP_DELETE, 0, "" }, /* w */
+	{ E_TEP_SELECTION,          E_TEP_NOP, 0, "" }, /* w */
 	{ E_TEP_SELECTION,          E_TEP_DELETE, 0, "" }, /* x */
 	{ E_TEP_SELECTION,          E_TEP_PASTE, 0, "" }, /* y */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" }           /* z */
@@ -69,7 +72,7 @@ static const ETextEventProcessorCommand alt_keys[26] =
 {
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* a */
 	{ E_TEP_BACKWARD_WORD,      E_TEP_MOVE, 0, "" }, /* b */
-	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* c */
+	{ E_TEP_SELECTION, E_TEP_CAPS, E_TEP_CAPS_TITLE, "" },/* c */
 	{ E_TEP_FORWARD_WORD,       E_TEP_DELETE, 0, "" }, /* d */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* e */
 	{ E_TEP_FORWARD_WORD,       E_TEP_MOVE, 0, "" }, /* f */
@@ -78,7 +81,7 @@ static const ETextEventProcessorCommand alt_keys[26] =
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* i */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* j */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* k */
-	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* l */
+	{ E_TEP_SELECTION, E_TEP_CAPS, E_TEP_CAPS_LOWER, "" },           /* l */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* m */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* n */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* o */
@@ -87,7 +90,7 @@ static const ETextEventProcessorCommand alt_keys[26] =
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* r */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* s */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* t */
-	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* u */
+	{ E_TEP_SELECTION, E_TEP_CAPS, E_TEP_CAPS_UPPER, "" },           /* u */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* v */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* w */
 	{ E_TEP_SELECTION, E_TEP_NOP, 0, "" },           /* x */
@@ -96,41 +99,21 @@ static const ETextEventProcessorCommand alt_keys[26] =
 
 };
 
-GtkType
-e_text_event_processor_emacs_like_get_type (void)
-{
-  static GtkType text_event_processor_emacs_like_type = 0;
-
-  if (!text_event_processor_emacs_like_type)
-    {
-      static const GtkTypeInfo text_event_processor_emacs_like_info =
-      {
-        "ETextEventProcessorEmacsLike",
-        sizeof (ETextEventProcessorEmacsLike),
-        sizeof (ETextEventProcessorEmacsLikeClass),
-        (GtkClassInitFunc) e_text_event_processor_emacs_like_class_init,
-        (GtkObjectInitFunc) e_text_event_processor_emacs_like_init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
-      };
-
-      text_event_processor_emacs_like_type = gtk_type_unique (e_text_event_processor_get_type (), &text_event_processor_emacs_like_info);
-    }
-
-  return text_event_processor_emacs_like_type;
-}
+E_MAKE_TYPE (e_text_event_processor_emacs_like,
+	     "ETextEventProcessorEmacsLike",
+	     ETextEventProcessorEmacsLike,
+	     e_text_event_processor_emacs_like_class_init,
+	     e_text_event_processor_emacs_like_init,
+	     PARENT_TYPE)
 
 static void
 e_text_event_processor_emacs_like_class_init (ETextEventProcessorEmacsLikeClass *klass)
 {
-  GtkObjectClass *object_class;
   ETextEventProcessorClass *processor_class;
 
-  object_class = (GtkObjectClass*) klass;
   processor_class = (ETextEventProcessorClass*) klass;
 
-  parent_class = gtk_type_class (e_text_event_processor_get_type ());
+  parent_class = g_type_class_ref (PARENT_TYPE);
 
   processor_class->event = e_text_event_processor_emacs_like_event;
 }
@@ -151,7 +134,7 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 		if (event->button.button == 1) {
 			command.action = E_TEP_GRAB;
 			command.time = event->button.time;
-			gtk_signal_emit_by_name (GTK_OBJECT (tep), "command", &command);
+			g_signal_emit_by_name (tep, "command", &command);
 			if (event->button.state & GDK_SHIFT_MASK)
 				command.action = E_TEP_SELECT;
 			else
@@ -180,16 +163,13 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 		if (event->button.button == 1) {
 			command.action = E_TEP_UNGRAB;
 			command.time = event->button.time;
-			gtk_signal_emit_by_name (GTK_OBJECT (tep), "command", &command);
-			command.time = event->button.time;
 			tep_el->mouse_down = FALSE;
-			command.action = E_TEP_NOP;
 		} else if (event->button.button == 2) {
 			command.action = E_TEP_MOVE;
 			command.position = E_TEP_VALUE;
 			command.value = event->button.position;
 			command.time = event->button.time;
-			gtk_signal_emit_by_name (GTK_OBJECT (tep), "command", &command);			
+			g_signal_emit_by_name (tep, "command", &command);
 
 			command.action = E_TEP_GET_SELECTION;
 			command.position = E_TEP_SELECTION;
@@ -286,7 +266,7 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 				} else if (key.state & GDK_SHIFT_MASK) {
 					command.action = E_TEP_COPY;
 					command.position = E_TEP_SELECTION;
-					gtk_signal_emit_by_name (GTK_OBJECT (tep), "command", &command);
+					g_signal_emit_by_name (tep, "command", &command);
 				
 					command.action = E_TEP_DELETE;
 					command.position = E_TEP_SELECTION;
@@ -435,7 +415,7 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 				break;
 
 			default:
-				if (key.state & GDK_CONTROL_MASK) {
+				if ((key.state & GDK_CONTROL_MASK) && !(key.state & GDK_MOD1_MASK)) {
 					if ((key.keyval >= 'A') && (key.keyval <= 'Z'))
 						key.keyval -= 'A' - 'a';
 					
@@ -447,17 +427,17 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 						command.string = control_keys[(int) (key.keyval - 'a')].string;
 					}
 
-					if (key.keyval == 'x' || key.keyval == 'w') {
+					if (key.keyval == 'x') {
 						command.action = E_TEP_COPY;
 						command.position = E_TEP_SELECTION;
-						gtk_signal_emit_by_name (GTK_OBJECT (tep), "command", &command);
+						g_signal_emit_by_name (tep, "command", &command);
 						
 						command.action = E_TEP_DELETE;
 						command.position = E_TEP_SELECTION;
 					}
 					
 					break;
-				} else if (key.state & GDK_MOD1_MASK) {
+				} else if ((key.state & GDK_MOD1_MASK) && !(key.state & GDK_CONTROL_MASK)) {
 					if ((key.keyval >= 'A') && (key.keyval <= 'Z'))
 						key.keyval -= 'A' - 'a';
 					
@@ -468,7 +448,7 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 						command.value = alt_keys[(int) (key.keyval - 'a')].value;
 						command.string = alt_keys[(int) (key.keyval - 'a')].string;
 					}
-				} else if (key.length > 0) {
+				} else if (!(key.state & GDK_MOD1_MASK) && !(key.state & GDK_CONTROL_MASK) && key.length > 0) {
 					if (key.keyval >= GDK_KP_0 && key.keyval <= GDK_KP_9) {
 						key.keyval = '0';
 						key.string = "0";
@@ -493,7 +473,7 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 		}
 	}
 	if (command.action != E_TEP_NOP) {
-		gtk_signal_emit_by_name (GTK_OBJECT (tep), "command", &command);
+		g_signal_emit_by_name (tep, "command", &command);
 		return 1;
 	}
 	else
@@ -503,7 +483,7 @@ e_text_event_processor_emacs_like_event (ETextEventProcessor *tep, ETextEventPro
 ETextEventProcessor *
 e_text_event_processor_emacs_like_new (void)
 {
-	ETextEventProcessorEmacsLike *retval = gtk_type_new (e_text_event_processor_emacs_like_get_type ());
+	ETextEventProcessorEmacsLike *retval = g_object_new (E_TEXT_EVENT_PROCESSOR_EMACS_LIKE_TYPE, NULL);
 	return E_TEXT_EVENT_PROCESSOR (retval);
 }
 
