@@ -23,6 +23,7 @@
 
 #include <config.h>
 #include "e-util.h"
+#include "e-i18n.h"
 
 #include <glib.h>
 #include <gtk/gtkobject.h>
@@ -36,33 +37,48 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
+#include <libgnome/gnome-util.h>
+#include <math.h>
 
 #if 0
 #include <libgnomevfs/gnome-vfs.h>
 #endif
 
 int
-g_str_compare(const void *x, const void *y)
+g_str_compare (const void *x, const void *y)
 {
 	if (x == NULL || y == NULL) {
 		if (x == y)
 			return 0;
 		else
 			return x ? -1 : 1;
-	} 
-		
-	return strcmp(x, y);
+	}
+	
+	return strcmp (x, y);
 }
 
 int
-g_int_compare(const void *x, const void *y)
+g_collate_compare (const void *x, const void *y)
 {
-  if ( GPOINTER_TO_INT(x) < GPOINTER_TO_INT(y) )
-    return -1;
-  else if ( GPOINTER_TO_INT(x) == GPOINTER_TO_INT(y) )
-    return 0;
-  else
-    return 1;
+	if (x == NULL || y == NULL) {
+		if (x == y)
+			return 0;
+		else
+			return x ? -1 : 1;
+	}
+	
+	return g_utf8_collate (x, y);
+}
+
+int
+g_int_compare (const void *x, const void *y)
+{
+	if (GPOINTER_TO_INT (x) < GPOINTER_TO_INT (y))
+		return -1;
+	else if (GPOINTER_TO_INT (x) == GPOINTER_TO_INT (y))
+		return 0;
+	else
+		return 1;
 }
 
 char *
@@ -88,7 +104,7 @@ e_free_object_list (GList *list)
 	GList *p;
 
 	for (p = list; p != NULL; p = p->next)
-		gtk_object_unref (GTK_OBJECT (p->data));
+		g_object_unref (p->data);
 
 	g_list_free (list);
 }
@@ -99,7 +115,7 @@ e_free_object_slist (GSList *list)
 	GSList *p;
 
 	for (p = list; p != NULL; p = p->next)
-		gtk_object_unref (GTK_OBJECT (p->data));
+		g_object_unref (p->data);
 
 	g_slist_free (list);
 }
@@ -248,13 +264,22 @@ e_mkdir_hier(const char *path, mode_t mode)
 {
 	char *copy, *p;
 
-	p = copy = g_strdup (path);
+	if (path[0] == '/') {
+		p = copy = g_strdup (path);
+	} else {
+		gchar *current_dir = g_get_current_dir();
+		p = copy = g_concat_dir_and_file (current_dir, path);
+	}
+
 	do {
 		p = strchr (p + 1, '/');
 		if (p)
 			*p = '\0';
-		if (access (copy, F_OK) == -1) {
-			if (mkdir (copy, mode) == -1) {
+		if (mkdir (copy, mode) == -1) {
+			switch (errno) {
+			case EEXIST:
+				break;
+			default:
 				g_free (copy);
 				return -1;
 			}
@@ -310,495 +335,10 @@ e_read_uri(const char *uri)
 }
 #endif
 
-typedef gint (*GtkSignal_INT__INT_INT_POINTER) (GtkObject * object,
-						gint arg1,
-						gint arg2,
-						gpointer arg3,
-						gpointer user_data);
+/* Include build marshalers */
 
-void
-e_marshal_INT__INT_INT_POINTER (GtkObject * object,
-				GtkSignalFunc func,
-				gpointer func_data, GtkArg * args)
-{
-	GtkSignal_INT__INT_INT_POINTER rfunc;
-	gint *return_val;
-	return_val = GTK_RETLOC_INT (args[3]);
-	rfunc = (GtkSignal_INT__INT_INT_POINTER) func;
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_INT     (args[0]),
-				GTK_VALUE_INT     (args[1]),
-				GTK_VALUE_POINTER (args[2]),
-				func_data);
-}
-
-typedef gint (*GtkSignal_INT__INT_POINTER_INT_POINTER) (GtkObject * object,
-							gint arg1,
-							gpointer arg2,
-							gint arg3,
-							gpointer arg4,
-							gpointer user_data);
-
-void
-e_marshal_INT__INT_POINTER_INT_POINTER (GtkObject * object,
-					GtkSignalFunc func,
-					gpointer func_data, GtkArg * args)
-{
-	GtkSignal_INT__INT_POINTER_INT_POINTER rfunc;
-	gint *return_val;
-	return_val = GTK_RETLOC_INT (args[4]);
-	rfunc = (GtkSignal_INT__INT_POINTER_INT_POINTER) func;
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_INT     (args[0]),
-				GTK_VALUE_POINTER (args[1]),
-				GTK_VALUE_INT     (args[2]),
-				GTK_VALUE_POINTER (args[3]),
-				func_data);
-}
-
-typedef void (*GtkSignal_NONE__OBJECT_DOUBLE_DOUBLE_BOOL) (GtkObject * object,
-							      GtkObject *arg1,
-							      gdouble arg2,
-							      gdouble arg3,
-							      gboolean arg4,
-							      gpointer user_data);
-
-void
-e_marshal_NONE__OBJECT_DOUBLE_DOUBLE_BOOL (GtkObject * object,
-					   GtkSignalFunc func,
-					   gpointer func_data, GtkArg * args)
-{
-	GtkSignal_NONE__OBJECT_DOUBLE_DOUBLE_BOOL rfunc;
-	rfunc = (GtkSignal_NONE__OBJECT_DOUBLE_DOUBLE_BOOL) func;
-	(*rfunc) (object,
-		  GTK_VALUE_OBJECT (args[0]),
-		  GTK_VALUE_DOUBLE (args[1]),
-		  GTK_VALUE_DOUBLE (args[2]),
-		  GTK_VALUE_BOOL   (args[3]),
-		  func_data);
-}
-
-typedef gdouble (*GtkSignal_DOUBLE__OBJECT_DOUBLE_DOUBLE_BOOL) (GtkObject * object,
-								GtkObject *arg1,
-								gdouble arg2,
-								gdouble arg3,
-								gboolean arg4,
-								gpointer user_data);
-
-void
-e_marshal_DOUBLE__OBJECT_DOUBLE_DOUBLE_BOOL (GtkObject * object,
-					GtkSignalFunc func,
-					gpointer func_data, GtkArg * args)
-{
-	GtkSignal_DOUBLE__OBJECT_DOUBLE_DOUBLE_BOOL rfunc;
-	gdouble *return_val;
-	return_val = GTK_RETLOC_DOUBLE (args[4]);
-	rfunc = (GtkSignal_DOUBLE__OBJECT_DOUBLE_DOUBLE_BOOL) func;
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_OBJECT (args[0]),
-				GTK_VALUE_DOUBLE (args[1]),
-				GTK_VALUE_DOUBLE (args[2]),
-				GTK_VALUE_BOOL   (args[3]),
-				func_data);
-}
-
-typedef gdouble (*GtkSignal_BOOL__OBJECT_DOUBLE_DOUBLE_BOOL) (GtkObject * object,
-							      GtkObject *arg1,
-							      gdouble arg2,
-							      gdouble arg3,
-							      gboolean arg4,
-							      gpointer user_data);
-
-void
-e_marshal_BOOL__OBJECT_DOUBLE_DOUBLE_BOOL (GtkObject * object,
-					GtkSignalFunc func,
-					gpointer func_data, GtkArg * args)
-{
-	GtkSignal_BOOL__OBJECT_DOUBLE_DOUBLE_BOOL rfunc;
-	gboolean *return_val;
-	return_val = GTK_RETLOC_BOOL (args[4]);
-	rfunc = (GtkSignal_BOOL__OBJECT_DOUBLE_DOUBLE_BOOL) func;
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_OBJECT (args[0]),
-				GTK_VALUE_DOUBLE (args[1]),
-				GTK_VALUE_DOUBLE (args[2]),
-				GTK_VALUE_BOOL   (args[3]),
-				func_data);
-}
-
-typedef void (*GtkSignal_NONE__INT_INT_POINTER_POINTER_INT_INT) (GtkObject * object,
-								 gint arg1, 
-								 gint arg2,
-								 gpointer arg3,
-								 gpointer arg4,
-								 gint arg5,
-								 gint arg6,
-								 gpointer user_data);
-void
-e_marshal_NONE__INT_INT_POINTER_POINTER_INT_INT (GtkObject * object,
-					   GtkSignalFunc func,
-					   gpointer func_data, GtkArg * args)
-{
-  GtkSignal_NONE__INT_INT_POINTER_POINTER_INT_INT rfunc;
-  rfunc = (GtkSignal_NONE__INT_INT_POINTER_POINTER_INT_INT) func;
-  (*rfunc) (object,
-	    GTK_VALUE_INT (args[0]), GTK_VALUE_INT (args[1]), 
-	    GTK_VALUE_POINTER (args[2]),
-	    GTK_VALUE_POINTER (args[3]),
-	    GTK_VALUE_INT (args[4]), GTK_VALUE_INT (args[5]), func_data);
-}
-
-typedef void (*GtkSignal_NONE__INT_POINTER_INT_POINTER_POINTER_INT_INT) (GtkObject * object,
-									 gint arg1, 
-									 gpointer arg2,
-									 gint arg3,
-									 gpointer arg4,
-									 gpointer arg5,
-									 gint arg6,
-									 gint arg7,
-									 gpointer user_data);
-void
-e_marshal_NONE__INT_POINTER_INT_POINTER_POINTER_INT_INT (GtkObject * object,
-							 GtkSignalFunc func,
-							 gpointer func_data, GtkArg * args)
-{
-  GtkSignal_NONE__INT_POINTER_INT_POINTER_POINTER_INT_INT rfunc;
-  rfunc = (GtkSignal_NONE__INT_POINTER_INT_POINTER_POINTER_INT_INT) func;
-  (*rfunc) (object,
-	    GTK_VALUE_INT (args[0]), GTK_VALUE_POINTER (args[1]), GTK_VALUE_INT (args[2]),
-	    GTK_VALUE_POINTER (args[3]),
-	    GTK_VALUE_POINTER (args[4]),
-	    GTK_VALUE_INT (args[5]), GTK_VALUE_INT (args[6]), func_data);
-}
-
-typedef void (*GtkSignal_NONE__INT_INT_POINTER_INT) (GtkObject * object,
-						     gint arg1, 
-						     gint arg2,
-						     gpointer arg3,
-						     gint arg4, gpointer user_data);
-void
-e_marshal_NONE__INT_INT_POINTER_INT (GtkObject * object,
-			       GtkSignalFunc func,
-			       gpointer func_data, GtkArg * args)
-{
-  GtkSignal_NONE__INT_INT_POINTER_INT rfunc;
-  rfunc = (GtkSignal_NONE__INT_INT_POINTER_INT) func;
-  (*rfunc) (object,
-	    GTK_VALUE_INT (args[0]), GTK_VALUE_INT (args[1]), 
-	    GTK_VALUE_POINTER (args[2]), GTK_VALUE_INT (args[3]), func_data);
-}
-
-typedef void (*GtkSignal_NONE__INT_POINTER_INT_POINTER_INT) (GtkObject * object,
-							     gint arg1, 
-							     gpointer arg2,
-							     gint arg3,
-							     gpointer arg4,
-							     gint arg5, gpointer user_data);
-void
-e_marshal_NONE__INT_POINTER_INT_POINTER_INT (GtkObject * object,
-					     GtkSignalFunc func,
-					     gpointer func_data, GtkArg * args)
-{
-  GtkSignal_NONE__INT_POINTER_INT_POINTER_INT rfunc;
-  rfunc = (GtkSignal_NONE__INT_POINTER_INT_POINTER_INT) func;
-  (*rfunc) (object,
-	    GTK_VALUE_INT (args[0]), GTK_VALUE_POINTER (args[1]), GTK_VALUE_INT (args[2]), 
-	    GTK_VALUE_POINTER (args[3]), GTK_VALUE_INT (args[4]), func_data);
-}
-
-typedef gboolean (*GtkSignal_BOOL__INT_INT_POINTER_INT_INT_INT) (GtkObject * object,
-								 gint arg1, 
-								 gint arg2,
-								 gpointer arg3,
-								 gint arg4,
-								 gint arg5,
-								 gint arg6,
-								 gpointer user_data);
-void
-e_marshal_BOOL__INT_INT_POINTER_INT_INT_INT (GtkObject * object,
-				       GtkSignalFunc func,
-				       gpointer func_data, GtkArg * args)
-{
-  GtkSignal_BOOL__INT_INT_POINTER_INT_INT_INT rfunc;
-  gboolean *return_val;
-  return_val = GTK_RETLOC_BOOL (args[6]);
-  rfunc = (GtkSignal_BOOL__INT_INT_POINTER_INT_INT_INT) func;
-  *return_val = (*rfunc) (object,
-			  GTK_VALUE_INT (args[0]),
-			  GTK_VALUE_INT (args[1]), 
-			  GTK_VALUE_POINTER (args[2]),
-			  GTK_VALUE_INT (args[3]),
-			  GTK_VALUE_INT (args[4]),
-			  GTK_VALUE_INT (args[5]), func_data);
-}
-
-typedef gboolean (*GtkSignal_BOOL__INT_POINTER_INT_POINTER_INT_INT_INT) (GtkObject * object,
-									 gint arg1, 
-									 gpointer arg2,
-									 gint arg3,
-									 gpointer arg4,
-									 gint arg5,
-									 gint arg6,
-									 gint arg7,
-									 gpointer user_data);
-void
-e_marshal_BOOL__INT_POINTER_INT_POINTER_INT_INT_INT (GtkObject * object,
-						     GtkSignalFunc func,
-						     gpointer func_data, GtkArg * args)
-{
-  GtkSignal_BOOL__INT_POINTER_INT_POINTER_INT_INT_INT rfunc;
-  gboolean *return_val;
-  return_val = GTK_RETLOC_BOOL (args[7]);
-  rfunc = (GtkSignal_BOOL__INT_POINTER_INT_POINTER_INT_INT_INT) func;
-  *return_val = (*rfunc) (object,
-			  GTK_VALUE_INT (args[0]),
-			  GTK_VALUE_POINTER (args[1]),
-			  GTK_VALUE_INT (args[2]), 
-			  GTK_VALUE_POINTER (args[3]),
-			  GTK_VALUE_INT (args[4]),
-			  GTK_VALUE_INT (args[5]),
-			  GTK_VALUE_INT (args[6]), func_data);
-}
-
-typedef void (*GtkSignal_NONE__INT_INT_POINTER_INT_INT_POINTER_INT_INT) (GtkObject *
-									 object,
-									 gint arg1, 
-									 gint arg2,
-									 gpointer
-									 arg3,
-									 gint arg4,
-									 gint arg5,
-									 gpointer
-									 arg6,
-									 gint arg7,
-									 gint arg8,
-									 gpointer
-									 user_data);
-
-void
-e_marshal_NONE__INT_INT_POINTER_INT_INT_POINTER_INT_INT (GtkObject * object,
-						   GtkSignalFunc func,
-						   gpointer func_data,
-						   GtkArg * args)
-{
-  GtkSignal_NONE__INT_INT_POINTER_INT_INT_POINTER_INT_INT rfunc;
-  rfunc = (GtkSignal_NONE__INT_INT_POINTER_INT_INT_POINTER_INT_INT) func;
-  (*rfunc) (object,
-	    GTK_VALUE_INT (args[0]), GTK_VALUE_INT (args[1]), 
-	    GTK_VALUE_POINTER (args[2]),
-	    GTK_VALUE_INT (args[3]),
-	    GTK_VALUE_INT (args[4]),
-	    GTK_VALUE_POINTER (args[5]),
-	    GTK_VALUE_INT (args[6]), GTK_VALUE_INT (args[7]), func_data);
-}
-
-typedef void (*GtkSignal_NONE__INT_POINTER_INT_POINTER_INT_INT_POINTER_INT_INT) (GtkObject *
-										 object,
-										 gint arg1, 
-										 gpointer arg2,
-										 gint arg3,
-										 gpointer arg4,
-										 gint arg5,
-										 gint arg6,
-										 gpointer arg7,
-										 gint arg8,
-										 gint arg9,
-										 gpointer user_data);
-
-void
-e_marshal_NONE__INT_POINTER_INT_POINTER_INT_INT_POINTER_INT_INT (GtkObject * object,
-								 GtkSignalFunc func,
-								 gpointer func_data,
-								 GtkArg * args)
-{
-  GtkSignal_NONE__INT_POINTER_INT_POINTER_INT_INT_POINTER_INT_INT rfunc;
-  rfunc = (GtkSignal_NONE__INT_POINTER_INT_POINTER_INT_INT_POINTER_INT_INT) func;
-  (*rfunc) (object,
-	    GTK_VALUE_INT (args[0]), 
-	    GTK_VALUE_POINTER (args[1]),
-	    GTK_VALUE_INT (args[2]), 
-	    GTK_VALUE_POINTER (args[3]),
-	    GTK_VALUE_INT (args[4]),
-	    GTK_VALUE_INT (args[5]),
-	    GTK_VALUE_POINTER (args[6]),
-	    GTK_VALUE_INT (args[7]), GTK_VALUE_INT (args[8]), func_data);
-}
-
-typedef void (*GtkSignal_NONE__POINTER_POINTER_INT) (GtkObject *, gpointer,
-						     gpointer, gint, gpointer);
-
-void
-e_marshal_NONE__POINTER_POINTER_INT (GtkObject * object, GtkSignalFunc func,
-				     gpointer func_data, GtkArg * args)
-{
-  GtkSignal_NONE__POINTER_POINTER_INT rfunc;
-  rfunc = (GtkSignal_NONE__POINTER_POINTER_INT) func;
-  (*rfunc) (object, GTK_VALUE_POINTER (args[0]), GTK_VALUE_POINTER (args[1]),
-	    GTK_VALUE_INT (args[2]), func_data);
-}	
-
-typedef void (*GtkSignal_NONE__INT_POINTER_INT_POINTER) (GtkObject *, gint, gpointer,
-							 gint, gpointer, gpointer);
-
-void
-e_marshal_NONE__INT_POINTER_INT_POINTER (GtkObject * object, GtkSignalFunc func,
-					 gpointer func_data, GtkArg * args)
-{
-  GtkSignal_NONE__INT_POINTER_INT_POINTER rfunc;
-  rfunc = (GtkSignal_NONE__INT_POINTER_INT_POINTER) func;
-  (*rfunc) (object, GTK_VALUE_INT (args[0]), GTK_VALUE_POINTER (args[1]),
-	    GTK_VALUE_INT (args[2]), GTK_VALUE_POINTER (args[3]), func_data);
-}	
-
-typedef int (*GtkSignal_INT__POINTER_POINTER) (GtkObject *,
-					       gpointer, gpointer, 
-					       gpointer user_data);
-void
-e_marshal_INT__POINTER_POINTER (GtkObject *object,
-				GtkSignalFunc func,
-				gpointer func_data,
-				GtkArg *args)
-{
-	GtkSignal_INT__POINTER_POINTER rfunc;
-	int *return_val;
-
-	rfunc = (GtkSignal_INT__POINTER_POINTER) func;
-	return_val = GTK_RETLOC_INT (args[2]);
-
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_POINTER (args[0]),
-				GTK_VALUE_POINTER (args[1]), 
-				func_data);
-}
-
-typedef int (*GtkSignal_INT__POINTER_POINTER_POINTER) (GtkObject *,
-						       gpointer, gpointer, gpointer, 
-						       gpointer user_data);
-void
-e_marshal_INT__POINTER_POINTER_POINTER (GtkObject *object,
-					GtkSignalFunc func,
-					gpointer func_data,
-					GtkArg *args)
-{
-	GtkSignal_INT__POINTER_POINTER_POINTER rfunc;
-	int *return_val;
-
-	rfunc = (GtkSignal_INT__POINTER_POINTER_POINTER) func;
-	return_val = GTK_RETLOC_INT (args[3]);
-
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_POINTER (args[0]),
-				GTK_VALUE_POINTER (args[1]), 
-				GTK_VALUE_POINTER (args[2]),
-				func_data);
-}
-
-typedef int (*GtkSignal_INT__POINTER_POINTER_POINTER_POINTER) (GtkObject *,
-							       gpointer, gpointer, gpointer, gpointer,
-							       gpointer user_data);
-void
-e_marshal_INT__POINTER_POINTER_POINTER_POINTER (GtkObject *object,
-						GtkSignalFunc func,
-						gpointer func_data,
-						GtkArg *args)
-{
-	GtkSignal_INT__POINTER_POINTER_POINTER_POINTER rfunc;
-	int *return_val;
-
-	rfunc = (GtkSignal_INT__POINTER_POINTER_POINTER_POINTER) func;
-	return_val = GTK_RETLOC_INT (args[4]);
-
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_POINTER (args[0]),
-				GTK_VALUE_POINTER (args[1]), 
-				GTK_VALUE_POINTER (args[2]),
-				GTK_VALUE_POINTER (args[3]),
-				func_data);
-}
-
-
-typedef int (*GtkSignal_INT__POINTER_POINTER_POINTER_POINTER_POINTER) (GtkObject *,
-								       gpointer, gpointer, gpointer, gpointer, gpointer,
-								       gpointer user_data);
-void
-e_marshal_INT__POINTER_POINTER_POINTER_POINTER_POINTER (GtkObject *object,
-							GtkSignalFunc func,
-							gpointer func_data,
-							GtkArg *args)
-{
-	GtkSignal_INT__POINTER_POINTER_POINTER_POINTER_POINTER rfunc;
-	int *return_val;
-
-	rfunc = (GtkSignal_INT__POINTER_POINTER_POINTER_POINTER_POINTER) func;
-	return_val = GTK_RETLOC_INT (args[4]);
-
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_POINTER (args[0]),
-				GTK_VALUE_POINTER (args[1]), 
-				GTK_VALUE_POINTER (args[2]),
-				GTK_VALUE_POINTER (args[3]),
-				GTK_VALUE_POINTER (args[4]),
-				func_data);
-}
-
-typedef void (*GtkSignal_NONE__POINTER_POINTER_POINTER_BOOL) (GtkObject *,
-							      gpointer, gpointer, gpointer, gboolean,
-							      gpointer user_data);
-
-void
-e_marshal_NONE__POINTER_POINTER_POINTER_BOOL (GtkObject *object,
-					      GtkSignalFunc func,
-					      gpointer func_data,
-					      GtkArg *args)
-{
-	GtkSignal_NONE__POINTER_POINTER_POINTER_BOOL rfunc;
-
-	rfunc = (GtkSignal_NONE__POINTER_POINTER_POINTER_BOOL) func;
-
-	(*rfunc) (object,
-		  GTK_VALUE_POINTER (args[0]),
-		  GTK_VALUE_POINTER (args[1]), 
-		  GTK_VALUE_POINTER (args[2]),
-		  GTK_VALUE_BOOL (args[3]),
-		  func_data);
-}
-
-void
-e_marshal_NONE__POINTER_INT_INT_INT (GtkObject       *object,
-				     GtkSignalFunc    func,
-				     gpointer         func_data,
-				     GtkArg          *args)
-{
-        (* (void (*)(GtkObject *, gpointer, int, int, int, gpointer)) func)
-                (object,
-		 GTK_VALUE_POINTER (args[0]),
-                 GTK_VALUE_INT (args[1]),
-                 GTK_VALUE_INT (args[2]),
-                 
-                 GTK_VALUE_INT (args[3]),
-                 func_data);
-}
-
-typedef int (*GtkSignal_INT__OBJECT_POINTER) (GtkObject *,
-					      GtkObject *, gpointer,
-					      gpointer user_data);
-void
-e_marshal_INT__OBJECT_POINTER (GtkObject *object,
-			       GtkSignalFunc func,
-			       gpointer func_data,
-			       GtkArg *args)
-{
-	GtkSignal_INT__OBJECT_POINTER rfunc;
-	int *return_val;
-
-	rfunc = (GtkSignal_INT__OBJECT_POINTER) func;
-	return_val = GTK_RETLOC_INT (args[2]);
-
-	*return_val = (*rfunc) (object,
-				GTK_VALUE_OBJECT  (args[0]),
-				GTK_VALUE_POINTER (args[1]),
-				func_data);
-}
+#include "e-marshal.h"
+#include "e-marshal.c"
 
 gchar**
 e_strsplit (const gchar *string,
@@ -877,6 +417,7 @@ e_strstrcase (const gchar *haystack, const gchar *needle)
 	return NULL;
 }
 
+/* This only makes a filename safe for usage as a filename.  It still may have shell meta-characters in it. */
 void
 e_filename_make_safe (gchar *string)
 {
@@ -962,10 +503,97 @@ e_format_number (gint number)
 	}
 }
 
+static gchar *
+do_format_number_as_float (double number)
+{
+	GList *iterator, *list = NULL;
+	struct lconv *locality;
+	gint char_length = 0;
+	gint group_count = 0;
+	guchar *grouping;
+	int last_count = 3;
+	int divider;
+	char *value;
+	char *value_iterator;
+	double fractional;
+
+	locality = localeconv();
+	grouping = locality->grouping;
+	while (number >= 1.0) {
+		char *group;
+		switch (*grouping) {
+		default:
+			last_count = *grouping;
+			grouping++;
+			/* Fall through */
+		case 0:
+			divider = epow10(last_count);
+			number /= divider;
+			fractional = modf (number, &number);
+			fractional *= divider;
+			fractional = floor (fractional);
+
+			if (number >= 1.0) {
+				group = g_strdup_printf("%0*d", last_count, (int) fractional);
+			} else {
+				group = g_strdup_printf("%d", (int) fractional);
+			}
+			break;
+		case CHAR_MAX:
+			divider = epow10(last_count);
+			number /= divider;
+			fractional = modf (number, &number);
+			fractional *= divider;
+			fractional = floor (fractional);
+
+			while (number >= 1.0) {
+				group = g_strdup_printf("%0*d", last_count, (int) fractional);
+
+				char_length += strlen(group);
+				list = g_list_prepend(list, group);
+				group_count ++;
+
+				divider = epow10(last_count);
+				number /= divider;
+				fractional = modf (number, &number);
+				fractional *= divider;
+				fractional = floor (fractional);
+			}
+
+			group = g_strdup_printf("%d", (int) fractional);
+			break;
+		}
+		char_length += strlen(group);
+		list = g_list_prepend(list, group);
+		group_count ++;
+	}
+
+	if (list) {
+		value = g_new(char, 1 + char_length + (group_count - 1) * strlen(locality->thousands_sep));
+
+		iterator = list;
+		value_iterator = value;
+
+		strcpy(value_iterator, iterator->data);
+		value_iterator += strlen(iterator->data);
+		for (iterator = iterator->next; iterator; iterator = iterator->next) {
+			strcpy(value_iterator, locality->thousands_sep);
+			value_iterator += strlen(locality->thousands_sep);
+
+			strcpy(value_iterator, iterator->data);
+			value_iterator += strlen(iterator->data);
+		}
+		e_free_string_list (list);
+		return value;
+	} else {
+		return g_strdup("0");
+	}
+}
+
 gchar *
 e_format_number_float (gfloat number)
 {
-	gint            int_part;
+	gfloat          int_part;
 	gint            fraction;
 	struct lconv   *locality;
 	gchar          *str_intpart;
@@ -975,8 +603,8 @@ e_format_number_float (gfloat number)
 
 	locality = localeconv();
 	
-	int_part = (int) number;
-	str_intpart = e_format_number (int_part);
+	int_part = floor (number);
+	str_intpart = do_format_number_as_float ((double) int_part);
 
 	if (!strcmp(locality->mon_decimal_point, "")) {
 		decimal_point = ".";
@@ -989,8 +617,7 @@ e_format_number_float (gfloat number)
 
 	if (fraction == 0) {
 		str_fraction = g_strdup ("00");
-	}
-	else {
+	} else {
 		str_fraction = g_strdup_printf ("%02d", fraction);
 	}
 
@@ -1005,63 +632,11 @@ e_format_number_float (gfloat number)
 gboolean
 e_create_directory (gchar *directory)
 {
-	gchar *full_name;
-	gchar *position;
-	gchar *current_dir = g_get_current_dir();
-	struct stat info;
-	gboolean return_value = TRUE;
-
-	if (directory[0] == '/') {
-		full_name = g_malloc0 (strlen (directory) + 1);
-		strcpy (full_name, directory);
-	} else {
-		full_name = g_malloc0 (strlen (directory) + strlen (current_dir) + 2);
-		sprintf (full_name, "%s/%s", current_dir, directory);
-	}
-
-	if ((position = strrchr (full_name, '/')) == full_name) {
-		if (stat (full_name, &info)) {
-			switch (errno) {
-			case ENOENT:
-				if (mkdir (full_name, 0777)) {
-					switch (errno) {
-					default:
-						return_value = FALSE;
-						break;
-					}
-				}
-				break;
-			default:
-				return_value = FALSE;
-				break;
-			}
-		}
-	} else {
-		*position = 0;
-		e_create_directory (full_name);
-		*position = '/';
-		if (stat (full_name, &info)) {
-			switch (errno) {
-			case ENOENT:
-				if (mkdir (full_name, 0777)) {
-					switch (errno) {
-					default:
-						return_value = FALSE;
-						break;
-					}
-				}
-				break;
-			default:
-				return_value = FALSE;
-				break;
-			}
-		}
-	}
-
-	g_free (current_dir);
-	g_free (full_name);
-
-	return (return_value);
+	gint ret_val = e_mkdir_hier (directory, 0777);
+	if (ret_val == -1)
+		return FALSE;
+	else
+		return TRUE;
 }
 
 
@@ -1535,3 +1110,44 @@ e_strdup_append_strings (gchar *first_string, ...)
 
 	return buffer;
 }
+
+gchar **
+e_strdupv (const gchar **str_array)
+{
+	if (str_array) {
+		gint i;
+		gchar **retval;
+
+		i = 0;
+		while (str_array[i])
+			i++;
+          
+		retval = g_new (gchar*, i + 1);
+
+		i = 0;
+		while (str_array[i]) {
+			retval[i] = g_strdup (str_array[i]);
+			i++;
+		}
+		retval[i] = NULL;
+
+		return retval;
+	} else {
+		return NULL;
+	}
+}
+
+char *
+e_gettext (const char *msgid)
+{
+	static gboolean initialized = FALSE;
+
+	if (!initialized) {
+		bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+		initialized = TRUE;
+	}        
+
+	return dgettext (GETTEXT_PACKAGE, msgid);
+}
+
