@@ -24,9 +24,11 @@
 #include <gtk/gtksignal.h>
 #include "e-canvas.h"
 #include "gal/util/e-util.h"
+#include <X11/Xlib.h>
+#include <gtk/gtkmain.h>
 
 static void e_canvas_init           (ECanvas         *card);
-static void e_canvas_destroy        (GtkObject        *object);
+static void e_canvas_dispose        (GObject        *object);
 static void e_canvas_class_init	    (ECanvasClass    *klass);
 static void e_canvas_realize        (GtkWidget        *widget);
 static void e_canvas_unrealize      (GtkWidget        *widget);
@@ -44,8 +46,12 @@ static gint e_canvas_focus_in       (GtkWidget        *widget,
 static gint e_canvas_focus_out      (GtkWidget        *widget,
 				     GdkEventFocus    *event);
 
+static void e_canvas_style_set      (GtkWidget        *widget,
+				     GtkStyle         *previous_style);
+
 static int emit_event (GnomeCanvas *canvas, GdkEvent *event);
 
+#define PARENT_TYPE GNOME_TYPE_CANVAS
 static GnomeCanvasClass *parent_class = NULL;
 
 #define d(x)
@@ -57,66 +63,48 @@ enum {
 
 static guint e_canvas_signals [LAST_SIGNAL] = { 0, };
 
-GtkType
-e_canvas_get_type (void)
-{
-	static GtkType canvas_type = 0;
-
-	if (!canvas_type)
-	{
-		static const GtkTypeInfo canvas_info =
-		{
-			"ECanvas",
-			sizeof (ECanvas),
-			sizeof (ECanvasClass),
-			(GtkClassInitFunc) e_canvas_class_init,
-			(GtkObjectInitFunc) e_canvas_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		canvas_type = gtk_type_unique (gnome_canvas_get_type (), &canvas_info);
-	}
-
-	return canvas_type;
-}
+E_MAKE_TYPE (e_canvas,
+	     "ECanvas",
+	     ECanvas,
+	     e_canvas_class_init,
+	     e_canvas_init,
+	     PARENT_TYPE)
 
 static void
 e_canvas_class_init (ECanvasClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GnomeCanvasClass *canvas_class;
 	GtkWidgetClass *widget_class;
 
-	object_class = (GtkObjectClass*) klass;
-	canvas_class = (GnomeCanvasClass *) klass;
-	widget_class = (GtkWidgetClass *) klass;
+	object_class                       = (GObjectClass*) klass;
+	canvas_class                       = (GnomeCanvasClass *) klass;
+	widget_class                       = (GtkWidgetClass *) klass;
 
-	parent_class = gtk_type_class (gnome_canvas_get_type ());
+	parent_class                       = g_type_class_ref (PARENT_TYPE);
 
-	object_class->destroy = e_canvas_destroy;
+	object_class->dispose              = e_canvas_dispose;
 
-	widget_class->key_press_event = e_canvas_key;
-	widget_class->key_release_event = e_canvas_key;
-	widget_class->button_press_event = e_canvas_button;
+	widget_class->key_press_event      = e_canvas_key;
+	widget_class->key_release_event    = e_canvas_key;
+	widget_class->button_press_event   = e_canvas_button;
 	widget_class->button_release_event = e_canvas_button;
-	widget_class->focus_in_event = e_canvas_focus_in;
-	widget_class->focus_out_event = e_canvas_focus_out;
-	widget_class->realize = e_canvas_realize;
-	widget_class->unrealize = e_canvas_unrealize;
+	widget_class->focus_in_event       = e_canvas_focus_in;
+	widget_class->focus_out_event      = e_canvas_focus_out;
+	widget_class->style_set            = e_canvas_style_set;
+	widget_class->realize              = e_canvas_realize;
+	widget_class->unrealize            = e_canvas_unrealize;
 
-	klass->reflow = NULL;
+	klass->reflow                      = NULL;
 
 	e_canvas_signals [REFLOW] =
-		gtk_signal_new ("reflow",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (ECanvasClass, reflow),
-				gtk_marshal_NONE__NONE,
-				GTK_TYPE_NONE, 0);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_canvas_signals, LAST_SIGNAL);
+		g_signal_new ("reflow",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ECanvasClass, reflow),
+			      NULL, NULL,
+			      e_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 }
 
 static void
@@ -124,13 +112,15 @@ e_canvas_init (ECanvas *canvas)
 {
 	canvas->selection = NULL;
 	canvas->cursor = NULL;
+#ifdef GAL_GDK_IM
 	canvas->ic = NULL;
 	canvas->ic_attr = NULL;
+#endif
 	canvas->tooltip_window = NULL;
 }
 
 static void
-e_canvas_destroy (GtkObject *object)
+e_canvas_dispose (GObject *object)
 {
 	ECanvas *canvas = E_CANVAS(object);
 
@@ -140,24 +130,24 @@ e_canvas_destroy (GtkObject *object)
 
 	if (canvas->toplevel) {
 		if (canvas->visibility_notify_id)
-			gtk_signal_disconnect (GTK_OBJECT(canvas->toplevel),
-					       canvas->visibility_notify_id);
+			g_signal_handler_disconnect (canvas->toplevel,
+						     canvas->visibility_notify_id);
 		canvas->visibility_notify_id = 0;
 
-		gtk_object_unref(GTK_OBJECT(canvas->toplevel));
+		g_object_unref (canvas->toplevel);
 		canvas->toplevel = NULL;
 	}
 
 	e_canvas_hide_tooltip(canvas);
 
-	if ((GTK_OBJECT_CLASS (parent_class))->destroy)
-		(*(GTK_OBJECT_CLASS (parent_class))->destroy) (object);
+	if ((G_OBJECT_CLASS (parent_class))->dispose)
+		(*(G_OBJECT_CLASS (parent_class))->dispose) (object);
 }
 
 GtkWidget *
 e_canvas_new ()
 {
-	return GTK_WIDGET (gtk_type_new (e_canvas_get_type ()));
+	return GTK_WIDGET (g_object_new (E_CANVAS_TYPE, NULL));
 }
 
 
@@ -263,17 +253,22 @@ emit_event (GnomeCanvas *canvas, GdkEvent *event)
 	finished = FALSE;
 
 	while (item && !finished) {
-		gtk_object_ref (GTK_OBJECT (item));
+		g_object_ref (item);
 
-		gtk_signal_emit_by_name (GTK_OBJECT (item), "event",
-					 &ev,
-					 &finished);
+		g_signal_emit_by_name (item, "event",
+				       &ev,
+				       &finished);
 
+#ifndef NO_WARNINGS
+#warning FIXME - needs thought
+#endif
+#if 0
 		if (GTK_OBJECT_DESTROYED (item))
 			finished = TRUE;
+#endif
 
 		parent = item->parent;
-		gtk_object_unref (GTK_OBJECT (item));
+		g_object_unref (item);
 
 		item = parent;
 	}
@@ -327,7 +322,7 @@ gnome_canvas_item_invoke_point (GnomeCanvasItem *item, double x, double y, int c
 	y = i.y;
 #endif
 
-	return (* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->point) (
+	return (* GNOME_CANVAS_ITEM_CLASS (GTK_OBJECT_GET_CLASS (item))->point) (
 		item, x, y, cx, cy, actual_item);
 }
 
@@ -403,11 +398,11 @@ pick_current_item (GnomeCanvas *canvas, GdkEvent *event)
 		/* these fields don't have the same offsets in both types of events */
 
 		if (canvas->pick_event.type == GDK_ENTER_NOTIFY) {
-			x = canvas->pick_event.crossing.x + DISPLAY_X1 (canvas) - canvas->zoom_xofs;
-			y = canvas->pick_event.crossing.y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs;
+			x = canvas->pick_event.crossing.x + canvas->scroll_x1 - canvas->zoom_xofs;
+			y = canvas->pick_event.crossing.y + canvas->scroll_y1 - canvas->zoom_yofs;
 		} else {
-			x = canvas->pick_event.motion.x + DISPLAY_X1 (canvas) - canvas->zoom_xofs;
-			y = canvas->pick_event.motion.y + DISPLAY_Y1 (canvas) - canvas->zoom_yofs;
+			x = canvas->pick_event.motion.x + canvas->scroll_x1 - canvas->zoom_xofs;
+			y = canvas->pick_event.motion.y + canvas->scroll_y1 - canvas->zoom_yofs;
 		}
 
 		/* canvas pixel coords */
@@ -629,8 +624,10 @@ e_canvas_focus_in (GtkWidget *widget, GdkEventFocus *event)
 
 	GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
 
+#ifdef GAL_GDK_IM
 	if (ecanvas->ic)
 		gdk_im_begin (ecanvas->ic, canvas->layout.bin_window);
+#endif
 
 	if (canvas->focused_item) {
 		full_event.focus_change = *event;
@@ -653,8 +650,10 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 
 	GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
 
+#ifdef GAL_GDK_IM
 	if (ecanvas->ic)
 		gdk_im_end ();
+#endif
 
 	if (canvas->focused_item) {
 		full_event.focus_change = *event;
@@ -664,17 +663,46 @@ e_canvas_focus_out (GtkWidget *widget, GdkEventFocus *event)
 	}
 }
 
+static void
+ec_style_set_recursive (GnomeCanvasItem *item, GtkStyle *previous_style)
+{
+	guint signal_id = g_signal_lookup ("style_set", G_OBJECT_TYPE (item));
+	if (signal_id >= 1) {
+		GSignalQuery query;
+		g_signal_query (signal_id, &query);
+		if (query.return_type == GTK_TYPE_NONE && query.n_params == 1 && query.param_types[0] == GTK_TYPE_STYLE) {
+			g_signal_emit (item, signal_id, 0, previous_style);
+		}
+	}
+
+	if (GNOME_IS_CANVAS_GROUP (item) ) {
+		GList *items = GNOME_CANVAS_GROUP (item)->item_list;
+		for (; items; items = items->next)
+			ec_style_set_recursive (items->data, previous_style);
+	}
+}
+
+static void
+e_canvas_style_set (GtkWidget *widget, GtkStyle *previous_style)
+{
+	ec_style_set_recursive (GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (widget))), previous_style);
+}
+
+
 static void
 e_canvas_realize (GtkWidget *widget)
 {
 	gint width, height;
+#ifdef GAL_GDK_IM
 	ECanvas *ecanvas = E_CANVAS (widget);
+#endif
 
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
 
 	gdk_window_set_back_pixmap (GTK_LAYOUT (widget)->bin_window, NULL, FALSE);
 
+#ifdef GAL_GDK_IM
 	if (gdk_im_ready () && (ecanvas->ic_attr = gdk_ic_attr_new ()) != NULL) {
 		GdkEventMask mask;
 		GdkICAttr *attr = ecanvas->ic_attr;
@@ -728,7 +756,7 @@ e_canvas_realize (GtkWidget *widget)
 		} else
 			g_warning ("Can't create input context.");
 	}
-
+#endif
 }
 
 static void
@@ -736,10 +764,12 @@ e_canvas_unrealize (GtkWidget *widget)
 {
 	ECanvas * ecanvas = E_CANVAS (widget);
 
-	if (ecanvas->idle_id)
+	if (ecanvas->idle_id) {
 		g_source_remove(ecanvas->idle_id);
-	ecanvas->idle_id = 0;
+		ecanvas->idle_id = 0;
+	}
 
+#ifdef GAL_GDK_IM
 	if (ecanvas->ic) {
 		gdk_ic_destroy (ecanvas->ic);
 		ecanvas->ic = NULL;
@@ -748,6 +778,7 @@ e_canvas_unrealize (GtkWidget *widget)
 		gdk_ic_attr_destroy (ecanvas->ic_attr);
 		ecanvas->ic_attr = NULL;
 	}
+#endif
 	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
 		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
@@ -770,8 +801,9 @@ e_canvas_item_invoke_reflow (GnomeCanvasItem *item, int flags)
 
 	if (item->object.flags & E_CANVAS_ITEM_NEEDS_REFLOW) {
 		ECanvasItemReflowFunc func;
-		func = gtk_object_get_data (GTK_OBJECT (item),
-					    "ECanvasItem::reflow_callback");
+		func = (ECanvasItemReflowFunc)
+			g_object_get_data (G_OBJECT (item),
+					   "ECanvasItem::reflow_callback");
 		if (func)
 			func (item, flags);
 	}
@@ -801,8 +833,8 @@ idle_handler (gpointer data)
 	/* Reset idle id */
 	canvas->idle_id = 0;
 
-	gtk_signal_emit (GTK_OBJECT (canvas),
-			 e_canvas_signals [REFLOW]);
+	g_signal_emit (canvas,
+		       e_canvas_signals [REFLOW], 0);
 
 	GDK_THREADS_LEAVE();
 
@@ -851,20 +883,20 @@ e_canvas_item_request_parent_reflow (GnomeCanvasItem *item)
 void
 e_canvas_item_set_reflow_callback (GnomeCanvasItem *item, ECanvasItemReflowFunc func)
 {
-	gtk_object_set_data(GTK_OBJECT(item), "ECanvasItem::reflow_callback", (gpointer) func);
+	g_object_set_data(G_OBJECT(item), "ECanvasItem::reflow_callback", (gpointer) func);
 }
 
 
 void
 e_canvas_item_set_selection_callback (GnomeCanvasItem *item, ECanvasItemSelectionFunc func)
 {
-	gtk_object_set_data(GTK_OBJECT(item), "ECanvasItem::selection_callback", (gpointer) func);
+	g_object_set_data(G_OBJECT(item), "ECanvasItem::selection_callback", (gpointer) func);
 }
 
 void
 e_canvas_item_set_selection_compare_callback (GnomeCanvasItem *item, ECanvasItemSelectionCompareFunc func)
 {
-	gtk_object_set_data(GTK_OBJECT(item), "ECanvasItem::selection_compare_callback", (gpointer) func);
+	g_object_set_data(G_OBJECT(item), "ECanvasItem::selection_compare_callback", (gpointer) func);
 }
 
 void
@@ -887,12 +919,13 @@ e_canvas_item_set_cursor (GnomeCanvasItem *item, gpointer id)
 	for (list = canvas->selection; list; list = g_list_next(list)) {
 		info = list->data;
 
-		func = gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
+		func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(info->item),
+								   "ECanvasItem::selection_callback");
 		if (func)
 			func(info->item, flags, info->id);
 		g_message ("ECANVAS: free info (2): item %p, id %p",
 			   info->item, info->id);
-		gtk_object_unref (GTK_OBJECT (info->item));
+		g_object_unref (info->item);
 		g_free(info);
 	}
 	g_list_free(canvas->selection);
@@ -903,12 +936,13 @@ e_canvas_item_set_cursor (GnomeCanvasItem *item, gpointer id)
 
 	info = g_new(ECanvasSelectionInfo, 1);
 	info->item = item;
-	gtk_object_ref (GTK_OBJECT (info->item));
+	g_object_ref (info->item);
 	info->id = id;
 	g_message ("ECANVAS: new info item %p, id %p", item, id);
 
 	flags = E_CANVAS_ITEM_SELECTION_SELECT | E_CANVAS_ITEM_SELECTION_CURSOR;
-	func = gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+	func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(item),
+							   "ECanvasItem::selection_callback");
 	if (func)
 		func(item, flags, id);
 
@@ -939,7 +973,8 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 	canvas = E_CANVAS(item->canvas);
 
 	if (canvas->cursor) {
-		func = gtk_object_get_data(GTK_OBJECT(canvas->cursor->item), "ECanvasItem::selection_callback");
+		func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(canvas->cursor->item),
+								   "ECanvasItem::selection_callback");
 		if (func)
 			func(canvas->cursor->item, flags, canvas->cursor->id);
 	}
@@ -954,11 +989,13 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 
 		if (search->item == item) {
 			ECanvasItemSelectionCompareFunc compare_func;
-			compare_func = gtk_object_get_data(GTK_OBJECT(search->item), "ECanvasItem::selection_compare_callback");
+			compare_func = (ECanvasItemSelectionCompareFunc)g_object_get_data(G_OBJECT(search->item),
+											  "ECanvasItem::selection_compare_callback");
 
 			if (compare_func(search->item, search->id, id, 0) == 0) {
 				canvas->cursor = search;
-				func = gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+				func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(item),
+										   "ECanvasItem::selection_callback");
 				if (func)
 					func(item, flags, search->id);
 				return;
@@ -968,11 +1005,12 @@ e_canvas_item_add_selection (GnomeCanvasItem *item, gpointer id)
 
 	info = g_new(ECanvasSelectionInfo, 1);
 	info->item = item;
-	gtk_object_ref (GTK_OBJECT (info->item));
+	g_object_ref (info->item);
 	info->id = id;
 	g_message ("ECANVAS: new info (2): item %p, id %p", item, id);
 
-	func = gtk_object_get_data(GTK_OBJECT(item), "ECanvasItem::selection_callback");
+	func = (ECanvasItemSelectionFunc)g_object_get_data(G_OBJECT(item),
+							   "ECanvasItem::selection_callback");
 	if (func)
 		func(item, flags, id);
 
@@ -1001,11 +1039,13 @@ e_canvas_item_remove_selection (GnomeCanvasItem *item, gpointer id)
 
 		if (info->item == item) {
 			ECanvasItemSelectionCompareFunc compare_func;
-			compare_func = gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_compare_callback");
+			compare_func = (ECanvasItemSelectionCompareFunc)g_object_get_data(G_OBJECT(info->item),
+											  "ECanvasItem::selection_compare_callback");
 
 			if (compare_func(info->item, info->id, id, 0) == 0) {
 				ECanvasItemSelectionFunc func;
-				func = gtk_object_get_data(GTK_OBJECT(info->item), "ECanvasItem::selection_callback");
+				func = (ECanvasItemSelectionFunc) g_object_get_data(G_OBJECT(info->item),
+										    "ECanvasItem::selection_callback");
 				if (func)
 					func(info->item, flags, info->id);
 				canvas->selection = g_list_remove_link(canvas->selection, list);
@@ -1015,7 +1055,7 @@ e_canvas_item_remove_selection (GnomeCanvasItem *item, gpointer id)
 
 				g_message ("ECANVAS: removing info: item %p, info %p",
 					   info->item, info->id);
-				gtk_object_unref (GTK_OBJECT (info->item));
+				g_object_unref (info->item);
 				g_free(info);
 				g_list_free_1(list);
 				break;
@@ -1035,13 +1075,14 @@ void e_canvas_popup_tooltip (ECanvas *canvas, GtkWidget *widget, int x, int y)
 		canvas->toplevel = gtk_widget_get_toplevel (GTK_WIDGET(canvas));
 		if (canvas->toplevel) {
 			gtk_widget_add_events(canvas->toplevel, GDK_VISIBILITY_NOTIFY_MASK);
-			gtk_object_ref(GTK_OBJECT(canvas->toplevel));
+			g_object_ref (canvas->toplevel);
 			canvas->visibility_notify_id =
-				gtk_signal_connect (GTK_OBJECT (canvas->toplevel), "visibility_notify_event",
-						    GTK_SIGNAL_FUNC (e_canvas_visibility), canvas);
+				g_signal_connect (canvas->toplevel, "visibility_notify_event",
+						  G_CALLBACK (e_canvas_visibility), canvas);
 		}
 	}
-	gtk_widget_popup (widget, x, y);
+	gtk_widget_set_uposition (widget, x, y);
+	gtk_widget_show (widget);
 }
 
 void e_canvas_hide_tooltip  (ECanvas *canvas)
@@ -1051,3 +1092,77 @@ void e_canvas_hide_tooltip  (ECanvas *canvas)
 		canvas->tooltip_window = NULL;
 	}
 }
+
+
+static gboolean
+grab_cancelled_check (gpointer data)
+{
+	ECanvas *canvas = data;
+
+	if (GNOME_CANVAS (canvas)->grabbed_item == NULL) {
+		canvas->grab_cancelled_cb = NULL;
+		canvas->grab_cancelled_check_id = 0;
+		canvas->grab_cancelled_time = 0;
+		canvas->grab_cancelled_data = NULL;
+		return FALSE;
+	}
+
+	if (gtk_grab_get_current ()) {
+		gnome_canvas_item_ungrab(GNOME_CANVAS (canvas)->grabbed_item, canvas->grab_cancelled_time);
+		if (canvas->grab_cancelled_cb) {
+			canvas->grab_cancelled_cb (canvas,
+						   GNOME_CANVAS (canvas)->grabbed_item,
+						   canvas->grab_cancelled_data);
+		}
+		canvas->grab_cancelled_cb = NULL;
+		canvas->grab_cancelled_check_id = 0;
+		canvas->grab_cancelled_time = 0;
+		canvas->grab_cancelled_data = NULL;
+		return FALSE;
+	}
+	return TRUE;
+}
+
+int
+e_canvas_item_grab (ECanvas *canvas,
+		    GnomeCanvasItem *item,
+		    guint event_mask,
+		    GdkCursor *cursor,
+		    guint32 etime,
+		    ECanvasItemGrabCancelled cancelled_cb,
+		    gpointer cancelled_data)
+{
+	if (gtk_grab_get_current ()) {
+		return AlreadyGrabbed;
+	} else {
+		int ret_val = gnome_canvas_item_grab (item, event_mask, cursor, etime);
+		if (ret_val == GrabSuccess) {
+			canvas->grab_cancelled_cb = cancelled_cb;
+			canvas->grab_cancelled_check_id =
+				g_timeout_add_full (G_PRIORITY_LOW,
+						    100,
+						    grab_cancelled_check,
+						    canvas,
+						    NULL);
+			canvas->grab_cancelled_time = etime;
+			canvas->grab_cancelled_data = cancelled_data;
+		}
+
+		return ret_val;
+	}
+}
+
+void
+e_canvas_item_ungrab (ECanvas *canvas,
+		      GnomeCanvasItem *item,
+		      guint32 etime)
+{
+	if (canvas->grab_cancelled_check_id) {
+		g_source_remove (canvas->grab_cancelled_check_id);
+		canvas->grab_cancelled_cb = NULL;
+		canvas->grab_cancelled_check_id = 0;
+		canvas->grab_cancelled_time = 0;
+		canvas->grab_cancelled_data = NULL;
+		gnome_canvas_item_ungrab (item, etime);
+	}
+}
