@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-categories-master-list-dialog.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -30,7 +30,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkbutton.h>
-#include <libgnomeui/gnome-dialog.h>
+#include <gtk/gtkdialog.h>
 #include <glade/glade.h>
 #include <gal/util/e-util.h>
 #include <gal/e-table/e-table-scrolled.h>
@@ -48,49 +48,51 @@ struct ECategoriesMasterListDialogPriv {
 	GladeXML *gui;
 };
 
-#define PARENT_TYPE (gtk_object_get_type())
+#define PARENT_TYPE GTK_TYPE_OBJECT
 
 static GtkObjectClass *parent_class;
 
 /* The arguments we take */
 enum {
-	ARG_0,
-	ARG_ECML
+	PROP_0,
+	PROP_ECML
 };
 
 static void
-ecmld_destroy (GtkObject *object)
+ecmld_dispose (GObject *object)
 {
 	ECategoriesMasterListDialog *ecmld = E_CATEGORIES_MASTER_LIST_DIALOG (object);
 
-	if (ecmld->priv->ecml)
-		gtk_object_unref (GTK_OBJECT (ecmld->priv->ecml));
-	if (ecmld->priv->gui)
-		gtk_object_unref (GTK_OBJECT (ecmld->priv->gui));
-	g_free (ecmld->priv);
-	ecmld->priv = NULL;
+	if (ecmld->priv) {
+		if (ecmld->priv->ecml)
+			g_object_unref (ecmld->priv->ecml);
+		if (ecmld->priv->gui)
+			g_object_unref (ecmld->priv->gui);
+		g_free (ecmld->priv);
+		ecmld->priv = NULL;
+	}
 	
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
-ecmld_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ecmld_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	ECategoriesMasterListDialog *ecmld;
 	GtkWidget *scrolled;
 
-	ecmld = E_CATEGORIES_MASTER_LIST_DIALOG (o);
+	ecmld = E_CATEGORIES_MASTER_LIST_DIALOG (object);
 	
-	switch (arg_id){
-	case ARG_ECML:
+	switch (prop_id){
+	case PROP_ECML:
 		if (ecmld->priv->ecml) {
-			gtk_object_unref (GTK_OBJECT (ecmld->priv->ecml));
+			g_object_unref (ecmld->priv->ecml);
 		}
 
-		ecmld->priv->ecml = (ECategoriesMasterList *) GTK_VALUE_OBJECT (*arg);
+		ecmld->priv->ecml = (ECategoriesMasterList *) g_value_get_object (value);
 
 		if (ecmld->priv->ecml)
-			gtk_object_ref (GTK_OBJECT(ecmld->priv->ecml));
+			g_object_ref (ecmld->priv->ecml);
 
 		scrolled = glade_xml_get_widget (ecmld->priv->gui, "custom-etable");
 		if (scrolled && E_IS_TABLE_SCROLLED (scrolled)) {
@@ -98,61 +100,66 @@ ecmld_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 			ETableModel *model;
 
 			table = e_table_scrolled_get_table (E_TABLE_SCROLLED(scrolled));
-			gtk_object_get (GTK_OBJECT(table),
-					"model", &model,
-					NULL);
-			gtk_object_set (GTK_OBJECT(model),
-					"ecml", ecmld->priv->ecml,
-					NULL);
+			g_object_get (table,
+				      "model", &model,
+				      NULL);
+			g_object_set (model,
+				      "ecml", ecmld->priv->ecml,
+				      NULL);
 		}
 		break;
 	}
 }
 
 static void
-ecmld_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+ecmld_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	ECategoriesMasterListDialog *ecmld;
 
-	ecmld = E_CATEGORIES_MASTER_LIST_DIALOG (o);
+	ecmld = E_CATEGORIES_MASTER_LIST_DIALOG (object);
 
-	switch (arg_id) {
-	case ARG_ECML:
-		GTK_VALUE_OBJECT (*arg) = (GtkObject *) ecmld->priv->ecml;
+	switch (prop_id) {
+	case PROP_ECML:
+		g_value_set_object (value, ecmld->priv->ecml);
 		break;
 
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
 static void
-ecmld_class_init (GtkObjectClass *object_class)
+ecmld_class_init (GObjectClass *object_class)
 {
-	parent_class   = gtk_type_class (PARENT_TYPE);
+	parent_class   = g_type_class_ref (PARENT_TYPE);
 
-	object_class->destroy = ecmld_destroy;
-	object_class->set_arg = ecmld_set_arg;
-	object_class->get_arg = ecmld_get_arg;
+	object_class->dispose = ecmld_dispose;
+	object_class->set_property = ecmld_set_property;
+	object_class->get_property = ecmld_get_property;
 
-	gtk_object_add_arg_type ("ECategoriesMasterListDialog::ecml", E_CATEGORIES_MASTER_LIST_TYPE,
-				 GTK_ARG_READWRITE, ARG_ECML);
+	g_object_class_install_property (object_class, PROP_ECML,
+					 g_param_spec_object ("ecml",
+							      _( "ECML" ),
+							      _( "ECategoriesMasterListCombo" ),
+							      E_CATEGORIES_MASTER_LIST_TYPE,
+							      G_PARAM_READWRITE));
 
 	glade_gnome_init ();
 }
 
 static void
-dialog_destroyed (GtkObject *dialog, ECategoriesMasterListDialog *ecmld)
+dialog_destroyed (gpointer data, GObject *where_object_was)
 {
-	gtk_object_destroy (GTK_OBJECT (ecmld));
+	ECategoriesMasterListDialog *ecmld = data;
+	g_object_unref (ecmld);
 }
 
 static void
-dialog_clicked (GtkObject *dialog, int button, ECategoriesMasterListDialog *ecmld)
+dialog_response (GtkObject *dialog, int id, ECategoriesMasterListDialog *ecmld)
 {
 	GtkWidget *scrolled;
-	switch (button) {
+	switch (id) {
 	case 0:    /* Remove */
 		scrolled = glade_xml_get_widget (ecmld->priv->gui, "custom-etable");
 		if (scrolled && E_IS_TABLE_SCROLLED (scrolled)) {
@@ -168,8 +175,8 @@ dialog_clicked (GtkObject *dialog, int button, ECategoriesMasterListDialog *ecml
 			}
 		}
 		break;
-	case 1:    /* Close */
-		gnome_dialog_close (GNOME_DIALOG (dialog));
+	case GTK_RESPONSE_CLOSE:
+		gtk_widget_destroy (GTK_WIDGET (dialog));
 		break;
 	}
 }
@@ -181,8 +188,8 @@ connect_button (ECategoriesMasterListDialog *ecmld, GladeXML *gui, const char *w
 	GtkWidget *button = glade_xml_get_widget (gui, widget_name);
 
 	if (button && GTK_IS_BUTTON (button))
-		gtk_signal_connect(GTK_OBJECT (button), "clicked",
-				   GTK_SIGNAL_FUNC (cback), ecmld);
+		g_signal_connect(button, "clicked",
+				 G_CALLBACK (cback), ecmld);
 }
 #endif
 
@@ -222,19 +229,19 @@ create_ecmld_etable(gchar *name,
 static void
 setup_gui (ECategoriesMasterListDialog *ecmld)
 {
-	GladeXML *gui = glade_xml_new_with_domain
-		(GAL_GLADEDIR "/e-categories-master-list-dialog.glade", NULL, PACKAGE);
+	GladeXML *gui = glade_xml_new (
+		GAL_GLADEDIR "/e-categories-master-list-dialog.glade", NULL, PACKAGE);
 	GtkWidget *dialog;
 
 	ecmld->priv->gui = gui;
 
 	dialog = glade_xml_get_widget (gui, "dialog-ecmld");
 
-	gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-			    GTK_SIGNAL_FUNC (dialog_destroyed), ecmld);
-	if (dialog && GNOME_IS_DIALOG (dialog))
-		gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
-				    GTK_SIGNAL_FUNC (dialog_clicked), ecmld);
+	g_object_weak_ref (G_OBJECT (dialog),
+			   dialog_destroyed, ecmld);
+	if (dialog && GTK_IS_DIALOG (dialog))
+		g_signal_connect (dialog, "response",
+				  G_CALLBACK (dialog_response), ecmld);
 }
 
 static void
@@ -259,17 +266,17 @@ e_categories_master_list_dialog_construct (ECategoriesMasterListDialog *ecmld,
 	g_return_val_if_fail (ecml != NULL, NULL);
 
 	ecmld->priv->ecml = ecml;
-	gtk_object_ref (GTK_OBJECT(ecmld->priv->ecml));
+	g_object_ref (ecmld->priv->ecml);
 
 	scrolled = glade_xml_get_widget (ecmld->priv->gui, "custom-etable");
 	if (scrolled && E_IS_TABLE_SCROLLED (scrolled)) {
 		ETable *table = e_table_scrolled_get_table (E_TABLE_SCROLLED(scrolled));
-		gtk_object_get (GTK_OBJECT(table),
-				"model", &model,
-				NULL);
-		gtk_object_set (GTK_OBJECT(model),
-				"ecml", ecml,
-				NULL);
+		g_object_get (table,
+			      "model", &model,
+			      NULL);
+		g_object_set (model,
+			      "ecml", ecml,
+			      NULL);
 	}
 
 	return E_CATEGORIES_MASTER_LIST_DIALOG (ecmld);
@@ -285,7 +292,7 @@ e_categories_master_list_dialog_construct (ECategoriesMasterListDialog *ecmld,
 ECategoriesMasterListDialog *
 e_categories_master_list_dialog_new (ECategoriesMasterList *ecml)
 {
-	ECategoriesMasterListDialog *ecmld = gtk_type_new (E_CATEGORIES_MASTER_LIST_DIALOG_TYPE);
+	ECategoriesMasterListDialog *ecmld = g_object_new (E_CATEGORIES_MASTER_LIST_DIALOG_TYPE, NULL);
 	GtkWidget *dialog;
 
 	if (e_categories_master_list_dialog_construct (ecmld, ecml) == NULL){
