@@ -22,13 +22,13 @@
  */
 
 #include <config.h>
+#include <string.h>
 
 #include "e-categories.h"
 
-#include "gal/util/e-i18n.h"
-#include <string.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnomeui/gnome-stock.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkbox.h>
 
 #include "gal/e-table/e-table-scrolled.h"
 #include "gal/e-table/e-table.h"
@@ -57,14 +57,13 @@ struct _ECategoriesPriv {
 	gint         ecml_changed_id;
 
 	ECategoriesMasterListDialog *ecmld;
-	int ecmld_destroy_id;
 };
 
 static void e_categories_init		(ECategories		 *card);
 static void e_categories_class_init	(ECategoriesClass	 *klass);
-static void e_categories_set_arg         (GtkObject *o, GtkArg *arg, guint arg_id);
-static void e_categories_get_arg         (GtkObject *object, GtkArg *arg, guint arg_id);
-static void e_categories_destroy         (GtkObject *object);
+static void e_categories_set_property    (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void e_categories_get_property    (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void e_categories_dispose         (GObject *object);
 static int e_categories_col_count (ETableModel *etc, gpointer data);
 static int e_categories_row_count (ETableModel *etc, gpointer data);
 static void *e_categories_value_at (ETableModel *etc, int col, int row, gpointer data);
@@ -78,63 +77,59 @@ static void *e_categories_initialize_value (ETableModel *etc, int col, gpointer
 static gboolean e_categories_value_is_empty (ETableModel *etc, int col, const void *value, gpointer data);
 static char * e_categories_value_to_string (ETableModel *etc, int col, const void *value, gpointer data);
 static void e_categories_toggle (ECategories *categories, int row);
+static void e_categories_rebuild (ECategories *categories);
 
-static GnomeDialogClass *parent_class = NULL;
+#define PARENT_TYPE GTK_TYPE_DIALOG
+static GtkDialogClass *parent_class = NULL;
 
 /* The arguments we take */
 enum {
-	ARG_0,
-	ARG_CATEGORIES,
-	ARG_HEADER,
-	ARG_ECML,
+	PROP_0,
+	PROP_CATEGORIES,
+	PROP_HEADER,
+	PROP_ECML
 };
 
-GtkType
-e_categories_get_type (void)
-{
-	static GtkType contact_editor_categories_type = 0;
-
-	if (!contact_editor_categories_type)
-		{
-			static const GtkTypeInfo contact_editor_categories_info =
-			{
-				"ECategories",
-				sizeof (ECategories),
-				sizeof (ECategoriesClass),
-				(GtkClassInitFunc) e_categories_class_init,
-				(GtkObjectInitFunc) e_categories_init,
-				/* reserved_1 */ NULL,
-				/* reserved_2 */ NULL,
-				(GtkClassInitFunc) NULL,
-			};
-
-			contact_editor_categories_type = gtk_type_unique (gnome_dialog_get_type (), &contact_editor_categories_info);
-		}
-
-	return contact_editor_categories_type;
-}
+E_MAKE_TYPE (e_categories,
+	     "ECategories",
+	     ECategories,
+	     e_categories_class_init,
+	     e_categories_init,
+	     PARENT_TYPE)
 
 static void
 e_categories_class_init (ECategoriesClass *klass)
 {
-	GtkObjectClass *object_class;
-	GnomeDialogClass *dialog_class;
+	GObjectClass *object_class;
 
-	object_class = (GtkObjectClass*) klass;
-	dialog_class = (GnomeDialogClass *) klass;
+	object_class = (GObjectClass*) klass;
 
-	parent_class = gtk_type_class (gnome_dialog_get_type ());
-
-	gtk_object_add_arg_type ("ECategories::categories", GTK_TYPE_STRING,
-				 GTK_ARG_READWRITE, ARG_CATEGORIES);
-	gtk_object_add_arg_type ("ECategories::header", GTK_TYPE_STRING,
-				 GTK_ARG_READWRITE, ARG_HEADER);
-	gtk_object_add_arg_type ("ECategories::ecml", E_CATEGORIES_MASTER_LIST_TYPE,
-				 GTK_ARG_READWRITE, ARG_ECML);
+	parent_class = g_type_class_ref (PARENT_TYPE);
  
-	object_class->set_arg = e_categories_set_arg;
-	object_class->get_arg = e_categories_get_arg;
-	object_class->destroy = e_categories_destroy;
+	object_class->set_property = e_categories_set_property;
+	object_class->get_property = e_categories_get_property;
+	object_class->dispose = e_categories_dispose;
+
+	g_object_class_install_property (object_class, PROP_CATEGORIES,
+					 g_param_spec_string ("categories",
+							      _( "Categories" ),
+							      _( "Categories" ),
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_HEADER,
+					 g_param_spec_string ("header",
+							      _( "Header" ),
+							      _( "Header" ),
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_ECML,
+					 g_param_spec_object ("ecml",
+							      _( "ECML" ),
+							      _( "ECategoriesMasterListCombo" ),
+							      E_CATEGORIES_MASTER_LIST_TYPE,
+							      G_PARAM_READWRITE));
 }
 
 static void
@@ -242,23 +237,27 @@ e_categories_entry_change (GtkWidget *entry,
 	do_parse_categories(categories);
 }
 
+static void
+e_categories_entry_activate (GtkWidget *entry, 
+			     ECategories *categories)
+{
+	gtk_dialog_response (GTK_DIALOG (categories), GTK_RESPONSE_OK);
+}
+
 static void
 e_categories_release_ecmld (ECategories *categories)
 {
 	if (categories->priv->ecmld) {
-		if (categories->priv->ecmld_destroy_id)
-			gtk_signal_disconnect (GTK_OBJECT(categories->priv->ecmld),
-					       categories->priv->ecmld_destroy_id);
-		categories->priv->ecmld_destroy_id = 0;
-		gtk_object_unref (GTK_OBJECT (categories->priv->ecmld));
+		g_object_unref (categories->priv->ecmld);
 		categories->priv->ecmld = NULL;
 	}
 }
 
 static void
-ecmld_destroyed (GtkObject *object, ECategories *categories)
+ecmld_destroyed (gpointer data, GObject *where_object_was)
 {
-	e_categories_release_ecmld (categories);
+	ECategories *categories = data;
+	categories->priv->ecmld = NULL;
 }
 
 static void
@@ -269,9 +268,9 @@ e_categories_button_clicked (GtkWidget *button,
 		e_categories_master_list_dialog_raise (categories->priv->ecmld);
 	else {
 		categories->priv->ecmld = e_categories_master_list_dialog_new (categories->priv->ecml);
-		categories->priv->ecmld_destroy_id = gtk_signal_connect (GTK_OBJECT (categories->priv->ecmld), "destroy",
-									 GTK_SIGNAL_FUNC (ecmld_destroyed), categories);
-		gtk_object_ref (GTK_OBJECT (categories->priv->ecmld));
+		g_object_weak_ref (G_OBJECT (categories->priv->ecmld),
+				   ecmld_destroyed, categories);
+		g_object_ref (categories->priv->ecmld);
 	}
 }
 
@@ -292,23 +291,23 @@ ec_set_ecml (ECategories *categories, ECategoriesMasterList *ecml)
 {
 	if (categories->priv->ecml) {
 		if (categories->priv->ecml_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT(categories->priv->ecml),
-					       categories->priv->ecml_changed_id);
-		gtk_object_unref (GTK_OBJECT (categories->priv->ecml));
+			g_signal_handler_disconnect (categories->priv->ecml,
+						     categories->priv->ecml_changed_id);
+		g_object_unref (categories->priv->ecml);
 	}
 	categories->priv->ecml = ecml;
 	if (categories->priv->ecml) {
-		gtk_object_ref (GTK_OBJECT (categories->priv->ecml));
+		g_object_ref (categories->priv->ecml);
 		categories->priv->ecml_changed_id =
-			gtk_signal_connect (GTK_OBJECT (categories->priv->ecml),
-					    "changed",
-					    GTK_SIGNAL_FUNC (ecml_changed),
-					    categories);
+			g_signal_connect (categories->priv->ecml,
+					  "changed",
+					  G_CALLBACK (ecml_changed),
+					  categories);
 	}
 	if (categories->priv->ecmld) {
-		gtk_object_set (GTK_OBJECT (categories->priv->ecmld),
-				"ecml", ecml,
-				NULL);
+		g_object_set (categories->priv->ecmld,
+			      "ecml", ecml,
+			      NULL);
 	}
 	do_parse_categories(categories);
 }
@@ -323,7 +322,7 @@ ec_set_header (ECategories *categories, const char *header)
 }
 
 
-#define INITIAL_SPEC "<ETableSpecification no-headers=\"true\" draw-grid=\"true\" cursor-mode=\"line\">\
+#define INITIAL_SPEC "<ETableSpecification no-headers=\"true\" draw-grid=\"true\" cursor-mode=\"line\" gettext-domain=\"" E_I18N_DOMAIN "\">\
   <ETableColumn model_col=\"0\" _title=\"Active\" expansion=\"0.0\" minimum_width=\"20\" resizable=\"false\" cell=\"checkbox\"       compare=\"integer\"/> \
   <ETableColumn model_col=\"1\" _title=\"Category\" expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\" compare=\"string\"/> \
         <ETableState>                                                           \
@@ -392,14 +391,10 @@ remove_category (GtkWidget *widget, gpointer user_data)
 }
 
 static EPopupMenu menu[] = {
-	{ N_("Add to global category list"),                         NULL,
-	  GTK_SIGNAL_FUNC (add_category),         NULL,  4 + 1 },
-	{ N_("Add all to global category list"),                         NULL,
-	  GTK_SIGNAL_FUNC (add_category),         NULL,  8 + 1 },
-	{ N_("Remove from global category list"),                         NULL,
-	  GTK_SIGNAL_FUNC (remove_category),         NULL,  4 + 2 },
-	{ N_("Remove all from global category list"),                         NULL,
-	  GTK_SIGNAL_FUNC (remove_category),         NULL,  8 + 2 },
+	E_POPUP_ITEM (N_("Add to global category list"), GTK_SIGNAL_FUNC (add_category), 4 + 1),
+	E_POPUP_ITEM (N_("Add all to global category list"), GTK_SIGNAL_FUNC (add_category), 8 + 1),
+	E_POPUP_ITEM (N_("Remove from global category list"), GTK_SIGNAL_FUNC (remove_category), 4 + 2),
+	E_POPUP_ITEM (N_("Remove all from global category list"), GTK_SIGNAL_FUNC (remove_category), 8 + 2),
 
 	E_POPUP_TERMINATOR
 };
@@ -462,6 +457,16 @@ table_right_click (ETable *table, int row, int col, GdkEvent *event, ECategories
 	return TRUE;
 }
 
+static gint
+table_click (ETable *table, int row, int col, GdkEvent *event, ECategories *categories)
+{
+	if (col == 1) {
+		categories->priv->selected_list[row] = !categories->priv->selected_list[row];
+		e_categories_rebuild (categories);
+	}
+	return TRUE;
+}
+
 static void
 e_categories_init (ECategories *categories)
 {
@@ -475,38 +480,39 @@ e_categories_init (ECategories *categories)
 	categories->priv->selected_list = NULL;
 	categories->priv->categories = g_strdup ("");
 	categories->priv->ecmld = NULL;
-	categories->priv->ecmld_destroy_id = 0;
 }
 
 static void
-e_categories_destroy (GtkObject *object)
+e_categories_dispose (GObject *object)
 {
 	ECategories *categories = E_CATEGORIES(object);
 	int i;
 
-	if (categories->priv->gui)
-		gtk_object_unref(GTK_OBJECT(categories->priv->gui));
+	if (categories->priv) {
+		if (categories->priv->gui)
+			g_object_unref (categories->priv->gui);
 
-	g_free(categories->priv->categories);
-	for (i = 0; i < categories->priv->list_length; i++)
-		g_free(categories->priv->category_list[i]);
+		g_free(categories->priv->categories);
+		for (i = 0; i < categories->priv->list_length; i++)
+			g_free(categories->priv->category_list[i]);
 
-	if (categories->priv->ecml) {
-		if (categories->priv->ecml_changed_id)
-			gtk_signal_disconnect (GTK_OBJECT(categories->priv->ecml),
-					       categories->priv->ecml_changed_id);
-		gtk_object_unref (GTK_OBJECT (categories->priv->ecml));
+		if (categories->priv->ecml) {
+			if (categories->priv->ecml_changed_id)
+				g_signal_handler_disconnect (categories->priv->ecml,
+							     categories->priv->ecml_changed_id);
+			g_object_unref (categories->priv->ecml);
+		}
+
+		e_categories_release_ecmld (categories);
+
+		g_free(categories->priv->category_list);
+		g_free(categories->priv->selected_list);
+		g_free(categories->priv);
+		categories->priv = NULL;
 	}
 
-	e_categories_release_ecmld (categories);
-
-	g_free(categories->priv->category_list);
-	g_free(categories->priv->selected_list);
-	g_free(categories->priv);
-	categories->priv = NULL;
-
-	if (GTK_OBJECT_CLASS (parent_class)->destroy)
-		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	if (G_OBJECT_CLASS (parent_class)->dispose)
+		(* G_OBJECT_CLASS (parent_class)->dispose) (object);
 }
 
 /**
@@ -532,35 +538,41 @@ e_categories_construct (ECategories *categories,
 
 	gui = glade_xml_new_with_domain (GAL_GLADEDIR "/gal-categories.glade", NULL, PACKAGE);
 
+	gtk_window_set_title (GTK_WINDOW (categories), _("Edit Categories"));
+
 	if (gui) {
 		categories->priv->gui = gui;
 
-		gnome_dialog_append_button ( GNOME_DIALOG(categories),
-					     GNOME_STOCK_BUTTON_OK);
-	
-		gnome_dialog_append_button ( GNOME_DIALOG(categories),
-					     GNOME_STOCK_BUTTON_CANCEL);
+		gtk_dialog_add_buttons (GTK_DIALOG (categories),
+					GTK_STOCK_OK, GTK_RESPONSE_OK,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					NULL);
 
-		gnome_dialog_set_default (GNOME_DIALOG (categories), 0);
+		gtk_dialog_set_default_response (GTK_DIALOG (categories),
+						 GTK_RESPONSE_OK);
 
 		gtk_window_set_policy(GTK_WINDOW(categories), FALSE, TRUE, FALSE);
 
 		table = glade_xml_get_widget(gui, "table-categories");
 		gtk_widget_ref(table);
 		gtk_container_remove(GTK_CONTAINER(table->parent), table);
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (categories)->vbox), table, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (categories)->vbox), table, TRUE, TRUE, 0);
 		gtk_widget_unref(table);
-		
+
 		categories->priv->entry = glade_xml_get_widget(gui, "entry-categories");
-	
-		gtk_signal_connect(GTK_OBJECT(categories->priv->entry), "changed",
-				   GTK_SIGNAL_FUNC(e_categories_entry_change), categories);
+		
+		g_signal_connect(categories->priv->entry, "changed",
+				 G_CALLBACK(e_categories_entry_change), categories);
+
+		if (GTK_IS_ENTRY (categories->priv->entry))
+			g_signal_connect(categories->priv->entry, "activate",
+					 G_CALLBACK(e_categories_entry_activate), categories);
 
 		button = glade_xml_get_widget(gui, "button-ecmld");
 		
-		gtk_signal_connect(GTK_OBJECT(button), "clicked",
-				   GTK_SIGNAL_FUNC(e_categories_button_clicked), categories);
-		
+		g_signal_connect(button, "clicked",
+				 G_CALLBACK(e_categories_button_clicked), categories);
+
 		categories->priv->model = e_table_simple_new(e_categories_col_count, 
 							     e_categories_row_count,
 							     NULL,
@@ -568,10 +580,10 @@ e_categories_construct (ECategories *categories,
 							     e_categories_value_at,
 							     e_categories_set_value_at,
 							     e_categories_is_cell_editable,
-							     
+
 							     e_categories_has_save_id,
 							     e_categories_get_save_id,
-							     
+
 							     e_categories_duplicate_value,
 							     e_categories_free_value,
 							     e_categories_initialize_value,
@@ -580,14 +592,14 @@ e_categories_construct (ECategories *categories,
 							     categories);
 
 		e_table = e_table_scrolled_new (categories->priv->model, NULL, INITIAL_SPEC, NULL);
-		
+
 		categories->priv->table = e_table_scrolled_get_table(E_TABLE_SCROLLED(e_table));
-		gtk_signal_connect(GTK_OBJECT(categories->priv->table), "key_press",
-				   GTK_SIGNAL_FUNC(table_key_press), categories);
-		gtk_signal_connect(GTK_OBJECT(categories->priv->table), "right_click",
-				   GTK_SIGNAL_FUNC(table_right_click), categories);
-		
-		gtk_object_sink(GTK_OBJECT(categories->priv->model));
+		g_signal_connect (categories->priv->table, "key_press",
+				  G_CALLBACK(table_key_press), categories);
+		g_signal_connect (categories->priv->table, "right_click",
+				  G_CALLBACK(table_right_click), categories);
+		g_signal_connect (categories->priv->table, "click",
+				  G_CALLBACK(table_click), categories);
 		
 		gtk_widget_show(e_table);
 
@@ -595,18 +607,16 @@ e_categories_construct (ECategories *categories,
 					  e_table, 
 					  0, 1,
 					  3, 4);
-		
 		gtk_widget_grab_focus (categories->priv->entry);
-		gnome_dialog_editable_enters (GNOME_DIALOG (categories), GTK_EDITABLE (categories->priv->entry));
-		
+
 		ecml = e_categories_master_list_array_new();
-		gtk_object_set (GTK_OBJECT (categories),
-				"ecml", ecml,
-				NULL);
-		gtk_object_unref(GTK_OBJECT (ecml));
-		
+		g_object_set (categories,
+			      "ecml", ecml,
+			      NULL);
+		g_object_unref (ecml);
+
 		ec_set_categories (categories->priv->entry, initial_category_list);
-		
+
 		gtk_window_set_default_size (GTK_WINDOW (categories), 200, 400);
 	}
 }
@@ -624,12 +634,12 @@ e_categories_new (const char *initial_category_list)
 {
 	ECategories *ecat;
 
-	ecat = E_CATEGORIES (gtk_type_new (e_categories_get_type ()));
+	ecat = E_CATEGORIES (g_object_new (E_CATEGORIES_TYPE, NULL));
 
 	e_categories_construct (ecat, initial_category_list);
 
 	if (ecat->priv->gui == NULL) {
-		gtk_object_unref (GTK_OBJECT (ecat));
+		g_object_unref (ecat);
 		return NULL;
 	}
 
@@ -637,29 +647,29 @@ e_categories_new (const char *initial_category_list)
 }
 
 static void
-e_categories_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+e_categories_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	ECategories *e_categories;
 
-	e_categories = E_CATEGORIES (o);
+	e_categories = E_CATEGORIES (object);
 	
-	switch (arg_id){
-	case ARG_CATEGORIES:
-		ec_set_categories (e_categories->priv->entry, GTK_VALUE_STRING (*arg));
+	switch (prop_id){
+	case PROP_CATEGORIES:
+		ec_set_categories (e_categories->priv->entry, g_value_get_string (value));
 		break;
 
-	case ARG_HEADER:
-		ec_set_header (e_categories, GTK_VALUE_STRING (*arg));
+	case PROP_HEADER:
+		ec_set_header (e_categories, g_value_get_string (value));
 		break;
 
-	case ARG_ECML:
-		ec_set_ecml (e_categories, (ECategoriesMasterList *) GTK_VALUE_OBJECT (*arg));
+	case PROP_ECML:
+		ec_set_ecml (e_categories, (ECategoriesMasterList *) g_value_get_object (value));
 		break;
 	}
 }
 
 static void
-e_categories_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+e_categories_get_property    (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	ECategories *e_categories;
 	GtkWidget *widget;
@@ -667,28 +677,25 @@ e_categories_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 
 	e_categories = E_CATEGORIES (object);
 
-	switch (arg_id) {
-	case ARG_CATEGORIES:
-		GTK_VALUE_STRING (*arg) = g_strdup(e_categories->priv->categories);
+	switch (prop_id) {
+	case PROP_CATEGORIES:
+		g_value_set_string (value, g_strdup(e_categories->priv->categories));
 		break;
 
-	case ARG_HEADER:
+	case PROP_HEADER:
 		widget = glade_xml_get_widget(e_categories->priv->gui, "label-header");
 		if (widget && GTK_IS_LABEL(widget)) {
-			gtk_object_get(GTK_OBJECT(widget),
-				       "label", &header,
-				       NULL);
-			GTK_VALUE_STRING (*arg) = header;
+			g_object_get_property (G_OBJECT (widget), "label", value);
 		} else
-			GTK_VALUE_STRING (*arg) = NULL;
+			g_value_set_string (value, NULL);
 		break;
 
-	case ARG_ECML:
-		GTK_VALUE_OBJECT (*arg) = (GtkObject *) e_categories->priv->ecml;
+	case PROP_ECML:
+		g_value_set_object (value, e_categories->priv->ecml);
 		break;
 
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
