@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-font.c - Temporary wrappers around GdkFonts to get unicode displaying
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -28,7 +28,6 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <errno.h>
 #include <iconv.h>
 #include "gal/util/e-cache.h"
 #include "e-font.h"
@@ -47,7 +46,7 @@ static int e_font_verbose = 0;
 
 #define E_FONT_VERBOSE
 #define E_FONT_CACHE_SIZE 32
-#define E_ALLOCA_MAX 2048
+
 enum {
 	E_XF_FOUNDRY,
 	E_XF_FAMILY,
@@ -80,7 +79,7 @@ static void split_name (gchar * c[], gchar * name);
 static gboolean find_variants (gchar **namelist, gint length,
 			       gchar *base_weight, gchar **light,
 			       gchar **bold);
-static int e_font_to_native (EFont *font, char **native, const char *in, int inlen);
+static gint         e_font_to_native   (EFont *font, gchar *native, const gchar *utf, gint bytes);
 #ifdef E_FONT_VERBOSE
 static void e_font_print_gdk_font_name (const GdkFont * font);
 #endif
@@ -177,61 +176,21 @@ static void
 set_nbsp_zero_width_flag (EFont *efont)
 {
 	guchar *nbsp = "\xc2\xa0";
-	guchar *native;
-	int nlen;
-	
+	guchar native_nbsp [8];
+
 	efont->nbsp_zero_width = FALSE;
-	
-	nlen = e_font_to_native (efont, (char **) &native, nbsp, 2);
-	efont->nbsp_zero_width = gdk_text_width (efont->font, native, nlen) ? FALSE : TRUE;
-	g_free (native);
+	efont->nbsp_zero_width = gdk_text_width (efont->font, native_nbsp, e_font_to_native (efont, native_nbsp, nbsp, 2))
+		? FALSE : TRUE;
 }
 
 /*
- * Simplified version used for gtkhtml library to reduce resources usage
+ * Creates EFont from existing GdkFont.
+ * 1. Use cache
+ * 2. Analyze, whether we need font or fontset. For fontset use e_font_from_gdk_fontset
+ * 3. Try, if we can simply use iso-10646 variant of font
+ * 4. Determine bold or light variant
  */
 
-EFont *
-e_font_from_gdk_font_gtkhtml (GdkFont *gdk_font)
-{
-	EFont *font;
-	XFontStruct *xfs;
-
-#ifdef E_FONT_VERBOSE
-	if (e_font_verbose) {
-		g_print ("(e_font_from_gdk_font_gtkhtml): ");
-		e_font_print_gdk_font_name (gdk_font);
-	}
-#endif
-
-	g_return_val_if_fail (gdk_font != NULL, NULL);
-
-	font = g_new (EFont, 1);
-
-	xfs = GDK_FONT_XFONT (gdk_font);
-
-	font->refcount = 1;
-	gdk_font_ref (gdk_font);
-	gdk_font_ref (gdk_font);
-	font->font = gdk_font;
-	font->bold = gdk_font;
-	font->twobyte = (gdk_font->type == GDK_FONT_FONTSET || ((xfs->min_byte1 != 0) || (xfs->max_byte1 != 0)));
-	font->to = e_iconv_to_gdk_font (font->font);
-	font->from = e_iconv_from_gdk_font (font->font);
-	set_nbsp_zero_width_flag (font);
-
-	return font;
-
-}
-
- /*
-  * Creates EFont from existing GdkFont.
-  * 1. Use cache
-  * 2. Analyze, whether we need font or fontset. For fontset use e_font_from_gdk_fontset
-  * 3. Try, if we can simply use iso-10646 variant of font
-  * 4. Determine bold or light variant
-  */
-
 EFont *
 e_font_from_gdk_font (GdkFont *gdkfont)
 {
@@ -720,166 +679,144 @@ no_conv_wrapper (EFont *font, gchar *native, const gchar *utf, gint bytes)
 	return len;
 }
 
-static int
-e_font_to_native (EFont *font, char **native, const char *in, int inlen)
+static guchar *
+replace_nbsp_with_spaces (const guchar *orig, gint *bytes)
 {
-	size_t inleft, outleft, converted = 0;
-	char *out, *outbuf, *buf = NULL;
-	const char *inbuf, *tmp;
-	size_t outlen;
-	int rv;
-	
-	outlen = inlen * 2 + 16;
-	out = g_malloc (outlen + 2);
+	guchar *rv, *p;
+	gboolean nbsp1;
+	gint remains = *bytes;
+
+	if (!orig)
+		return NULL;
+
+	/* printf ("bytes: %d orig\n%s\n", *bytes, orig); */
+
+	p = rv = (guchar *) g_malloc (strlen (orig) + 1);
+	nbsp1  = FALSE;
+
+	while (remains) {
+		if (*orig == 0xc2)
+			nbsp1 = TRUE;
+		else if (nbsp1) {
+			if (*orig == 0xa0) {
+				*p = ' ';   p ++;
+				(*bytes) --;
+			} else {
+				*p = 0xc2;  p ++;
+				*p = *orig; p ++;
+			}
+			nbsp1 = FALSE;
+		} else {
+			*p = *orig;
+			p ++;
+		}
+		orig ++;
+		remains --;
+	}
+
+	if (nbsp1) {
+		*p = 0xc2; p ++;
+	}
+	*p = 0;
+
+	/* printf ("rv: %s\n", rv); */
+
+	return rv;
+}
+
+static gint
+e_font_to_native (EFont *font, gchar *native, const gchar *utf, gint bytes)
+{
+	const char *ib;
+	char *ob;
+	size_t ibl, obl;
+	gint rv;
+
+	if (font->nbsp_zero_width)
+		utf = replace_nbsp_with_spaces (utf, &bytes);
 
 	if (font->to == (iconv_t) -1)
-		goto noop;
-	
-	inbuf = in;
-	inleft = inlen;
+		rv = no_conv_wrapper (font, native, utf, bytes);
+	else {
+		ib = utf;
+		ibl = bytes;
+		ob = native;
+		obl = bytes * 4;
 
-	do {
-		outbuf = out + converted;
-		outleft = outlen - converted;
-		
-		converted = e_iconv (font->to, (const char **) &inbuf, &inleft, &outbuf, &outleft);
-		if (converted == (size_t) -1) {
-			if (errno == E2BIG) {
-				/*
-				 * E2BIG   There is not sufficient room at *outbuf.
-				 *
-				 * We just need to grow our outbuffer and try again.
-				 */
-				converted = outbuf - (char *) out;
-				outlen += inleft * 2 + 16;
-				out = g_realloc (out, outlen + 2);
-				outbuf = out + converted;
-			} else if (errno == EILSEQ) {
-				/*
-				 * EILSEQ An invalid multibyte sequence has been  encountered
-				 *        in the input.
-				 *
-				 * An illegal multibyte sequence has been encountered.
-				 */
-				gunichar uc;
-				size_t rlen = 1;
-				char *rbuf;
-				
-				uc = g_utf8_get_char (inbuf);
-				
-#ifdef E_FONT_VERBOSE
-				g_warning ("failed trying to convert unichar=%xd", uc);
-#endif
-				/*
-				 * We should really try to do a compatabilty normalization
-				 * here and try to convert again, but gal's g_utf8 stuff
-				 * doesn't have include the glib normalization function
-				 * so we'll just try for some basic ascii so that we see
-				 * something.
-				 */
-				switch (uc) {
-				case 0x0000: 
-					break;
-				case 0x00a0: /* NBSP */
-					rbuf = " ";
-					break;
-				default:
-					rbuf= "_";
-					break;
+		while (ibl > 0) {
+			e_iconv (font->to, &ib, &ibl, &ob, &obl);
+			if (ibl > 0) {
+				ib = g_utf8_next_char (ib);
+
+				ibl = bytes - (ib - utf);
+				if (ibl > bytes) ibl = 0;
+				if (!font->twobyte) {
+					*ob++ = '_';
+					obl--;
+				} else {
+					*((guint16 *) ob) = '_';
+					ob += 2;
+					obl -= 2;
 				}
-				
-				/* we don't really care about errors here we've done our best */
-				e_iconv (font->to, (const char **) &rbuf, &rlen, &outbuf, &outleft);
-
-				tmp = g_utf8_next_char (inbuf);
-				inleft -= (tmp - inbuf);
-				inbuf = tmp;
-			} else if (errno == EINVAL) {
-				/*
-				 * EINVAL  An  incomplete  multibyte sequence has been encoun
-				 *         tered in the input.
-				 *
-				 * We'll just have to ignore it...
-				 */
-				break;
-			} else {
-				/* Invalid error code */
-				goto noop;
 			}
 		}
-		converted = outbuf - (char *) out;
-		
-	} while (((int) inleft) > 0);
-	
-	/* flush and reset the iconv conversion */
-	e_iconv (font->to, NULL, NULL, &outbuf, &outleft);
-	
-	*native = out;
-	
-	return outbuf - out;
-	
- noop:
-	
-	/* reset the cd */
-	if (font->to != (iconv_t) -1)
-		e_iconv (font->to, NULL, NULL, NULL, NULL);
-	
-	out = g_realloc (out, inlen * 2);
-	
-	rv = no_conv_wrapper (font, out, in, inlen);
-	
-	*native = out;
-	g_free (buf);
-	
+		rv = ob - native;
+	}
+
+	if (font->nbsp_zero_width)
+		g_free ((void *) utf);
+
 	return rv;
 }
 
 void
-e_font_draw_utf8_text (GdkDrawable *drawable, EFont *font, EFontStyle style, GdkGC *gc,
-		       gint x, gint y, const gchar *text, gint numbytes)
+e_font_draw_utf8_text (GdkDrawable *drawable, EFont *font, EFontStyle style, GdkGC *gc, gint x, gint y, const gchar *text, gint numbytes)
 {
-	char *native;
-	int nlen;
-	
-	g_return_if_fail (font != NULL);
-	g_return_if_fail (text != NULL);
-	
-	if (numbytes < 1)
-		return;
+	gchar *native;
+	gint native_bytes;
 
-	nlen = e_font_to_native (font, &native, text, numbytes);
+	g_return_if_fail (drawable != NULL);
+	g_return_if_fail (font != NULL);
+	g_return_if_fail (gc != NULL);
+	g_return_if_fail (text != NULL);
+
+	if (numbytes < 1) return;
+
+	native = alloca (numbytes * 4);
+
+	native_bytes = e_font_to_native (font, native, text, numbytes);
 	
 	if ((style & E_FONT_BOLD) && (font->bold)) {
-		gdk_draw_text (drawable, font->bold, gc, x, y, native, nlen);
+		gdk_draw_text (drawable, font->bold, gc, x, y, native, native_bytes);
 	} else {
-		gdk_draw_text (drawable, font->font, gc, x, y, native, nlen);
+		gdk_draw_text (drawable, font->font, gc, x, y, native, native_bytes);
 		if (style & E_FONT_BOLD)
-			gdk_draw_text (drawable, font->font, gc, x + 1, y, native, nlen);
+			gdk_draw_text (drawable, font->font, gc, x + 1, y, native, native_bytes);
 	}
-	g_free (native);
 }
 
 gint
 e_font_utf8_text_width (EFont *font, EFontStyle style, const char *text, int numbytes)
 {
-	char *native;
-	int width, nlen;
-	
+	gchar *native;
+	gint native_bytes;
+	gint width;
+
 	g_return_val_if_fail (font != NULL, 0);
 	g_return_val_if_fail (text != NULL, 0);
-	
-	if (numbytes < 1)
-		return 0;
 
-	nlen = e_font_to_native (font, &native, text, numbytes);
-	
+	if (numbytes < 1) return 0;
+
+	native = alloca (numbytes * 4);
+
+	native_bytes = e_font_to_native (font, native, text, numbytes);
+
 	if ((style & E_FONT_BOLD) && (font->bold)) {
-		width = gdk_text_width (font->bold, native, nlen);
+		width = gdk_text_width (font->bold, native, native_bytes);
 	} else {
-		width = gdk_text_width (font->font, native, nlen);
+		width = gdk_text_width (font->font, native, native_bytes);
 	}
-	
-	g_free (native);
+
 	return width;
 }
 
@@ -896,7 +833,8 @@ const gchar *
 e_gdk_font_encoding (GdkFont *font)
 {
 	static ECache * cache = NULL;
-	Atom font_atom, atom;
+	GdkAtom font_atom;
+	Atom x_font_atom, x_atom;
 	Bool status;
 	char *name, *p;
 	const gchar *encoding;
@@ -925,10 +863,11 @@ e_gdk_font_encoding (GdkFont *font)
 		/* We are fontset, so try to find right encoding from locale */
 		encoding = e_iconv_charset_name(e_iconv_locale_charset());
 		if (encoding)
-			return encoding;
-	}
+			    return encoding;
+		}
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
+	x_font_atom = gdk_x11_atom_to_xatom (font_atom);
 
 	if (font->type == GDK_FONT_FONTSET) {
 		XFontStruct **font_structs;
@@ -939,12 +878,12 @@ e_gdk_font_encoding (GdkFont *font)
 					     &font_structs,
 					     &font_names);
 		status = XGetFontProperty (font_structs[0],
-					   font_atom,
-					   &atom);
+					   x_font_atom,
+					   &x_atom);
 	} else {
 		status = XGetFontProperty (GDK_FONT_XFONT (font),
-					   font_atom,
-					   &atom);
+					   x_font_atom,
+					   &x_atom);
 	}
 
 	if (!status) {
@@ -953,7 +892,7 @@ e_gdk_font_encoding (GdkFont *font)
 		return NULL;
 	}
 
-	name = p = gdk_atom_name (atom);
+	name = p = gdk_atom_name (gdk_x11_xatom_to_atom (x_atom));
 
 	for (i = 0; i < 13; i++) {
 		/* Skip hyphen */
@@ -1006,7 +945,8 @@ e_iconv_to_gdk_font (GdkFont *font)
 static gchar *
 get_font_name (const GdkFont * font)
 {
-	Atom font_atom, atom;
+	GdkAtom font_atom;
+	Atom x_font_atom, x_atom;
 	Bool status;
 
 #ifdef E_FONT_VERBOSE
@@ -1016,13 +956,14 @@ get_font_name (const GdkFont * font)
 #endif
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
+	x_font_atom = gdk_x11_atom_to_xatom (font_atom);
 
 	if (font->type == GDK_FONT_FONTSET) {
 		XFontStruct **font_structs;
 		gint num_fonts;
 		gchar **font_names;
 
-		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font), &font_structs, &font_names);
+		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT ((GdkFont *)font), &font_structs, &font_names);
 #ifdef E_FONT_VERBOSE
 		if (e_font_verbose) {
 			gint i;
@@ -1032,13 +973,14 @@ get_font_name (const GdkFont * font)
 				g_print ("  %s\n", font_names[i]);
 		}
 #endif
-		status = XGetFontProperty (font_structs[0], font_atom, &atom);
+		status = XGetFontProperty (font_structs[0], x_font_atom, &x_atom);
 	} else {
-		status = XGetFontProperty (GDK_FONT_XFONT (font), font_atom, &atom);
+		status = XGetFontProperty (GDK_FONT_XFONT ((GdkFont *)font),
+					   x_font_atom, &x_atom);
 	}
 
 	if (status) {
-		return gdk_atom_name (atom);
+		return gdk_atom_name (gdk_x11_xatom_to_atom (x_atom));
 	}
 
 	return NULL;
@@ -1161,10 +1103,12 @@ find_variants (gchar **namelist, gint length, gchar *weight,
 static void
 e_font_print_gdk_font_name (const GdkFont * font)
 {
-	Atom font_atom, atom;
+	GdkAtom font_atom;
+	Atom x_font_atom, x_atom;
 	Bool status;
 
 	font_atom = gdk_atom_intern ("FONT", FALSE);
+	x_font_atom = gdk_x11_atom_to_xatom (font_atom);
 
 	if (font == NULL) {
 		g_print ("GdkFont is NULL\n");
@@ -1174,16 +1118,19 @@ e_font_print_gdk_font_name (const GdkFont * font)
 		gchar **font_names;
 		gint i;
 
-		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font), &font_structs, &font_names);
+		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT ((GdkFont *)font),
+					     &font_structs, &font_names);
 
-		g_print ("Gdk Fontset, locale: %s\n", XLocaleOfFontSet (GDK_FONT_XFONT (font)));
+		g_print ("Gdk Fontset, locale: %s\n",
+			 XLocaleOfFontSet (GDK_FONT_XFONT ((GdkFont *)font)));
 		for (i = 0; i < num_fonts; i++) {
 			g_print ("    %s\n", font_names[i]);
 		}
 	} else {
 		gchar * name;
-		status = XGetFontProperty (GDK_FONT_XFONT (font), font_atom, &atom);
-		name = gdk_atom_name (atom);
+		status = XGetFontProperty (
+			GDK_FONT_XFONT ((GdkFont *)font), x_font_atom, &x_atom);
+		name = gdk_atom_name (gdk_x11_xatom_to_atom (x_atom));
 		g_print ("GdkFont: %s\n", name);
 		if (name) g_free (name);
 	}
