@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * e-gui-utils.c
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -27,13 +27,15 @@
 
 #include <gtk/gtkentry.h>
 #include <gtk/gtksignal.h>
-#include <libgnomeui/gnome-messagebox.h>
-#include <libgnomeui/gnome-stock.h>
+#include <gtk/gtkmessagedialog.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gdk-pixbuf/gnome-canvas-pixbuf.h>
+#include <libgnomecanvas/gnome-canvas-pixbuf.h>
 
+/* should probably just deprecate/remove this and have callers change
+   to using gtk_message_dialog_new */
+/* This should probably be moved into evolution/e-util */
 void
-e_notice (GtkWindow *window, const char *type, const char *format, ...)
+e_notice (GtkWindow *parent, GtkMessageType type, const char *format, ...)
 {
 	GtkWidget *dialog;
 	va_list args;
@@ -41,14 +43,15 @@ e_notice (GtkWindow *window, const char *type, const char *format, ...)
 
 	va_start (args, format);
 	str = g_strdup_vprintf (format, args);
-	dialog = gnome_message_box_new (str, type, GNOME_STOCK_BUTTON_OK, NULL);
+	dialog = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT, type,
+					 GTK_BUTTONS_CLOSE,
+					 "%s",
+					 str);
 	va_end (args);
 	g_free (str);
 	
-	if (window)
-		gnome_dialog_set_parent (GNOME_DIALOG (dialog), window);
-
-	gnome_dialog_run (GNOME_DIALOG (dialog));
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 }
 
 static void
@@ -57,7 +60,7 @@ kill_popup_menu (GtkWidget *widget, GtkMenu *menu)
 	g_return_if_fail (menu != NULL);
 	g_return_if_fail (GTK_IS_MENU (menu));
 
-	gtk_object_unref (GTK_OBJECT (menu));
+	g_object_unref (menu);
 }
 
 void
@@ -66,8 +69,8 @@ e_auto_kill_popup_menu_on_hide (GtkMenu *menu)
 	g_return_if_fail (menu != NULL);
 	g_return_if_fail (GTK_IS_MENU (menu));
 
-	gtk_signal_connect (GTK_OBJECT (menu), "hide",
-			    GTK_SIGNAL_FUNC (kill_popup_menu), menu);
+	g_signal_connect (menu, "hide",
+			  G_CALLBACK (kill_popup_menu), menu);
 }
 
 void
@@ -119,7 +122,7 @@ static void
 e_container_change_tab_order_destroy_notify(gpointer data)
 {
 	GList *list = data;
-	g_list_foreach(list, (GFunc) gtk_object_unref, NULL);
+	g_list_foreach(list, (GFunc) g_object_unref, NULL);
 	g_list_free(list);
 }
 
@@ -162,22 +165,22 @@ e_container_change_tab_order_callback(GtkContainer *container,
 				if (GTK_WIDGET_DRAWABLE (child) &&
 				    GTK_IS_CONTAINER (child) &&
 				    !GTK_WIDGET_HAS_FOCUS (child))
-					if (gtk_container_focus (GTK_CONTAINER (child), direction)) {
-						gtk_signal_emit_stop_by_name(GTK_OBJECT(container), "focus");
+					if (gtk_widget_child_focus (GTK_WIDGET (child), direction)) {
+						g_signal_stop_emission_by_name (container, "focus");
 						return TRUE;
 					}
 			}
 		}
 		else if (GTK_WIDGET_DRAWABLE (child)) {
 			if (GTK_IS_CONTAINER (child)) {
-				if (gtk_container_focus (GTK_CONTAINER (child), direction)) {
-					gtk_signal_emit_stop_by_name(GTK_OBJECT(container), "focus");
+				if (gtk_widget_child_focus (GTK_WIDGET (child), direction)) {
+					g_signal_stop_emission_by_name (container, "focus");
 					return TRUE;
 				}
 			}
 			else if (GTK_WIDGET_CAN_FOCUS (child)) {
 				gtk_widget_grab_focus (child);
-				gtk_signal_emit_stop_by_name(GTK_OBJECT(container), "focus");
+				g_signal_stop_emission_by_name (container, "focus");
 				return TRUE;
 			}
 		}
@@ -191,7 +194,7 @@ e_container_change_tab_order(GtkContainer *container, GList *widgets)
 {
 	GList *list;
 	list = g_list_copy(widgets);
-	g_list_foreach(list, (GFunc) gtk_object_ref, NULL);
+	g_list_foreach(list, (GFunc) g_object_ref, NULL);
 	return gtk_signal_connect_full(GTK_OBJECT(container), "focus",
 				       GTK_SIGNAL_FUNC(e_container_change_tab_order_callback),
 				       NULL, list,
@@ -229,3 +232,34 @@ e_container_focus_nth_entry(GtkContainer *container, int n)
 	if (data.widget)
 		gtk_widget_grab_focus(data.widget);
 }
+
+gboolean
+e_glade_xml_connect_widget (GladeXML *gui, char *name, char *signal, GCallback cb, gpointer closure)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (gui, name);
+
+	if (widget) {
+		g_signal_connect (widget, signal,
+				  cb, closure);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+e_glade_xml_set_sensitive (GladeXML *gui, char *name, gboolean sensitive)
+{
+	GtkWidget *widget;
+
+	widget = glade_xml_get_widget (gui, name);
+
+	if (widget) {
+		gtk_widget_set_sensitive (widget, sensitive);
+		return TRUE;
+	}
+
+	return FALSE;
+}
