@@ -31,6 +31,7 @@
 #include "e-canvas.h"
 #include "gal/e-text/e-text.h"
 #include "gal/util/e-util.h"
+#include "gal/util/e-i18n.h"
 #include "gal/widgets/e-unicode.h"
 #include <gtk/gtksignal.h>
 #include "e-selection-model-simple.h"
@@ -53,21 +54,23 @@ static void e_reflow_resize_children (GnomeCanvasItem *item);
 #define E_REFLOW_BORDER_WIDTH 7
 #define E_REFLOW_FULL_GUTTER (E_REFLOW_DIVIDER_WIDTH + E_REFLOW_BORDER_WIDTH * 2)
 
+#define PARENT_TYPE GNOME_TYPE_CANVAS_GROUP
 static GnomeCanvasGroupClass *parent_class = NULL;
 
 /* The arguments we take */
 enum {
-	ARG_0,
-	ARG_MINIMUM_WIDTH,
-	ARG_WIDTH,
-	ARG_HEIGHT,
-	ARG_EMPTY_MESSAGE,
-	ARG_MODEL,
-	ARG_COLUMN_WIDTH,
+	PROP_0,
+	PROP_MINIMUM_WIDTH,
+	PROP_WIDTH,
+	PROP_HEIGHT,
+	PROP_EMPTY_MESSAGE,
+	PROP_MODEL,
+	PROP_COLUMN_WIDTH
 };
 
 enum {
 	SELECTION_EVENT,
+	COLUMN_WIDTH_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -121,15 +124,15 @@ static inline void
 e_reflow_update_selection_row (EReflow *reflow, int row)
 {
 	if (reflow->items[row]) {
-		gtk_object_set(GTK_OBJECT(reflow->items[row]),
-			       "selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(reflow->selection), row),
-			       NULL);
+		g_object_set(reflow->items[row],
+			     "selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(reflow->selection), row),
+			     NULL);
 	} else if (e_selection_model_is_row_selected (E_SELECTION_MODEL (reflow->selection), row)) {
 		reflow->items[row] = e_reflow_model_incarnate (reflow->model, row, GNOME_CANVAS_GROUP (reflow));
-		gtk_object_set (GTK_OBJECT (reflow->items[row]),
-				"selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(reflow->selection), row),
-				"width", (double) reflow->column_width,
-				NULL);
+		g_object_set (reflow->items[row],
+			      "selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(reflow->selection), row),
+			      "width", (double) reflow->column_width,
+			      NULL);
 	}
 }
 
@@ -165,9 +168,9 @@ cursor_changed (ESelectionModel *selection, int row, int col, EReflow *reflow)
 
 	if (old_cursor < count && old_cursor >= 0) {
 		if (reflow->items[old_cursor]) {
-			gtk_object_set (GTK_OBJECT (reflow->items[old_cursor]),
-					"has_cursor", FALSE,
-					NULL);
+			g_object_set (reflow->items[old_cursor],
+				      "has_cursor", FALSE,
+				      NULL);
 		}
 	}
 
@@ -175,15 +178,15 @@ cursor_changed (ESelectionModel *selection, int row, int col, EReflow *reflow)
 
 	if (row < count && row >= 0) {
 		if (reflow->items[row]) {
-			gtk_object_set (GTK_OBJECT (reflow->items[row]),
-					"has_cursor", TRUE,
-					NULL);
+			g_object_set (reflow->items[row],
+				      "has_cursor", TRUE,
+				      NULL);
 		} else {
 			reflow->items[row] = e_reflow_model_incarnate (reflow->model, row, GNOME_CANVAS_GROUP (reflow));
-			gtk_object_set (GTK_OBJECT (reflow->items[row]),
-					"has_cursor", TRUE,
-					"width", (double) reflow->column_width,
-					NULL);
+			g_object_set (reflow->items[row],
+				      "has_cursor", TRUE,
+				      "width", (double) reflow->column_width,
+				      NULL);
 		}
 	}
 }
@@ -223,10 +226,10 @@ incarnate (EReflow *reflow)
 		if (reflow->items[unsorted] == NULL) {
 			if (reflow->model) {
 				reflow->items[unsorted] = e_reflow_model_incarnate (reflow->model, unsorted, GNOME_CANVAS_GROUP (reflow));
-				gtk_object_set (GTK_OBJECT (reflow->items[unsorted]),
-						"selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(reflow->selection), unsorted),
-						"width", (double) reflow->column_width,
-						NULL);
+				g_object_set (reflow->items[unsorted],
+					      "selected", e_selection_model_is_row_selected(E_SELECTION_MODEL(reflow->selection), unsorted),
+					      "width", (double) reflow->column_width,
+					      NULL);
 			}
 		}
 	}
@@ -393,22 +396,19 @@ set_empty(EReflow *reflow)
 	if (reflow->count == 0) {
 		if (reflow->empty_text) {
 			if (reflow->empty_message) {
-				char *empty_message = e_utf8_to_gtk_string (GTK_WIDGET (GNOME_CANVAS_ITEM (reflow)->canvas), reflow->empty_message);
 				gnome_canvas_item_set(reflow->empty_text,
 						      "width", reflow->minimum_width,
-						      "text", empty_message,
+						      "text", reflow->empty_message,
 						      NULL);
 				e_canvas_item_move_absolute(reflow->empty_text,
 							    reflow->minimum_width / 2,
 							    0);
-				g_free (empty_message);
 			} else {
 				gtk_object_destroy(GTK_OBJECT(reflow->empty_text));
 				reflow->empty_text = NULL;
 			}
 		} else {
 			if (reflow->empty_message) {
-				char *empty_message = e_utf8_to_gtk_string (GTK_WIDGET (GNOME_CANVAS_ITEM (reflow)->canvas), reflow->empty_message);
 				reflow->empty_text =
 					gnome_canvas_item_new(GNOME_CANVAS_GROUP(reflow),
 							      e_text_get_type(),
@@ -416,13 +416,12 @@ set_empty(EReflow *reflow)
 							      "width", reflow->minimum_width,
 							      "clip", TRUE,
 							      "use_ellipsis", TRUE,
-							      "font_gdk", GTK_WIDGET(GNOME_CANVAS_ITEM(reflow)->canvas)->style->font,
+							      "font_gdk", gtk_style_get_font (GTK_WIDGET(GNOME_CANVAS_ITEM(reflow)->canvas)->style),
 							      "fill_color", "black",
 							      "justification", GTK_JUSTIFY_CENTER,
-							      "text", empty_message,
+							      "text", reflow->empty_message,
 							      "draw_background", FALSE,
 							      NULL);
-				g_free (empty_message);
 				e_canvas_item_move_absolute(reflow->empty_text,
 							    reflow->minimum_width / 2,
 							    0);
@@ -442,13 +441,13 @@ disconnect_model (EReflow *reflow)
 	if (reflow->model == NULL)
 		return;
 
-	gtk_signal_disconnect (GTK_OBJECT (reflow->model),
-			       reflow->model_changed_id);
-	gtk_signal_disconnect (GTK_OBJECT (reflow->model),
-			       reflow->model_items_inserted_id);
-	gtk_signal_disconnect (GTK_OBJECT (reflow->model),
-			       reflow->model_item_changed_id);
-	gtk_object_unref (GTK_OBJECT (reflow->model));
+	g_signal_handler_disconnect (reflow->model,
+				     reflow->model_changed_id);
+	g_signal_handler_disconnect (reflow->model,
+				     reflow->model_items_inserted_id);
+	g_signal_handler_disconnect (reflow->model,
+				     reflow->model_item_changed_id);
+	g_object_unref (reflow->model);
 
 	reflow->model_changed_id        = 0;
 	reflow->model_items_inserted_id = 0;
@@ -462,13 +461,13 @@ disconnect_selection (EReflow *reflow)
 	if (reflow->selection == NULL)
 		return;
 
-	gtk_signal_disconnect (GTK_OBJECT (reflow->selection),
-			       reflow->selection_changed_id);
-	gtk_signal_disconnect (GTK_OBJECT (reflow->selection),
-			       reflow->selection_row_changed_id);
-	gtk_signal_disconnect (GTK_OBJECT (reflow->selection),
-			       reflow->cursor_changed_id);
-	gtk_object_unref (GTK_OBJECT (reflow->selection));
+	g_signal_handler_disconnect (reflow->selection,
+				     reflow->selection_changed_id);
+	g_signal_handler_disconnect (reflow->selection,
+				     reflow->selection_row_changed_id);
+	g_signal_handler_disconnect (reflow->selection,
+				     reflow->cursor_changed_id);
+	g_object_unref (reflow->selection);
 
 	reflow->selection_changed_id = 0;
 	reflow->selection_row_changed_id = 0;
@@ -486,16 +485,16 @@ connect_model (EReflow *reflow, EReflowModel *model)
 		return;
 
 	reflow->model = model;
-	gtk_object_ref (GTK_OBJECT (reflow->model));
+	g_object_ref (reflow->model);
 	reflow->model_changed_id =
-		gtk_signal_connect (GTK_OBJECT (reflow->model), "model_changed",
-				    GTK_SIGNAL_FUNC (model_changed), reflow);
+		g_signal_connect (reflow->model, "model_changed",
+				  G_CALLBACK (model_changed), reflow);
 	reflow->model_items_inserted_id =
-		gtk_signal_connect (GTK_OBJECT (reflow->model), "model_items_inserted",
-				    GTK_SIGNAL_FUNC (items_inserted), reflow);
+		g_signal_connect (reflow->model, "model_items_inserted",
+				  G_CALLBACK (items_inserted), reflow);
 	reflow->model_item_changed_id =
-		gtk_signal_connect (GTK_OBJECT (reflow->model), "model_item_changed",
-				    GTK_SIGNAL_FUNC (item_changed), reflow);
+		g_signal_connect (reflow->model, "model_item_changed",
+				  G_CALLBACK (item_changed), reflow);
 	model_changed (model, reflow);
 }
 
@@ -511,12 +510,12 @@ disconnect_adjustment (EReflow *reflow)
 	if (reflow->adjustment == NULL)
 		return;
 
-	gtk_signal_disconnect (GTK_OBJECT (reflow->adjustment),
-			       reflow->adjustment_changed_id);
-	gtk_signal_disconnect (GTK_OBJECT (reflow->adjustment),
-			       reflow->adjustment_value_changed_id);
+	g_signal_handler_disconnect (reflow->adjustment,
+				     reflow->adjustment_changed_id);
+	g_signal_handler_disconnect (reflow->adjustment,
+				     reflow->adjustment_value_changed_id);
 
-	gtk_object_unref (GTK_OBJECT (reflow->adjustment));
+	g_object_unref (reflow->adjustment);
 
 	reflow->adjustment_changed_id = 0;
 	reflow->adjustment_value_changed_id = 0;
@@ -534,12 +533,12 @@ connect_adjustment (EReflow *reflow, GtkAdjustment *adjustment)
 
 	reflow->adjustment = adjustment;
 	reflow->adjustment_changed_id =
-		gtk_signal_connect (GTK_OBJECT (adjustment), "changed",
-				    adjustment_changed, reflow);
+		g_signal_connect (adjustment, "changed",
+				  G_CALLBACK (adjustment_changed), reflow);
 	reflow->adjustment_value_changed_id =
-		gtk_signal_connect (GTK_OBJECT (adjustment), "value_changed",
-				    adjustment_changed, reflow);
-	gtk_object_ref (GTK_OBJECT (adjustment));
+		g_signal_connect (adjustment, "value_changed",
+				  G_CALLBACK (adjustment_changed), reflow);
+	g_object_ref (adjustment);
 }
 
 #if 0
@@ -553,9 +552,9 @@ static void
 connect_set_adjustment (EReflow *reflow)
 {
 	reflow->set_scroll_adjustments_id =
-		gtk_signal_connect (GTK_OBJECT (GNOME_CANVAS_ITEM (reflow)->canvas),
-				    "set_scroll_adjustments",
-				    GTK_SIGNAL_FUNC (set_scroll_adjustments), reflow);
+		g_signal_connect (GNOME_CANVAS_ITEM (reflow)->canvas,
+				  "set_scroll_adjustments",
+				  G_CALLBACK (set_scroll_adjustments), reflow);
 }
 #endif
 
@@ -563,51 +562,58 @@ static void
 disconnect_set_adjustment (EReflow *reflow)
 {
 	if (reflow->set_scroll_adjustments_id != 0) {
-		gtk_signal_disconnect (GTK_OBJECT (GNOME_CANVAS_ITEM (reflow)->canvas),
-				       reflow->set_scroll_adjustments_id);
+		g_signal_handler_disconnect (GNOME_CANVAS_ITEM (reflow)->canvas,
+					     reflow->set_scroll_adjustments_id);
 		reflow->set_scroll_adjustments_id = 0;
 	}
 }
 
+static void
+column_width_changed (EReflow *reflow)
+{
+	g_signal_emit (reflow, signals[COLUMN_WIDTH_CHANGED], 0, reflow->column_width);
+}
+
 
 
 
 /* Virtual functions */
 static void
-e_reflow_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+e_reflow_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	GnomeCanvasItem *item;
 	EReflow *reflow;
 
-	item = GNOME_CANVAS_ITEM (o);
-	reflow = E_REFLOW (o);
+	item = GNOME_CANVAS_ITEM (object);
+	reflow = E_REFLOW (object);
 	
-	switch (arg_id){
-	case ARG_HEIGHT:
-		reflow->height = GTK_VALUE_DOUBLE (*arg);
+	switch (prop_id){
+	case PROP_HEIGHT:
+		reflow->height = g_value_get_double (value);
 		reflow->need_reflow_columns = TRUE;
 		e_canvas_item_request_reflow(item);
 		break;
-	case ARG_MINIMUM_WIDTH:
-		reflow->minimum_width = GTK_VALUE_DOUBLE (*arg);
-		if (GNOME_CANVAS_ITEM_REALIZED & GTK_OBJECT_FLAGS(o))
+	case PROP_MINIMUM_WIDTH:
+		reflow->minimum_width = g_value_get_double (value);
+		if (GNOME_CANVAS_ITEM_REALIZED & GTK_OBJECT_FLAGS(object))
 			set_empty(reflow);
 		e_canvas_item_request_reflow(item);
 		break;
-	case ARG_EMPTY_MESSAGE:
+	case PROP_EMPTY_MESSAGE:
 		g_free(reflow->empty_message);
-		reflow->empty_message = g_strdup(GTK_VALUE_STRING (*arg));
-		if (GNOME_CANVAS_ITEM_REALIZED & GTK_OBJECT_FLAGS(o))
+		reflow->empty_message = g_strdup(g_value_get_string (value));
+		if (GNOME_CANVAS_ITEM_REALIZED & GTK_OBJECT_FLAGS(object))
 			set_empty(reflow);
 		break;
-	case ARG_MODEL:
-		connect_model (reflow, (EReflowModel *) GTK_VALUE_OBJECT (*arg));
+	case PROP_MODEL:
+		connect_model (reflow, (EReflowModel *) g_value_get_object (value));
 		break;
-	case ARG_COLUMN_WIDTH:
-		if (reflow->column_width != GTK_VALUE_INT (*arg)) {
+	case PROP_COLUMN_WIDTH:
+		if (reflow->column_width != g_value_get_double (value)) {
 			GtkAdjustment *adjustment = gtk_layout_get_hadjustment(GTK_LAYOUT(item->canvas));
+			double old_width = reflow->column_width;
 
-			reflow->column_width = GTK_VALUE_INT (*arg);
+			reflow->column_width = g_value_get_double (value);
 			adjustment->step_increment = (reflow->column_width + E_REFLOW_FULL_GUTTER) / 2;
 			adjustment->page_increment = adjustment->page_size - adjustment->step_increment;
 			gtk_adjustment_changed(adjustment);
@@ -616,45 +622,48 @@ e_reflow_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 
 			reflow->need_column_resize = TRUE;
 			gnome_canvas_item_request_update(item);
+
+			if (old_width != reflow->column_width)
+				column_width_changed (reflow);
 		}
 		break;
 	}
 }
 
 static void
-e_reflow_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+e_reflow_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	EReflow *reflow;
 
 	reflow = E_REFLOW (object);
 
-	switch (arg_id) {
-	case ARG_MINIMUM_WIDTH:
-		GTK_VALUE_DOUBLE (*arg) = reflow->minimum_width;
+	switch (prop_id) {
+	case PROP_MINIMUM_WIDTH:
+		g_value_set_double (value, reflow->minimum_width);
 		break;
-	case ARG_WIDTH:
-		GTK_VALUE_DOUBLE (*arg) = reflow->width;
+	case PROP_WIDTH:
+		g_value_set_double (value, reflow->width);
 		break;
-	case ARG_HEIGHT:
-		GTK_VALUE_DOUBLE (*arg) = reflow->height;
+	case PROP_HEIGHT:
+		g_value_set_double (value, reflow->height);
 		break;
-	case ARG_EMPTY_MESSAGE:
-		GTK_VALUE_STRING (*arg) = g_strdup(reflow->empty_message);
+	case PROP_EMPTY_MESSAGE:
+		g_value_set_string (value, g_strdup(reflow->empty_message));
 		break;
-	case ARG_MODEL:
-		GTK_VALUE_OBJECT (*arg) = (GtkObject *) reflow->model;
+	case PROP_MODEL:
+		g_value_set_object (value, reflow->model);
 		break;
-	case ARG_COLUMN_WIDTH:
-		GTK_VALUE_INT (*arg) = reflow->column_width;
+	case PROP_COLUMN_WIDTH:
+		g_value_set_double (value, reflow->column_width);
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
 static void
-e_reflow_destroy (GtkObject *object)
+e_reflow_dispose (GObject *object)
 {
 	EReflow *reflow = E_REFLOW(object);
 
@@ -668,15 +677,17 @@ e_reflow_destroy (GtkObject *object)
 	reflow->count          = 0;
 	reflow->allocated_count = 0;
 
-	if (reflow->incarnate_idle_id != 0)
+	if (reflow->incarnate_idle_id)
 		g_source_remove (reflow->incarnate_idle_id);
+	reflow->incarnate_idle_id = 0;
 
 	disconnect_model (reflow);
 	disconnect_selection (reflow);
 	
 	g_free(reflow->empty_message);
+	reflow->empty_message = NULL;
   
-	GTK_OBJECT_CLASS(parent_class)->destroy (object);
+	G_OBJECT_CLASS(parent_class)->dispose (object);
 }
 
 static void
@@ -701,7 +712,7 @@ e_reflow_realize (GnomeCanvasItem *item)
 	for(i = 0; i < count; i++) {
 		if (reflow->items[i])
 			gnome_canvas_item_set(reflow->items[i],
-					      "width", (double) reflow->column_width,
+					      "width", reflow->column_width,
 					      NULL);
 	}
 
@@ -775,9 +786,9 @@ e_reflow_event (GnomeCanvasItem *item, GdkEvent *event)
 					GnomeCanvasItem *item = reflow->items[unsorted];
 					EFocus has_focus;
 					if (item) {
-						gtk_object_get(GTK_OBJECT(item),
-							       "has_focus", &has_focus,
-							       NULL);
+						g_object_get(item,
+							     "has_focus", &has_focus,
+							     NULL);
 						if (has_focus) {
 							if (event->key.state & GDK_SHIFT_MASK) {
 								if (i == 0)
@@ -874,6 +885,7 @@ e_reflow_event (GnomeCanvasItem *item, GdkEvent *event)
 					gtk_adjustment_changed(adjustment);
 					e_reflow_resize_children(item);
 					e_canvas_item_request_reflow(item);
+					column_width_changed (reflow);
 				}
 				reflow->need_column_resize = TRUE;
 				gnome_canvas_item_request_update(item);
@@ -1208,7 +1220,13 @@ e_reflow_selection_event_real (EReflow *reflow, GnomeCanvasItem *item, GdkEvent
 		case 1: /* Fall through. */
 		case 2:
 			row = er_find_item (reflow, item);
-			e_selection_model_do_something(reflow->selection, row, 0, event->button.state);
+			if (event->button.button == 1) {
+				reflow->maybe_did_something = 
+					e_selection_model_maybe_do_something(reflow->selection, row, 0, event->button.state);
+				reflow->maybe_in_drag = TRUE;
+			} else {
+				e_selection_model_do_something(reflow->selection, row, 0, event->button.state);
+			}
 			break;
 		case 3:
 			row = er_find_item (reflow, item);
@@ -1219,6 +1237,17 @@ e_reflow_selection_event_real (EReflow *reflow, GnomeCanvasItem *item, GdkEvent
 			break;
 		}
 		break;
+	case GDK_BUTTON_RELEASE:
+		if (event->button.button == 1) {
+			if (reflow->maybe_in_drag) {
+				reflow->maybe_in_drag = FALSE;
+				if (!reflow->maybe_did_something) {
+					row = er_find_item (reflow, item);
+					e_selection_model_do_something(reflow->selection, row, 0, event->button.state);
+				}
+			}
+		}
+		break;
 	case GDK_KEY_PRESS:
 		return_val = e_selection_model_key_press(reflow->selection, (GdkEventKey *) event);
 		break;
@@ -1233,40 +1262,17 @@ e_reflow_selection_event_real (EReflow *reflow, GnomeCanvasItem *item, GdkEvent
 static void
 e_reflow_class_init (EReflowClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GnomeCanvasItemClass *item_class;
 
-	object_class = (GtkObjectClass*) klass;
+	object_class = (GObjectClass*) klass;
 	item_class = (GnomeCanvasItemClass *) klass;
 
-	parent_class = gtk_type_class (gnome_canvas_group_get_type ());
-  
-	gtk_object_add_arg_type ("EReflow::minimum_width", GTK_TYPE_DOUBLE,
-				 GTK_ARG_READWRITE, ARG_MINIMUM_WIDTH);
-	gtk_object_add_arg_type ("EReflow::width", GTK_TYPE_DOUBLE,
-				 GTK_ARG_READABLE, ARG_WIDTH);
-	gtk_object_add_arg_type ("EReflow::height", GTK_TYPE_DOUBLE,
-				 GTK_ARG_READWRITE, ARG_HEIGHT);
-	gtk_object_add_arg_type ("EReflow::empty_message", GTK_TYPE_STRING,
-				 GTK_ARG_READWRITE, ARG_EMPTY_MESSAGE);
-	gtk_object_add_arg_type ("EReflow::model", E_REFLOW_MODEL_TYPE,
-				 GTK_ARG_READWRITE, ARG_MODEL);
-	gtk_object_add_arg_type ("EReflow::column_width", GTK_TYPE_INT,
-				 GTK_ARG_READWRITE, ARG_COLUMN_WIDTH);
+	parent_class = g_type_class_ref (PARENT_TYPE);
 
-	signals [SELECTION_EVENT] =
-		gtk_signal_new ("selection_event",
-				GTK_RUN_LAST,
-				E_OBJECT_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (EReflowClass, selection_event),
-				e_marshal_INT__OBJECT_POINTER,
-				GTK_TYPE_INT, 2, GTK_TYPE_OBJECT, GTK_TYPE_GDK_EVENT);
-
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, signals, LAST_SIGNAL);
-
-	object_class->set_arg  = e_reflow_set_arg;
-	object_class->get_arg  = e_reflow_get_arg;
-	object_class->destroy  = e_reflow_destroy;
+	object_class->set_property  = e_reflow_set_property;
+	object_class->get_property  = e_reflow_get_property;
+	object_class->dispose  = e_reflow_dispose;
   
 	/* GnomeCanvasItem method overrides */
 	item_class->event      = e_reflow_event;
@@ -1277,6 +1283,69 @@ e_reflow_class_init (EReflowClass *klass)
 	item_class->point      = e_reflow_point;
 
 	klass->selection_event = e_reflow_selection_event_real;
+	klass->column_width_changed = NULL;
+
+	g_object_class_install_property (object_class, PROP_MINIMUM_WIDTH,
+					 g_param_spec_double ("minimum_width",
+							      _( "Minimum width" ),
+							      _( "Minimum Width" ),
+							      0.0, G_MAXDOUBLE, 0.0,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_WIDTH,
+					 g_param_spec_double ("width",
+							      _( "Width" ),
+							      _( "Width" ),
+							      0.0, G_MAXDOUBLE, 0.0,
+							      G_PARAM_READABLE));
+
+
+	g_object_class_install_property (object_class, PROP_HEIGHT,
+					 g_param_spec_double ("height",
+							      _( "Height" ),
+							      _( "Height" ),
+							      0.0, G_MAXDOUBLE, 0.0,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_EMPTY_MESSAGE,
+					 g_param_spec_string ("empty_message",
+							      _( "Empty message" ),
+							      _( "Empty message" ),
+							      NULL,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_MODEL,
+					 g_param_spec_object ("model",
+							      _( "Reflow model" ),
+							      _( "Reflow model" ),
+							      E_REFLOW_MODEL_TYPE,
+							      G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class, PROP_COLUMN_WIDTH,
+					 g_param_spec_double ("column_width",
+							      _( "Column width" ),
+							      _( "Column width" ),
+							      0.0, G_MAXDOUBLE, 150.0,
+							      G_PARAM_READWRITE));
+
+	signals [SELECTION_EVENT] =
+		g_signal_new ("selection_event",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EReflowClass, selection_event),
+			      NULL, NULL,
+			      e_marshal_INT__OBJECT_BOXED,
+			      G_TYPE_INT, 2, G_TYPE_OBJECT,
+			      GDK_TYPE_EVENT);
+
+	signals [COLUMN_WIDTH_CHANGED] =
+		g_signal_new ("column_width_changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EReflowClass, column_width_changed),
+			      NULL, NULL,
+			      e_marshal_NONE__DOUBLE,
+			      G_TYPE_NONE, 1, G_TYPE_DOUBLE);
 }
 
 static void
@@ -1303,6 +1372,10 @@ e_reflow_init (EReflow *reflow)
 
 	reflow->need_height_update        = FALSE;
 	reflow->need_column_resize        = FALSE;
+	reflow->need_reflow_columns       = FALSE;
+
+	reflow->maybe_did_something       = FALSE;
+	reflow->maybe_in_drag             = FALSE;
 
 	reflow->default_cursor_shown      = TRUE;
 	reflow->arrow_cursor              = NULL;
@@ -1316,42 +1389,26 @@ e_reflow_init (EReflow *reflow)
 	reflow->selection                 = E_SELECTION_MODEL (e_selection_model_simple_new());
 	reflow->sorter                    = e_sorter_array_new (er_compare, reflow);
 
-	gtk_object_set (GTK_OBJECT (reflow->selection),
-			"sorter", reflow->sorter,
-			NULL);
+	g_object_set (reflow->selection,
+		      "sorter", reflow->sorter,
+		      NULL);
 
 	reflow->selection_changed_id = 
-		gtk_signal_connect(GTK_OBJECT(reflow->selection), "selection_changed",
-				   GTK_SIGNAL_FUNC(selection_changed), reflow);
+		g_signal_connect(reflow->selection, "selection_changed",
+				 G_CALLBACK (selection_changed), reflow);
 	reflow->selection_row_changed_id = 
-		gtk_signal_connect(GTK_OBJECT(reflow->selection), "selection_row_changed",
-				   GTK_SIGNAL_FUNC(selection_row_changed), reflow);
+		g_signal_connect(reflow->selection, "selection_row_changed",
+				 G_CALLBACK (selection_row_changed), reflow);
 	reflow->cursor_changed_id = 
-		gtk_signal_connect(GTK_OBJECT(reflow->selection), "cursor_changed",
-				   GTK_SIGNAL_FUNC(cursor_changed), reflow);
+		g_signal_connect(reflow->selection, "cursor_changed",
+				 G_CALLBACK (cursor_changed), reflow);
 
 	e_canvas_item_set_reflow_callback(GNOME_CANVAS_ITEM(reflow), e_reflow_reflow);
 }
 
-GtkType
-e_reflow_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type) {
-		static const GtkTypeInfo info = {
-			"EReflow",
-			sizeof (EReflow),
-			sizeof (EReflowClass),
-			(GtkClassInitFunc) e_reflow_class_init,
-			(GtkObjectInitFunc) e_reflow_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		type = gtk_type_unique (gnome_canvas_group_get_type (), &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE (e_reflow,
+	     "EReflow",
+	     EReflow,
+	     e_reflow_class_init,
+	     e_reflow_init,
+	     PARENT_TYPE)
