@@ -1,28 +1,39 @@
-#ifndef _E_UNICODE_H_
-#define _E_UNICODE_H_
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * UTF-8 support functions for gal
+ * e-unicode.h - utf-8 support functions for gal
+ * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
- *   Lauris Kaplinski <lauris@helixcode.com>
+ *   Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2000-2001 Helix Code, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
+#ifndef _E_UNICODE_H_
+#define _E_UNICODE_H_
+
 #include <sys/types.h>
 #include <glib.h>
 #include <gtk/gtkclist.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkwidget.h>
-#include <libgnome/gnome-defs.h>
-#include <gnome-xml/tree.h>
-#include <gal/unicode/gunicode.h>
+#include <libxml/tree.h>
 #include <iconv.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define G_UTF8_IN_GAL
 
@@ -115,7 +126,7 @@ guint32      gdk_keyval_to_unicode                          (guint          keys
 gchar       *e_xml_get_translated_utf8_string_prop_by_name  (const xmlNode *parent,
 							     const xmlChar *prop_name);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
 
