@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * gtk-combo-box.c - a customizable combobox
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -39,7 +39,9 @@
 #include "gtk-combo-box.h"
 #include "gal/util/e-util.h"
 
-static GtkHBoxClass *gtk_combo_box_parent_class;
+#define PARENT_TYPE GTK_TYPE_HBOX
+static GObjectClass *gtk_combo_box_parent_class;
+
 static int gtk_combo_toggle_pressed (GtkToggleButton *tbutton,
 				     GtkComboBox *combo_box);
 static void gtk_combo_popup_tear_off (GtkComboBox *combo,
@@ -86,78 +88,81 @@ struct _GtkComboBoxPrivate {
 };
 
 static void
-gtk_combo_box_finalize (GtkObject *object)
+gtk_combo_box_finalize (GObject *object)
 {
 	GtkComboBox *combo_box = GTK_COMBO_BOX (object);
 
-	gtk_object_destroy (GTK_OBJECT (combo_box->priv->toplevel));
-	gtk_object_unref (GTK_OBJECT (combo_box->priv->toplevel));
+	g_free (combo_box->priv);
+
+	gtk_combo_box_parent_class->finalize (object);
+}
+
+static void
+gtk_combo_box_destroy (GtkObject *object)
+{
+	GtkObjectClass *klass = (GtkObjectClass *)gtk_combo_box_parent_class;
+	GtkComboBox *combo_box = GTK_COMBO_BOX (object);
+
+	if (combo_box->priv->toplevel) {
+		gtk_object_destroy (GTK_OBJECT (combo_box->priv->toplevel));
+		g_object_unref (combo_box->priv->toplevel);
+		combo_box->priv->toplevel = NULL;
+	}
+
 	if (combo_box->priv->tearoff_window) {
 		gtk_object_destroy
 			(GTK_OBJECT (combo_box->priv->tearoff_window));
-		gtk_object_unref
-			(GTK_OBJECT (combo_box->priv->tearoff_window)); /* ?? */
+		g_object_unref
+			(combo_box->priv->tearoff_window); /* ?? */
 	}
-	g_free (combo_box->priv);
 
-	GTK_OBJECT_CLASS (gtk_combo_box_parent_class)->finalize (object);
-}
-
-typedef GtkObject * (*GtkSignal_POINTER__NONE) (GtkObject * object,
-						gpointer user_data);
-static void
-my_marshal_POINTER__NONE (GtkObject * object,
-			  GtkSignalFunc func,
-			  gpointer func_data,
-			  GtkArg * args)
-{
-	GtkSignal_POINTER__NONE rfunc;
-	GtkObject **return_val;
-	return_val = GTK_RETLOC_OBJECT (args[0]);
-	rfunc = (GtkSignal_POINTER__NONE) func;
-	*return_val = (*rfunc) (object, func_data);
+	if (klass->destroy)
+                klass->destroy (object);
 }
 
 static void
-gtk_combo_box_class_init (GtkObjectClass *object_class)
+gtk_combo_box_class_init (GObjectClass *object_class)
 {
-	gtk_combo_box_parent_class = gtk_type_class (gtk_hbox_get_type ());
+	gtk_combo_box_parent_class = g_type_class_peek_parent (object_class);
 
 	object_class->finalize = gtk_combo_box_finalize;
+	((GtkObjectClass *)object_class)->destroy = gtk_combo_box_destroy;
 
-	gtk_combo_box_signals [POP_DOWN_WIDGET] = gtk_signal_new (
+	gtk_combo_box_signals [POP_DOWN_WIDGET] = g_signal_new (
 		"pop_down_widget",
-		GTK_RUN_LAST,
-		E_OBJECT_CLASS_TYPE (object_class),
-		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pop_down_widget),
-		my_marshal_POINTER__NONE,
-		GTK_TYPE_POINTER, 0, GTK_TYPE_NONE);
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GtkComboBoxClass, pop_down_widget),
+		NULL, NULL,
+		e_marshal_POINTER__NONE,
+		G_TYPE_POINTER, 0, G_TYPE_NONE);
 
-	gtk_combo_box_signals [POP_DOWN_DONE] = gtk_signal_new (
+	gtk_combo_box_signals [POP_DOWN_DONE] = g_signal_new (
 		"pop_down_done",
-		GTK_RUN_LAST,
-		E_OBJECT_CLASS_TYPE (object_class),
-		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pop_down_done),
-		gtk_marshal_BOOL__POINTER,
-		GTK_TYPE_BOOL, 1, GTK_TYPE_OBJECT);
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GtkComboBoxClass, pop_down_done),
+		NULL, NULL,
+		e_marshal_BOOLEAN__OBJECT,
+		G_TYPE_BOOLEAN, 1, G_TYPE_OBJECT);
 
-	gtk_combo_box_signals [PRE_POP_DOWN] = gtk_signal_new (
+	gtk_combo_box_signals [PRE_POP_DOWN] = g_signal_new (
 		"pre_pop_down",
-		GTK_RUN_LAST,
-		E_OBJECT_CLASS_TYPE (object_class),
-		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pre_pop_down),
-		gtk_marshal_NONE__NONE,
-		GTK_TYPE_NONE, 0);
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GtkComboBoxClass, pre_pop_down),
+		NULL, NULL,
+		e_marshal_NONE__NONE,
+		G_TYPE_NONE, 0);
 
-	gtk_combo_box_signals [POST_POP_HIDE] = gtk_signal_new (
+	gtk_combo_box_signals [POST_POP_HIDE] = g_signal_new (
 		"post_pop_hide",
-		GTK_RUN_LAST,
-		E_OBJECT_CLASS_TYPE (object_class),
-		GTK_SIGNAL_OFFSET (GtkComboBoxClass, post_pop_hide),
-		gtk_marshal_NONE__NONE,
-		GTK_TYPE_NONE, 0);
-	
-	E_OBJECT_CLASS_ADD_SIGNALS (object_class, gtk_combo_box_signals, LAST_SIGNAL);
+		G_OBJECT_CLASS_TYPE (object_class),
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (GtkComboBoxClass, post_pop_hide),
+		NULL, NULL,
+		e_marshal_NONE__NONE,
+		G_TYPE_NONE, 0);
 }
 
 static void
@@ -166,15 +171,17 @@ deactivate_arrow (GtkComboBox *combo_box)
 	GtkToggleButton *arrow;
 
 	arrow = GTK_TOGGLE_BUTTON (combo_box->priv->arrow_button);
-	gtk_signal_handler_block_by_func
-		(GTK_OBJECT (arrow),
-		 GTK_SIGNAL_FUNC (gtk_combo_toggle_pressed), combo_box);
+	g_signal_handlers_block_matched (arrow,
+					 G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+					 0, 0, NULL,
+					 gtk_combo_toggle_pressed, combo_box);
 
 	gtk_toggle_button_set_active (arrow, FALSE);
 	
-       	gtk_signal_handler_unblock_by_func
-		(GTK_OBJECT (arrow),
-		 GTK_SIGNAL_FUNC (gtk_combo_toggle_pressed), combo_box);
+       	g_signal_handlers_unblock_matched (arrow,
+					   G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+					   0, 0, NULL,
+					   gtk_combo_toggle_pressed, combo_box);
 }
 
 /**
@@ -202,10 +209,10 @@ gtk_combo_box_popup_hide_unconditional (GtkComboBox *combo_box)
 	gtk_grab_remove (combo_box->priv->toplevel);
 	gdk_pointer_ungrab (GDK_CURRENT_TIME);
 		
-	gtk_object_ref (GTK_OBJECT (combo_box->priv->pop_down_widget));
-	gtk_signal_emit (GTK_OBJECT (combo_box),
-			 gtk_combo_box_signals [POP_DOWN_DONE],
-			 combo_box->priv->pop_down_widget, &popup_info_destroyed);
+	g_object_ref (combo_box->priv->pop_down_widget);
+	g_signal_emit (combo_box,
+		       gtk_combo_box_signals [POP_DOWN_DONE], 0,
+		       combo_box->priv->pop_down_widget, &popup_info_destroyed);
 
 	if (popup_info_destroyed){
 		gtk_container_remove (
@@ -213,10 +220,10 @@ gtk_combo_box_popup_hide_unconditional (GtkComboBox *combo_box)
 			combo_box->priv->pop_down_widget);
 		combo_box->priv->pop_down_widget = NULL;
 	}
-	gtk_object_unref (GTK_OBJECT (combo_box->priv->pop_down_widget));
+	g_object_unref (combo_box->priv->pop_down_widget);
 	deactivate_arrow (combo_box);
 
-	gtk_signal_emit (GTK_OBJECT (combo_box), gtk_combo_box_signals [POST_POP_HIDE]);
+	g_signal_emit (combo_box, gtk_combo_box_signals [POST_POP_HIDE], 0);
 }
 
 /**
@@ -278,14 +285,14 @@ gtk_combo_box_popup_display (GtkComboBox *combo_box)
 	if (!combo_box->priv->pop_down_widget){
 		GtkWidget *pw = NULL;
 
-		gtk_signal_emit (GTK_OBJECT (combo_box),
-				 gtk_combo_box_signals [POP_DOWN_WIDGET], &pw);
+		g_signal_emit (combo_box,
+			       gtk_combo_box_signals [POP_DOWN_WIDGET], 0, &pw);
 		g_assert (pw != NULL);
 		combo_box->priv->pop_down_widget = pw;
 		gtk_container_add (GTK_CONTAINER (combo_box->priv->frame), pw);
 	}
 
-	gtk_signal_emit (GTK_OBJECT (combo_box), gtk_combo_box_signals [PRE_POP_DOWN]);
+	g_signal_emit (combo_box, gtk_combo_box_signals [PRE_POP_DOWN], 0);
 	
 	if (combo_box->priv->torn_off) {
 		/* To give the illusion that tearoff still displays the
@@ -385,22 +392,21 @@ gtk_combo_box_init (GtkComboBox *combo_box)
 	 * Create the arrow
 	 */
 	combo_box->priv->arrow_button = gtk_toggle_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (combo_box->priv->arrow_button), GTK_RELIEF_NONE);
 	GTK_WIDGET_UNSET_FLAGS (combo_box->priv->arrow_button, GTK_CAN_FOCUS);
 
 	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (combo_box->priv->arrow_button), arrow);
 	gtk_box_pack_end (GTK_BOX (combo_box), combo_box->priv->arrow_button, FALSE, FALSE, 0);
-	gtk_signal_connect (
-		GTK_OBJECT (combo_box->priv->arrow_button), "toggled",
-		GTK_SIGNAL_FUNC (gtk_combo_toggle_pressed), combo_box);
+	g_signal_connect (combo_box->priv->arrow_button, "toggled",
+			  G_CALLBACK (gtk_combo_toggle_pressed), combo_box);
 	gtk_widget_show_all (combo_box->priv->arrow_button);
 
 	/*
 	 * prelight the display widget when mousing over the arrow.
 	 */
-	gtk_signal_connect (
-		GTK_OBJECT (combo_box->priv->arrow_button), "state-changed",
-		GTK_SIGNAL_FUNC (cb_state_change), combo_box);
+	g_signal_connect (combo_box->priv->arrow_button, "state-changed",
+			  G_CALLBACK (cb_state_change), combo_box);
 
 	/*
 	 * The pop-down container
@@ -430,36 +436,18 @@ gtk_combo_box_init (GtkComboBox *combo_box)
 			   combo_box->priv->frame);
 	gtk_frame_set_shadow_type (GTK_FRAME (combo_box->priv->frame), GTK_SHADOW_OUT);
 
-	gtk_signal_connect (
-		GTK_OBJECT (combo_box->priv->toplevel), "button_press_event",
-		GTK_SIGNAL_FUNC (gtk_combo_box_button_press), combo_box);
-	gtk_signal_connect (
-		GTK_OBJECT (combo_box->priv->toplevel), "key_press_event",
-		GTK_SIGNAL_FUNC (gtk_combo_box_key_press), combo_box);
+	g_signal_connect (combo_box->priv->toplevel, "button_press_event",
+			  G_CALLBACK (gtk_combo_box_button_press), combo_box);
+	g_signal_connect (combo_box->priv->toplevel, "key_press_event",
+			  G_CALLBACK (gtk_combo_box_key_press), combo_box);
 }
 
-GtkType
-gtk_combo_box_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type){
-		GtkTypeInfo info = {
-			"MyGtkComboBox",
-			sizeof (GtkComboBox),
-			sizeof (GtkComboBoxClass),
-			(GtkClassInitFunc) gtk_combo_box_class_init,
-			(GtkObjectInitFunc) gtk_combo_box_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (gtk_hbox_get_type (), &info);
-	}
-
-	return type;
-}
+E_MAKE_TYPE (gtk_combo_box,
+	     "MyGtkComboBox",
+	     GtkComboBox,
+	     gtk_combo_box_class_init,
+	     gtk_combo_box_init,
+	     PARENT_TYPE)
 
 /**
  * gtk_combo_box_set_display:
@@ -516,17 +504,18 @@ gtk_combo_popup_tear_off (GtkComboBox *combo, gboolean set_position)
 		GtkWidget *tearoff;
 		gchar *title;
 		
-		tearoff = gtk_window_new (GTK_WINDOW_DIALOG);
+		/* FIXME: made this a toplevel, not a dialog ! */
+		tearoff = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 		gtk_widget_ref (tearoff);
 		gtk_object_sink (GTK_OBJECT (tearoff));
 		combo->priv->tearoff_window = tearoff;
 		gtk_widget_set_app_paintable (tearoff, TRUE);
-		gtk_signal_connect (GTK_OBJECT (tearoff), "key_press_event",
-				    GTK_SIGNAL_FUNC (gtk_combo_box_key_press),
-				    GTK_OBJECT (combo));
+		g_signal_connect (tearoff, "key_press_event",
+				  G_CALLBACK (gtk_combo_box_key_press),
+				  combo);
 		gtk_widget_realize (tearoff);
-		title = gtk_object_get_data (GTK_OBJECT (combo),
-					     "gtk-combo-title");
+		title = g_object_get_data (G_OBJECT (combo),
+					   "gtk-combo-title");
 		if (title)
 			gdk_window_set_title (tearoff->window, title);
 		gtk_window_set_policy (GTK_WINDOW (tearoff),
@@ -648,14 +637,14 @@ gtk_combo_popup_reparent (GtkWidget *popup,
 	GtkObject *object = GTK_OBJECT (popup);
 	gboolean was_floating = GTK_OBJECT_FLOATING (object);
 
-	gtk_object_ref (object);
+	g_object_ref (object);
 	gtk_object_sink (object);
 
 	if (unrealize) {
-		gtk_object_ref (object);
+		g_object_ref (object);
 		gtk_container_remove (GTK_CONTAINER (popup->parent), popup);
 		gtk_container_add (GTK_CONTAINER (new_parent), popup);
-		gtk_object_unref (object);
+		g_object_unref (object);
 	}
 	else
 		gtk_widget_reparent (GTK_WIDGET (popup), new_parent);
@@ -664,7 +653,7 @@ gtk_combo_popup_reparent (GtkWidget *popup,
 	if (was_floating)
 		GTK_OBJECT_SET_FLAGS (object, GTK_FLOATING);
 	else
-		gtk_object_unref (object);
+		g_object_unref (object);
 }
 
 /**
@@ -693,12 +682,10 @@ cb_tearable_button_release (GtkWidget *w, GdkEventButton *event,
 		need_connect = (!combo->priv->tearoff_window);
 		gtk_combo_set_tearoff_state (combo, TRUE);
 		if (need_connect)
-			gtk_signal_connect
-				(GTK_OBJECT
-				 (combo->priv->tearoff_window),  
-				 "delete_event",
-				 GTK_SIGNAL_FUNC (cb_popup_delete),
-				 combo);
+			g_signal_connect (combo->priv->tearoff_window,  
+					  "delete_event",
+					  G_CALLBACK (cb_popup_delete),
+					  combo);
 	} else
 		gtk_combo_box_popup_hide_unconditional (combo);
 	
@@ -731,15 +718,15 @@ gtk_combo_box_construct (GtkComboBox *combo_box, GtkWidget *display_widget, GtkW
 
 	vbox = gtk_vbox_new (FALSE, 5);
 	tearable = gtk_tearoff_menu_item_new ();
-	gtk_signal_connect (GTK_OBJECT (tearable), "enter-notify-event",
-			    GTK_SIGNAL_FUNC (cb_tearable_enter_leave),
-			    GINT_TO_POINTER (TRUE));
-	gtk_signal_connect (GTK_OBJECT (tearable), "leave-notify-event",
-			    GTK_SIGNAL_FUNC (cb_tearable_enter_leave),
-			    GINT_TO_POINTER (FALSE));
-	gtk_signal_connect (GTK_OBJECT (tearable), "button-release-event",
-			    GTK_SIGNAL_FUNC (cb_tearable_button_release),
-			    (gpointer) combo_box);
+	g_signal_connect (tearable, "enter-notify-event",
+			  G_CALLBACK (cb_tearable_enter_leave),
+			  GINT_TO_POINTER (TRUE));
+	g_signal_connect (tearable, "leave-notify-event",
+			  G_CALLBACK (cb_tearable_enter_leave),
+			  GINT_TO_POINTER (FALSE));
+	g_signal_connect (tearable, "button-release-event",
+			  G_CALLBACK (cb_tearable_button_release),
+			  (gpointer) combo_box);
 	gtk_box_pack_start (GTK_BOX (vbox), tearable, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), pop_down_widget, TRUE, TRUE, 0);
 	combo_box->priv->tearable = tearable;
@@ -761,7 +748,7 @@ gtk_combo_box_new (GtkWidget *display_widget, GtkWidget *optional_popdown)
 	g_return_val_if_fail (display_widget  != NULL, NULL);
 	g_return_val_if_fail (GTK_IS_WIDGET (display_widget), NULL);
 
-	combo_box = gtk_type_new (gtk_combo_box_get_type ());
+	combo_box = g_object_new (GTK_COMBO_BOX_TYPE, NULL);
 	gtk_combo_box_construct (combo_box, display_widget, optional_popdown);
 	return GTK_WIDGET (combo_box);
 }
@@ -796,8 +783,8 @@ gtk_combo_box_set_title (GtkComboBox *combo,
 	g_return_if_fail (combo != NULL);
 	g_return_if_fail (GTK_IS_COMBO_BOX (combo));
 	
-	gtk_object_set_data_full (GTK_OBJECT (combo), "gtk-combo-title",
-				  g_strdup (title), (GtkDestroyNotify) g_free);
+	g_object_set_data_full (G_OBJECT (combo), "gtk-combo-title",
+				g_strdup (title), (GDestroyNotify) g_free);
 }
 
 /**
