@@ -1,3 +1,25 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * test-font-loading.c
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #define _TEST_E_FONT_C_
 
 #include <gdk/gdkx.h>
@@ -49,7 +71,7 @@ print_gdk_font_name (const GdkFont * font, const gchar * text)
 	Atom font_atom, atom;
 	Bool status;
 
-	font_atom = gdk_atom_intern ("FONT", FALSE);
+	font_atom = gdk_x11_atom_to_xatom (gdk_atom_intern ("FONT", FALSE));
 
 	g_print (text);
 
@@ -60,13 +82,14 @@ print_gdk_font_name (const GdkFont * font, const gchar * text)
 		gint num_fonts;
 		gchar **font_names;
 
-		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font), &font_structs, &font_names);
+		num_fonts = XFontsOfFontSet (GDK_FONT_XFONT (font),
+					     &font_structs, &font_names);
 
 		g_print (" fontset: %s\n",font_names[0]);
 	} else {
 		gchar * name;
 		status = XGetFontProperty (GDK_FONT_XFONT (font), font_atom, &atom);
-		name = gdk_atom_name (atom);
+		name = gdk_atom_name (gdk_x11_xatom_to_atom (atom));
 		g_print (" font: %s\n", name);
 		if (name) g_free (name);
 	}
